-- | Decision procedure for ground confluence of LV-TRSs.
module Main (
    main,
) where

import qualified GTT
import qualified RRn

import qualified Problem as P
import qualified Data.Rewriting.Rule as R
import qualified Data.Rewriting.Term as T

import System.Environment
import System.Exit
import qualified Util.Set as S
import Util.Rewriting (isLVTRS)

main :: IO ()
main = do
    f <- getArgs >>= \fs -> case fs of
        [f] -> return f
        _ -> die "Usage: gcr FILE.trs"
    p <- P.parseProblemFile f
    let trs = concatMap P.rules p
    if not (isLVTRS trs) then die "input is not an LV-TRS" else do
        let sig = S.nubOrd $ concatMap P.signature p
            -- GTTs for -∥->, ->*, *<-
            gtt = GTT.relabel (GTT.fromTRSI sig trs)
            gttr = GTT.trim (GTT.relabel (GTT.gtrancl gtt))
            gttl = GTT.ginv gttr
            mkRRn = RRn.trim . RRn.relabel
                . RRn.parallel RRn.PAny RRn.EParallel . RRn.fromGTT sig
                . GTT.trim . GTT.relabel
            -- RR2 automata for ↑ (peak) and ↓ (valley)
            rrnp = mkRRn (GTT.gcomp gttl gttr)
            rrnv = mkRRn (GTT.gcomp gttr gttl)
{-
        print ("trs", trs)
        print ("sig", sig)
        print ("gtt", gtt)
        print ("gttr", gttr)
        print ("gttl", gttl)
        print ("gttp", GTT.gcomp gttl gttr)
        print ("gttv", GTT.gcomp gttr gttl)
        print ("tgttp", GTT.trim . GTT.relabel $ GTT.gcomp gttl gttr)
        print ("tgttv", GTT.trim . GTT.relabel $ GTT.gcomp gttr gttl)
        print ("rrnp", rrnp)
        print ("rrnv", rrnv)
-}
        if RRn.subset rrnp rrnv
        then putStrLn "YES"
        else putStrLn "NO"
