-- | Formula simplification.
module Formula.Simplify (
    exForm,
) where

import Formula.Type

-- | Simplify: Convert to existential form and minimize negations.
exForm :: Formula -> Formula
exForm = go False where
    go b f@FRR1{} = if b then FNot f else f
    go b f@FRR2{} = if b then FNot f else f
    go b (FAnd fs) = (if b then fOr else fAnd) (map (go b) fs)
    go b (FOr fs) = (if b then fAnd else fOr) (map (go b) fs)
    go b (FNot f) = go (not b) f
    go b (FExists f) = (if b then FNot . FExists else FExists) (go False f)
    go b (FForall f) = (if b then FExists else FNot . FExists) (go True f)
    go b FTrue = if b then FFalse else FTrue
    go b FFalse = if b then FTrue else FFalse
    go b (FExtendSig f n) = let f' = go b f in extendSig f' n
    go _ (FRestrict _ _) = error "exForm: expected FRestrict-free formula"

    fAnd = FAnd -- tbd

    fOr = FOr -- tbd

    -- shift signature extention through negations and quantifiers
    -- to keep witness generation
    extendSig (FNot f) n = FNot (extendSig f n)
    extendSig (FExists f) n = FExists (extendSig f n)
    extendSig f n = FExtendSig f n

