-- | GTT type definition and trivial operations.
module GTT.Type (
    GTT (..),
    states,
    interface,
    map,
    merge,
) where

import qualified TA
import Prelude hiding (map)
import qualified Data.Set as S

-- | A ground tree transducer.
data GTT f q = GTT {
    leftTA :: TA.TA f q,
    rightTA :: TA.TA f q
} deriving Show

-- | Compute all states of a GTT.
states :: Ord q => GTT f q -> S.Set q
states g = TA.states (leftTA g) `S.union` TA.states (rightTA g)

-- | Compute interface states of a GTT.
interface :: Ord q => GTT f q -> S.Set q
interface g = TA.states (leftTA g) `S.intersection` TA.states (rightTA g)

-- | Map function symbols and states of a GTT.
map :: (f -> f') -> (q -> q') -> GTT f q -> GTT f' q'
map ff fq gtt = GTT {
    leftTA  = TA.map ff fq (leftTA  gtt),
    rightTA = TA.map ff fq (rightTA gtt)
}

-- | Merge two GTTs.
merge :: GTT f q -> GTT f q -> GTT f q
merge g1 g2 = GTT {
    leftTA = leftTA g1 `TA.merge` leftTA g2,
    rightTA = rightTA g1 `TA.merge` rightTA g2
}
