-- | Defines the type for many-sorted TRSs.
module Problem.Type where

import qualified Formula as Fm

import qualified Data.Rewriting.Rule as R


-- | Problem Type for many-sorted TRSs
data Problem s f v
    = Problem { sorts     :: [s] -- ^ sorts
              , symbols   :: [f] -- ^ function symbols
              , vars      :: [v] -- ^ variables
              , signature :: [(f,[s],s)] -- ^ many-sorted signature
              , rules     :: [R.Rule f v] -- ^ the TRS
              }
-- | Maps functions over the sorts, symbols and variables of a problem.
map :: (s -> s') -> (f -> f') -> (v -> v')
    -> Problem s f v -> Problem s' f' v'
map s f v p = p { sorts     = s <$> sorts p
                , symbols   = f <$> symbols p
                , vars      = v <$> vars p
                , signature = [(f g, s <$> ps, s q) | (g,ps,q) <- signature p]
                , rules     = R.map f v <$> rules p
                }
