-- | Internal /RRn/ helper function.
module RRn.Internal (
    prodAux,
) where

import Prelude as P
import qualified TA
import qualified Data.Set as S

-- | Cartesian product of two /RR1/ relations.
--
-- Used in 'RRn.Basics.prod' and 'RRn.GTT.fromGTT'.
prodAux :: (Ord p, Ord q) =>
    TA.TA f p -> TA.TA g q -> TA.TA (Maybe f, Maybe g) (Maybe p, Maybe q)
prodAux la ra = TA.TA {
    TA.finals = [],
    TA.trans = [(jn f, P.map jn ps, jn p) | (f, ps, p) <- TA.trans la] ++
        [(nj f, P.map nj qs, nj q) | (f, qs, q) <- TA.trans ra] ++
        [(jj f g, zipQs ps qs, jj p q)
        | (f, ps, p) <- TA.trans la, (g, qs, q) <- TA.trans ra],
    TA.eps = [((Just p, q), (Just p', q))
        | (p, p') <- TA.eps la, q <- Nothing : P.map Just (S.toList rs)] ++
        [((p, Just q), (p, Just q'))
        | p <- Nothing : P.map Just (S.toList ls), (q, q') <- TA.eps ra]
} where
    (ls, rs) = (TA.states la, TA.states ra)
    zipQs (p : ps) (q : qs) = jj p q : zipQs ps qs
    zipQs ps       []       = P.map jn ps
    zipQs []       qs       = P.map nj qs
    jn p = (Just p, Nothing)
    nj q = (Nothing, Just q)
    jj p q = (Just p, Just q)
