-- | Lifting root steps.
module RRn.Parallel (
    Pos_step (..),
    Ext_step (..),
    parallel,
) where

import Prelude as P
import RRn.Type
import Formula.Type (Pos_step(..), Ext_step(..))
import qualified TA
import Control.Monad
import Data.List
import qualified Util.Set as S
import Util.Signature

-- | Lift root steps by parallel or context closure.
--
-- The added context is constrained by the given signature and restrictions
-- on rewrite steps.
parallel :: Ord f => Pos_step -> Ext_step ->
    RRn s f q -> RRn s f (Either (Bool, Bool, Bool) q)
parallel pos ext rrn
    | arity rrn /= 2 = error "RRn.parallel: expect binary relation"
    | otherwise = RRn {
        arity = arity rrn,
        signature = signature rrn,
        ta = TA.map id Right ((ta rrn){ TA.finals = [] }) `TA.merge`
            TA.TA { TA.finals = finals, TA.trans = trans, TA.eps = eps }
    }
  where
    eps = [(Right q, Left (squash (True, False, True))) |
        q <- TA.finals (ta rrn)]
    trans = [([Just f, Just f], P.map Left qs, Left (combine qs))
        | (f, n) <- arities (signature rrn),
          qs <- replicateM n flags]
    combine fs = squash
        (False,
         all (\(_, empty, _) -> empty) fs,
         all (\(_, empty, single) -> empty || single) fs &&
         length (filter (\(_, _, single) -> single) fs) == 1)
    flags = nub [squash (root, empty, single)
        | root <- [False, True],
          empty <- [False, True],
          single <- [False, True],
          -- impossible combinations
          not root || (single && not empty),
          not (single && empty),
          -- optimization: filter useless flag combinations
          if ext == ESingle then empty || single else True,
          if pos == PRoot then empty || root else True,
          if pos == PRoot && ext /= EParallel then not empty else True]
    squash (root, empty, single) =
        -- optimization: squash useless flag combinations
        (if pos == PAny then False else root,
         if ext == EParallel then False else empty,
         if ext /= ESingle then False else single)
    finals = [Left f
        | f@(root, empty, single) <- flags,
          case pos of
              PRoot -> root || empty
              PNonRoot -> not root
              PAny -> True,
          case ext of
              ESingle -> single
              EParallel -> True
              EStrictParallel -> not empty]
