{-# LANGUAGE ScopedTypeVariables #-}
-- | Tree automata determinization.
module TA.Det (
    det,
    -- * Incremental determinization
    detTrans,
    detFinal,
) where

import TA.Type
import Util.Trancl
import Util.Saturate
import qualified Prelude as P
import Prelude hiding (map)
import qualified Util.Set as S
import qualified Data.IntSet as IS
import qualified Data.Map.Strict as M

-- | Determinize tree automaton.
det :: forall f. (Ord f) => TA f Int -> TA f (IS.IntSet)
det ta = TA {
    finals = filter (detFinal ta) (S.toList qss),
    trans  = S.nubOrd $ [(f, pss, transf f pss)
        | (f, ps, _) <- trans ta,
          let qss' = [[qs | qs <- S.toList qss, p `IS.member` qs] | p <- ps],
          pss <- sequence qss'],
    eps    = []
} where
    transf :: f -> [IS.IntSet] -> IS.IntSet
    transf = detTrans ta

    qss :: S.Set (IS.IntSet)
    qss = saturate h0 h1 where
        h0 :: [IS.IntSet]
        h0 = [transf f [] | (f, [], _) <- trans ta]
        h1 :: IS.IntSet -> S.Set (IS.IntSet) -> [IS.IntSet]
        h1 qs qss = [transf f pss
            | (f, ps, _) <- trans ta,
              let qss' = [[qs | qs <- S.toList qss, p `IS.member` qs] | p <- ps],
              (_, i) <- filter ((`IS.member` qs) . fst) (zip ps [0..]),
              pss <- sequence [if j == i then [qs] else qs'
                              | (qs', j) <- zip qss' [0 :: Int ..]]]

-- | The transition function underlying the deterministic TA.
detTrans :: forall f. (Ord f) => TA f Int -> f -> [IS.IntSet] -> IS.IntSet
detTrans ta = go
  where
    go :: f -> [IS.IntSet] -> IS.IntSet
    go f qss = epscl . IS.fromList $
        [q | (qs, q) <- M.findWithDefault [] (f,length qss) trans',
             all (uncurry IS.member) (zip qs qss)]

    trans' :: M.Map (f, Int) [([Int], Int)]
    trans' = M.fromListWith (++) [((f,length qs),[(qs,q)]) | (f, qs, q) <- trans ta]

    epscl1 :: Int -> [Int]
    epscl1 = rtrancl' (eps ta)

    epscl :: IS.IntSet -> IS.IntSet
    epscl = IS.fromList . concat . P.map epscl1 . IS.toList

-- | The final state test underlying the deterministic TA.
detFinal :: forall f. TA f Int -> IS.IntSet -> Bool
detFinal ta = \qs -> any (`IS.member` qs) (finals ta)
