{-# LANGUAGE FlexibleContexts #-}
-- | Rewriting related utility functions.
module Util.Rewriting (
    relabelTerm,
    relabelRule,
    isLVTRS,
) where

import qualified Data.Rewriting.Term as T
import qualified Data.Rewriting.Rule as R
import qualified Data.Map.Strict as M
import Control.Monad.State.Strict

-- | Relabel variables of a term consistently.
relabelTerm :: Ord v => T.Term f v -> T.Term f Int
relabelTerm t = evalState (T.fold var fun t) (0, M.empty)
  where
    var v = T.Var <$> do
        (i, m) <- get
        case m M.!? v of
            Nothing -> put (i+1, M.insert v i m) >> return i
            Just j -> return j
    fun f ts = T.Fun f <$> sequence ts

-- | Relabel variables of a rule consistently.
relabelRule :: Ord v => R.Rule f v -> R.Rule f Int
relabelRule R.Rule{ R.lhs = l, R.rhs = r } = R.Rule{ R.lhs = l', R.rhs = r' }
  where
    T.Fun _ [l', r'] = relabelTerm (T.Fun undefined [l, r])

-- | Check whether the given TRS is an linear and variable-separated.
isLVTRS :: Ord v => [R.Rule f v] -> Bool
isLVTRS = all (\r -> T.isLinear (T.Fun undefined [R.lhs r, R.rhs r]))
