-- | Set related utilities.
module Util.Set (
    filterFst,
    lookupFst,
    lookupFstL,
    nubOrd,
    module S,
) where

import Data.Set as S

-- | Extract subset of a set of pairs matching the given first component.
filterFst :: (Ord a, Ord b) => S.Set (a, b) -> a -> S.Set (a, b)
filterFst s a = S.takeWhileAntitone ((== a) . fst) .
    S.dropWhileAntitone ((< a) . fst) $ s

-- | Similar to `filterFst` but restrict result to the second components.
lookupFst :: (Ord a, Ord b) => S.Set (a, b) -> a -> S.Set b
lookupFst s = S.mapMonotonic snd . filterFst s

-- | Similar to `lookupFst` but return result as list.
lookupFstL :: (Ord a, Ord b) => S.Set (a, b) -> a -> [b]
lookupFstL s = S.toAscList . lookupFst s

-- | Set-based variant of `nub`.
nubOrd :: Ord a => [a] -> [a]
nubOrd = S.toAscList . S.fromList
