-- | Transitive closure computations.
module Util.Trancl (
    trancl,
    trancl',
    rtrancl,
    rtrancl',
) where

import Util.Saturate
import qualified Util.Set as S
import qualified Data.Map.Strict as M

import qualified Data.HashSet as HS

import Data.Hashable

-- | Compute transitive closure of a relation.
trancl :: (Ord a, Hashable a) => [(a, a)] -> HS.HashSet (a, a)
trancl r = saturateHash h0 h1 where
    h0 = r
    r' = M.fromListWith (++) [(x, [y]) | (x, y) <- r]
    h1 (x, y) _ = [(x, z) | z <- M.findWithDefault [] y r']

-- | Compute transitive closure of a relation.
--
-- This version computes a list of successors of a given element.
trancl' :: (Ord a, Hashable a) => [(a, a)] -> a -> [a]
trancl' r = \x -> maybe [] id (M.lookup x r') where
    r' = M.fromListWith (++) [(x, [y]) | (x, y) <- HS.toList (trancl r)]

-- | Compute reflexive transitive closure of a relation.
--
-- Note that in order to yield a finite result, only pairs `(x,y')`
-- for which some `(x,y)` is in the given relation are generated.
rtrancl :: (Ord a, Hashable a) => [(a, a)] -> HS.HashSet (a, a)
rtrancl r = HS.fromList [(x, x) | (x, _) <- r] `HS.union` trancl r

-- | Compute reflexive transitive closure of a relation.
--
-- This version computes a list of successors of a given element.
rtrancl' :: (Ord a, Hashable a) => [(a, a)] -> a -> [a]
rtrancl' r = \x -> maybe [x] id (M.lookup x r') where
    r' = M.fromListWith (++) [(x, [y]) | (x, y) <- HS.toList (rtrancl r)]
