module Main(main) where

import Pat_Test_Generated

import TA.Type
import TA.Basics (relabel)
import TA.Subset (subset)

import Control.Monad (forM_)
import Text.Printf (printf)
import Data.Time.Clock
import System.IO(hFlush,stdout)
import System.Mem(performMajorGC)
import System.Directory(removeFile)
import System.Process(callProcess)

import System.Environment

cState :: q -> q
cState = id

cSym :: f -> Integer -> f
cSym = const

cRule :: f -> [q] -> q -> (f, [q], q)
cRule f qs q = (f, qs, q)

cAut :: Ord q => [f] -> [q] -> [q] -> [(f, [q], q)] -> TA f Int
cAut funs states finalStates rules = relabel $
    TA { finals = finalStates
       , trans = rules
       , eps = []
       }

checkSubset :: (Ord f, Ord p) => TA f p -> TA f Int -> Bool
checkSubset = subset

type Perms = [([Integer], [Integer])]

runTATest :: Integer -> Integer -> Perms -> Bool
runTATest = patCompleteAutomataTest cState cSym cRule cAut checkSubset

liftTest :: (Integer -> Integer -> Perms -> Bool) -> Integer -> Integer -> Perms -> IO ()
liftTest f c n ps = let r = f c n ps in r `seq` (return ())

getPerms :: Integer -> Integer -> IO Perms
getPerms c n
  | c <= 15 = return []
  | otherwise = do
        perms <- readFile ("../permutations/perm_" ++ show n ++ ".txt")
        return $ Prelude.map read (lines perms)        

runGHCTest :: Integer -> Integer -> Perms -> IO ()
runGHCTest c n ps = do
  let hsString = createHaskellInput c n ps
  let tmpFile = "Test.hs"
  writeFile tmpFile hsString
  callProcess "ghc" ([
    "-c", 
    "-Wincomplete-patterns", 
    "-fmax-pmcheck-models=1000000"
    ] ++ [tmpFile]) 
  removeFile tmpFile
  removeFile "Test.hi"
  removeFile "Test.o"  

main = do 
  args' <- getArgs;
  let args = (if length args' >= 2 then args' else help);
  let mode = head args;
  let params = tail args
  select mode params
    
runTests :: (Integer -> Integer -> Perms -> IO ()) -> String -> Integer -> IO ()
runTests runTest mode nmax = runMain 0 1 
 where
  runMain :: Integer -> Integer -> IO ()
  runMain c n
   | n > nmax = if c == 15 then return () else runMain(c+1) 1
   | otherwise = do
   -- average on three runs of same config 
      let configs = [c,c,c]
      perms <- getPerms c n
      performMajorGC
      startTime <- getCurrentTime
      mapM_ (\ c -> runTest c n perms) configs
      endTime <- getCurrentTime
      let diffTime = diffUTCTime endTime startTime
      let time = (realToFrac diffTime :: Double) / fromIntegral (length configs)
      printf "%s\tc:\t%d\tn:\t%d\tav-time:\t%.3f\tseconds\n" mode c n time
      hFlush stdout
      runMain c ((if time <= 60 then n else nmax) +1)
      
resultsPath = "../results/"
     
select mode [n]
  | mode == "testTA" = runTests (liftTest runTATest) "FORT-h on tree automata encoding" (read n)
  | mode == "testOUR" = runTests (liftTest pat_complete_alg_test) "our decision procedure" (read n)
  | mode == "testGHC" = runTests runGHCTest "ghc pattern completeness" (read n)
select mode [c,n]
  | mode == "showTA" = do
       perms <- getPerms (read c) (read n)
       putStrLn $ showAutomata (read c) (read n) perms
  | mode == "showTRS" = do
       perms <- getPerms (read c) (read n)
       putStrLn $ show_pat_complete_test (read c) (read n) perms
  | mode == "showHS" = do
       perms <- getPerms (read c) (read n)
       putStrLn $ createHaskellInput (read c) (read n) perms
select _ _ = help 
    
  
help = error $ unlines $ [
   "invoke with one of the following list of arguments",
   "testOUR n",
   "testTA n",
   "testGHC n",
   "showTA c n",
   "showTRS c n",
   "showHS c n"]