functor MsTpdbLrValsFun(structure Token : TOKEN)
 : sig structure ParserData : PARSER_DATA
       structure Tokens : MsTpdb_TOKENS
   end
 = 
struct
structure ParserData=
struct
structure Header = 
struct
(******************************************************************************
 * Copyright (c) 2012-2013, Toyama&Aoto Laboratory, Tohoku University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(******************************************************************************
 * file: rwtools/commands/tpdb.yacc
 * description: parser for TPDB (Termination Problem Data Base) format
 * author: AOTO Takahito
 * 
 ******************************************************************************)

local 
    open Term
    structure L = List
    structure LU = ListUtil
    structure PU = PrintUtil
in
(* datatype tpdbExpr = VAR of string list  *)
(*        		  | FUN of (fun_key * sort_key) option list *)
(*        		  | RULES of (term * term) option list *)
(*  		  | THEORY of (string * fun_key list) list *)

datatype tpdb_thdecl = THEORIES of string * string list 
                     | EQUATIONS of (term * term) list

datatype tpdb_cond = ORIENT of (term * term)
                   | JOIN of (term * term)

datatype tpdb_rule = NORMAL of (term * term * tpdb_cond list)
                   | RELATIVE of (term * term * tpdb_cond list)

datatype tpdb_strat = STRAT_INNERMOST
                    | STRAT_OUTERMOST
                    | STRAT_CS of (fun_key * int list) list

datatype tpdb_decl = VAR of (var_key * sort_key) list 
		   | FUN of (fun_key * sort_key) list 
		   | THEORY of tpdb_thdecl list
                   | RULES of tpdb_rule list 
                   | STRATEGY of tpdb_strat
                   | REST of string list

exception MsTpdbError of string

fun getCategory f xss = let val yss = List.mapPartial f xss 
			in LU.mapAppend (fn x=>x) yss
			end
val getVAR = getCategory (fn top => case top of VAR xs => SOME xs | _ => NONE) 
val getFUN = getCategory (fn top => case top of FUN xs => SOME xs | _ => NONE) 
val getRULES = getCategory (fn top => case top of RULES xs => SOME xs | _ => NONE)
val getTHEORY = getCategory (fn top => case top of THEORY xs => SOME xs | _ => NONE)
val getREST = getCategory (fn top => case top of REST xs => SOME xs | _ => NONE)

(* for the stragegy, only the fist one is considerd *)
fun getSTRATEGY (top::rest) = (case top of STRATEGY xs => SOME xs | _ => getSTRATEGY rest)
  | getSTRATEGY [] = NONE
						  
fun makeTrs vars rules = 
    let val vars2 = L.map (fn (v,_) => Fun.fromString (Var.name v)) vars
       fun transTerm (t as (Fun (y,[],_))) = 
	    if ListUtil.member' Fun.equal y vars2
	    then Var (Var.fromString (Fun.toString y), Sort.null)
	    else t
	  | transTerm (Fun (y,ys,_)) = Fun (y, List.map transTerm ys, Sort.null)
    in List.map (fn (l,r) => (transTerm l, transTerm r)) rules
    end

local
    exception InValidTerm
    exception NonSupportedRule
    structure VM = VarMap
in
fun checkSortedRules funs vars rules = 
    let val vars2 = L.map (fn (v,ty) => (Fun.fromString (Var.name v), ty)) vars
        fun getNormal rs = 
            List.mapPartial (fn y => case y of 
                                  NORMAL (l,r,ex) => if null ex then SOME (l,r)  else raise NonSupportedRule
                                 | _ => raise NonSupportedRule) rs
        val rules2 = getNormal rules
        fun lookupVars name = L.find (fn (f,_) => Fun.equal (name,f)) vars2
        fun lookupFuns name = L.find (fn (f,_) => Fun.equal (name,f)) funs
        fun transTerm (t as (Fun (c,[],_))) = 
	    (case lookupFuns c of
		 SOME _ => t
               | NONE => Var (Var.fromString (Fun.toString c),Sort.null))
	  | transTerm (Fun (f,ts,_)) = Fun (f, List.map transTerm ts, Sort.null)
         fun makeDecls funs = L.map (fn (f,ty) => {sym = f,sort= ty}) funs 
         val decls = makeDecls funs
	 val vartype = List.foldr (fn ((k,ty),vmap) => VM.insert (vmap, Var.fromString (Var.name k), ty)) VM.empty  vars
         val sortedrules = Trs.attachSortToRulesWithEnv decls vartype (List.map (fn (l,r) => (transTerm l, transTerm r)) rules2)
     in case sortedrules of
            SOME rs => (decls, rs, [])
          | NONE => (PU.printStdErr "Sort attachment failed.\n"; raise (MsTpdbError "unsupported format"))
     end
    handle NonSupportedRule => 
                 (PU.printStdErr "Unsupported rules.\n"; raise (MsTpdbError "unsupported format"))
(*********
    let val vars2 = L.map (fn (v,ty) => (Fun.fromString (Var.name v), ty)) vars
        fun getNormal rs = 
            List.mapPartial (fn y => case y of 
                                  NORMAL (l,r,ex) => if null ex then SOME (l,r)  else raise NonSupportedRule
                                 | _ => raise NonSupportedRule) rs
        val rules2 = getNormal rules
        fun lookupVars name = L.find (fn (f,_) => Fun.equal (name,f)) vars2
        fun transTerm (t as (Fun (c,[],_))) = 
	    (case lookupVars c of
              SOME (f,ty) => Var (Var.fromString (Fun.toString f),ty)
            | NONE => t)
	  | transTerm (Fun (f,ts,_)) = Fun (f, List.map transTerm ts, Sort.null)
         fun makeDecls funs = L.map (fn (f,ty) => {sym = f,sort= ty}) funs 
         val decls = makeDecls funs
         val sortedrules = Trs.addFunSortToRules decls (List.map (fn (l,r) => (transTerm l, transTerm r)) rules2)
     in case sortedrules of
            SOME rs => (decls, rs, [])
          | NONE => (PU.printStdErr "Sort attachment failed.\n"; raise (MsTpdbError "unsupported format"))
     end
    handle NonSupportedRule => 
                 (PU.printStdErr "Unsupported rules.\n"; raise (MsTpdbError "unsupported format"))

*****)
end

local
    exception NonSupportedRule
in
fun checkRules xs ys = 
    let	fun getNormal ys = 
            List.mapPartial (fn y => case y of 
                                  NORMAL (l,r,ex) => if null ex then SOME (l,r)  else raise NonSupportedRule
                                 | _ => NONE) ys
         fun getRelative ys = 
            List.mapPartial (fn y => case y of 
                                RELATIVE (l,r,ex) => if null ex then SOME (l,r) else raise NonSupportedRule
                                | _ => NONE) ys
    in ([], 
        makeTrs xs (getNormal ys),
        makeTrs xs (getRelative ys))
    end
    handle NonSupportedRule => 
                 (PU.printStdErr "CONDITIONAL rules are not supported.\n"; 
                  raise (MsTpdbError "unsupported format"))
end

fun getRules decls = checkRules (getVAR decls) (getRULES decls)
fun getSortedRules decls = checkSortedRules (getFUN decls) (getVAR decls) (getRULES decls)

fun checkDecl decls = 
    if not (null (getTHEORY decls))
    then (PU.printStdErr "THEORY is not supported.\n"; 
          raise (MsTpdbError "unsupported format"))
    else if (isSome (getSTRATEGY decls))
    then (PU.printStdErr "STRATEGY is not supported.\n"; 
          raise (MsTpdbError "unsupported format"))
    else let val rest = getREST decls
             val funs = getFUN decls
             fun printfundecl (f,ty) = (Fun.toString f) ^ ":" ^ (Sort.toString ty) ^ "\n"
             fun printfundecls xs = "(FUN\n" ^ (LU.toStringSpace printfundecl xs) ^ ")\n"
         in if (!CrSolver.proveGroundConfluence)
            then if (null rest) 
                 then getSortedRules decls
                 else (PU.print ("(ignored inputs)" ^ (ListUtil.toStringSpace (fn x=>x) rest) ^ "\n");
                       getSortedRules decls)
            else if (null rest) andalso (null funs)
            then getRules decls
            else (PU.print ("(ignored inputs)" ^ (printfundecls funs) 
                              ^ (ListUtil.toStringSpace (fn x=>x) rest) ^ "\n");
                  getRules decls)
         end
end

(* 桼 *)

end
structure LrTable = Token.LrTable
structure Token = Token
local open LrTable in 
val table=let val actionRows =
"\
\\001\000\001\000\179\000\003\000\179\000\004\000\183\000\009\000\103\000\000\000\
\\001\000\001\000\011\000\016\000\010\000\017\000\009\000\018\000\008\000\
\\019\000\007\000\020\000\006\000\000\000\
\\001\000\001\000\028\000\000\000\
\\001\000\001\000\028\000\003\000\098\000\000\000\
\\001\000\001\000\057\000\013\000\056\000\000\000\
\\001\000\001\000\086\000\000\000\
\\001\000\001\000\087\000\000\000\
\\001\000\001\000\093\000\000\000\
\\001\000\001\000\115\000\000\000\
\\001\000\001\000\117\000\000\000\
\\001\000\003\000\012\000\000\000\
\\001\000\003\000\088\000\000\000\
\\001\000\003\000\101\000\000\000\
\\001\000\003\000\112\000\000\000\
\\001\000\003\000\120\000\000\000\
\\001\000\004\000\061\000\006\000\060\000\000\000\
\\001\000\004\000\102\000\000\000\
\\001\000\004\000\128\000\007\000\127\000\000\000\
\\001\000\005\000\105\000\000\000\
\\001\000\010\000\052\000\000\000\
\\001\000\011\000\099\000\000\000\
\\001\000\012\000\024\000\014\000\023\000\015\000\022\000\000\000\
\\001\000\021\000\000\000\000\000\
\\134\000\000\000\
\\135\000\000\000\
\\136\000\002\000\004\000\000\000\
\\137\000\000\000\
\\138\000\000\000\
\\139\000\000\000\
\\140\000\000\000\
\\141\000\000\000\
\\142\000\000\000\
\\143\000\000\000\
\\144\000\000\000\
\\145\000\000\000\
\\146\000\000\000\
\\147\000\000\000\
\\148\000\000\000\
\\149\000\000\000\
\\150\000\000\000\
\\151\000\000\000\
\\152\000\000\000\
\\153\000\000\000\
\\154\000\000\000\
\\155\000\000\000\
\\156\000\000\000\
\\157\000\000\000\
\\158\000\000\000\
\\159\000\000\000\
\\160\000\000\000\
\\161\000\000\000\
\\162\000\001\000\049\000\002\000\048\000\004\000\047\000\005\000\046\000\
\\006\000\045\000\007\000\044\000\008\000\043\000\009\000\042\000\
\\010\000\041\000\011\000\040\000\012\000\039\000\013\000\038\000\
\\014\000\037\000\015\000\036\000\016\000\035\000\017\000\034\000\
\\018\000\033\000\019\000\032\000\020\000\031\000\000\000\
\\163\000\000\000\
\\164\000\000\000\
\\165\000\001\000\075\000\000\000\
\\166\000\000\000\
\\167\000\002\000\020\000\000\000\
\\168\000\000\000\
\\169\000\000\000\
\\170\000\000\000\
\\171\000\001\000\028\000\000\000\
\\172\000\000\000\
\\173\000\000\000\
\\174\000\001\000\028\000\000\000\
\\175\000\000\000\
\\176\000\001\000\015\000\000\000\
\\177\000\000\000\
\\178\000\001\000\018\000\000\000\
\\180\000\000\000\
\\181\000\010\000\054\000\000\000\
\\182\000\000\000\
\\183\000\009\000\103\000\000\000\
\\184\000\000\000\
\\185\000\008\000\111\000\000\000\
\\186\000\000\000\
\\187\000\008\000\110\000\000\000\
\\188\000\000\000\
\\189\000\009\000\129\000\000\000\
\\190\000\000\000\
\\191\000\000\000\
\\192\000\000\000\
\\193\000\002\000\063\000\000\000\
\\194\000\000\000\
\\195\000\000\000\
\\196\000\000\000\
\\197\000\009\000\113\000\000\000\
\\198\000\000\000\
\\199\000\000\000\
\\200\000\000\000\
\\201\000\000\000\
\\202\000\002\000\059\000\000\000\
\\203\000\000\000\
\\204\000\001\000\109\000\000\000\
\\205\000\000\000\
\"
val actionRowNumbers =
"\025\000\023\000\001\000\010\000\
\\065\000\067\000\056\000\021\000\
\\063\000\051\000\025\000\065\000\
\\027\000\019\000\067\000\026\000\
\\069\000\028\000\004\000\030\000\
\\087\000\086\000\090\000\015\000\
\\063\000\029\000\081\000\051\000\
\\031\000\051\000\051\000\051\000\
\\051\000\051\000\051\000\051\000\
\\051\000\051\000\054\000\051\000\
\\051\000\051\000\051\000\051\000\
\\051\000\051\000\051\000\051\000\
\\024\000\064\000\005\000\066\000\
\\006\000\011\000\060\000\054\000\
\\088\000\007\000\002\000\002\000\
\\062\000\003\000\033\000\043\000\
\\042\000\050\000\049\000\044\000\
\\048\000\047\000\046\000\045\000\
\\020\000\054\000\041\000\040\000\
\\038\000\035\000\039\000\037\000\
\\036\000\012\000\032\000\016\000\
\\000\000\070\000\056\000\018\000\
\\060\000\058\000\057\000\092\000\
\\075\000\073\000\013\000\085\000\
\\082\000\052\000\053\000\051\000\
\\008\000\009\000\055\000\002\000\
\\059\000\092\000\014\000\093\000\
\\002\000\002\000\083\000\002\000\
\\034\000\068\000\072\000\071\000\
\\061\000\091\000\090\000\017\000\
\\077\000\076\000\074\000\084\000\
\\089\000\002\000\002\000\002\000\
\\080\000\079\000\078\000\022\000"
val gotoT =
"\
\\001\000\131\000\002\000\001\000\000\000\
\\000\000\
\\003\000\003\000\000\000\
\\000\000\
\\015\000\012\000\018\000\011\000\000\000\
\\016\000\015\000\017\000\014\000\000\000\
\\007\000\017\000\000\000\
\\022\000\019\000\000\000\
\\011\000\025\000\012\000\024\000\020\000\023\000\000\000\
\\004\000\028\000\005\000\027\000\000\000\
\\002\000\048\000\000\000\
\\015\000\049\000\018\000\011\000\000\000\
\\000\000\
\\000\000\
\\016\000\051\000\017\000\014\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\008\000\053\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\023\000\056\000\000\000\
\\000\000\
\\011\000\060\000\012\000\024\000\020\000\023\000\000\000\
\\000\000\
\\000\000\
\\004\000\062\000\005\000\027\000\000\000\
\\000\000\
\\004\000\063\000\005\000\027\000\000\000\
\\004\000\064\000\005\000\027\000\000\000\
\\004\000\065\000\005\000\027\000\000\000\
\\004\000\066\000\005\000\027\000\000\000\
\\004\000\067\000\005\000\027\000\000\000\
\\004\000\068\000\005\000\027\000\000\000\
\\004\000\069\000\005\000\027\000\000\000\
\\004\000\070\000\005\000\027\000\000\000\
\\004\000\071\000\005\000\027\000\000\000\
\\006\000\072\000\000\000\
\\004\000\074\000\005\000\027\000\000\000\
\\004\000\075\000\005\000\027\000\000\000\
\\004\000\076\000\005\000\027\000\000\000\
\\004\000\077\000\005\000\027\000\000\000\
\\004\000\078\000\005\000\027\000\000\000\
\\004\000\079\000\005\000\027\000\000\000\
\\004\000\080\000\005\000\027\000\000\000\
\\004\000\081\000\005\000\027\000\000\000\
\\004\000\082\000\005\000\027\000\000\000\
\\000\000\
\\000\000\
\\019\000\083\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\009\000\089\000\010\000\088\000\020\000\087\000\000\000\
\\006\000\090\000\000\000\
\\000\000\
\\000\000\
\\020\000\092\000\000\000\
\\020\000\093\000\000\000\
\\000\000\
\\020\000\095\000\021\000\094\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\006\000\098\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\007\000\102\000\000\000\
\\000\000\
\\009\000\104\000\010\000\088\000\020\000\087\000\000\000\
\\000\000\
\\000\000\
\\024\000\106\000\025\000\105\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\004\000\112\000\005\000\027\000\000\000\
\\000\000\
\\019\000\114\000\000\000\
\\000\000\
\\020\000\116\000\000\000\
\\000\000\
\\024\000\117\000\025\000\105\000\000\000\
\\000\000\
\\000\000\
\\013\000\121\000\014\000\120\000\020\000\119\000\000\000\
\\013\000\122\000\014\000\120\000\020\000\119\000\000\000\
\\000\000\
\\020\000\095\000\021\000\123\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\023\000\124\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\020\000\128\000\000\000\
\\020\000\129\000\000\000\
\\013\000\130\000\014\000\120\000\020\000\119\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\"
val numstates = 132
val numrules = 72
val s = ref "" and index = ref 0
val string_to_int = fn () => 
let val i = !index
in index := i+2; Char.ord(String.sub(!s,i)) + Char.ord(String.sub(!s,i+1)) * 256
end
val string_to_list = fn s' =>
    let val len = String.size s'
        fun f () =
           if !index < len then string_to_int() :: f()
           else nil
   in index := 0; s := s'; f ()
   end
val string_to_pairlist = fn (conv_key,conv_entry) =>
     let fun f () =
         case string_to_int()
         of 0 => EMPTY
          | n => PAIR(conv_key (n-1),conv_entry (string_to_int()),f())
     in f
     end
val string_to_pairlist_default = fn (conv_key,conv_entry) =>
    let val conv_row = string_to_pairlist(conv_key,conv_entry)
    in fn () =>
       let val default = conv_entry(string_to_int())
           val row = conv_row()
       in (row,default)
       end
   end
val string_to_table = fn (convert_row,s') =>
    let val len = String.size s'
        fun f ()=
           if !index < len then convert_row() :: f()
           else nil
     in (s := s'; index := 0; f ())
     end
local
  val memo = Array.array(numstates+numrules,ERROR)
  val _ =let fun g i=(Array.update(memo,i,REDUCE(i-numstates)); g(i+1))
       fun f i =
            if i=numstates then g i
            else (Array.update(memo,i,SHIFT (STATE i)); f (i+1))
          in f 0 handle General.Subscript => ()
          end
in
val entry_to_action = fn 0 => ACCEPT | 1 => ERROR | j => Array.sub(memo,(j-2))
end
val gotoT=Array.fromList(string_to_table(string_to_pairlist(NT,STATE),gotoT))
val actionRows=string_to_table(string_to_pairlist_default(T,entry_to_action),actionRows)
val actionRowNumbers = string_to_list actionRowNumbers
val actionT = let val actionRowLookUp=
let val a=Array.fromList(actionRows) in fn i=>Array.sub(a,i) end
in Array.fromList(List.map actionRowLookUp actionRowNumbers)
end
in LrTable.mkLrTable {actions=actionT,gotos=gotoT,numRules=numrules,
numStates=numstates,initialState=STATE 0}
end
end
local open Header in
type pos = int
type arg = unit
structure MlyValue = 
struct
datatype svalue = VOID | ntVOID of unit ->  unit
 | ID of unit ->  (string) | XintX of unit ->  (int)
 | XintlistX of unit ->  (int list)
 | XcsstratlistX of unit ->  ( ( Term.fun_key * int list )  list)
 | XstrategydeclX of unit ->  (tpdb_strat)
 | XtermlistX of unit ->  (Term.term list)
 | XtermX of unit ->  (Term.term)
 | XsortlistX of unit ->  (Term.sort_key list)
 | XfunX of unit ->  (Term.fun_key*Term.sort_key)
 | XvarX of unit ->  (Term.var_key*Term.sort_key)
 | XvarlistX of unit ->  ( ( Term.var_key * Term.sort_key )  list)
 | XfunlistX of unit ->  ( ( Term.fun_key * Term.sort_key )  list)
 | XcondX of unit ->  (tpdb_cond)
 | XcondlistX of unit ->  (tpdb_cond list)
 | XruleX of unit ->  (tpdb_rule)
 | XlistofrulesX of unit ->  (tpdb_rule list)
 | XequationX of unit ->  (Term.term*Term.term)
 | XeqlistX of unit ->  ( ( Term.term * Term.term )  list)
 | XthdeclX of unit ->  (tpdb_thdecl)
 | XlistofthdeclX of unit ->  (tpdb_thdecl list)
 | XidlistX of unit ->  (string list)
 | XstringX of unit ->  (string list)
 | XanylistX of unit ->  (string list)
 | XdeclX of unit ->  (tpdb_decl)
 | XspecX of unit ->  (tpdb_decl list)
 | XstartX of unit ->  ( ( Term.decl list * (Term.term * Term.term) list * (Term.term * Term.term) list ) )
end
type svalue = MlyValue.svalue
type result = 
 ( Term.decl list * (Term.term * Term.term) list * (Term.term * Term.term) list ) 
end
structure EC=
struct
open LrTable
infix 5 $$
fun x $$ y = y::x
val is_keyword =
fn _ => false
val preferred_change : (term list * term list) list = 
nil
val noShift = 
fn (T 20) => true | _ => false
val showTerminal =
fn (T 0) => "ID"
  | (T 1) => "LPAREN"
  | (T 2) => "RPAREN"
  | (T 3) => "ARROW"
  | (T 4) => "EQUAL"
  | (T 5) => "ARROWEQ"
  | (T 6) => "JOIN"
  | (T 7) => "BAR"
  | (T 8) => "COMMA"
  | (T 9) => "COLON"
  | (T 10) => "DQUOTE"
  | (T 11) => "CONTEXTSENSITIVE"
  | (T 12) => "EQUATIONS"
  | (T 13) => "INNERMOST"
  | (T 14) => "OUTERMOST"
  | (T 15) => "RULES"
  | (T 16) => "STRATEGY"
  | (T 17) => "THEORY"
  | (T 18) => "VAR"
  | (T 19) => "FUN"
  | (T 20) => "EOF"
  | _ => "bogus-term"
local open Header in
val errtermvalue=
fn _ => MlyValue.VOID
end
val terms : term list = nil
 $$ (T 20) $$ (T 19) $$ (T 18) $$ (T 17) $$ (T 16) $$ (T 15) $$ (T 14)
 $$ (T 13) $$ (T 12) $$ (T 11) $$ (T 10) $$ (T 9) $$ (T 8) $$ (T 7)
 $$ (T 6) $$ (T 5) $$ (T 4) $$ (T 3) $$ (T 2) $$ (T 1)end
structure Actions =
struct 
exception mlyAction of int
local open Header in
val actions = 
fn (i392,defaultPos,stack,
    (()):arg) =>
case (i392,stack)
of  ( 0, ( ( _, ( MlyValue.XspecX XspecX1, XspecX1left, XspecX1right))
 :: rest671)) => let val  result = MlyValue.XstartX (fn _ => let val 
 (XspecX as XspecX1) = XspecX1 ()
 in (checkDecl (XspecX))
end)
 in ( LrTable.NT 0, ( result, XspecX1left, XspecX1right), rest671)
end
|  ( 1, ( ( _, ( MlyValue.XspecX XspecX1, _, XspecX1right)) :: _ :: (
 _, ( MlyValue.XdeclX XdeclX1, _, _)) :: ( _, ( _, LPAREN1left, _)) ::
 rest671)) => let val  result = MlyValue.XspecX (fn _ => let val  (
XdeclX as XdeclX1) = XdeclX1 ()
 val  (XspecX as XspecX1) = XspecX1 ()
 in (XdeclX::XspecX)
end)
 in ( LrTable.NT 1, ( result, LPAREN1left, XspecX1right), rest671)
end
|  ( 2, ( rest671)) => let val  result = MlyValue.XspecX (fn _ => ([])
)
 in ( LrTable.NT 1, ( result, defaultPos, defaultPos), rest671)
end
|  ( 3, ( ( _, ( MlyValue.XvarlistX XvarlistX1, _, XvarlistX1right))
 :: ( _, ( _, VAR1left, _)) :: rest671)) => let val  result = 
MlyValue.XdeclX (fn _ => let val  (XvarlistX as XvarlistX1) = 
XvarlistX1 ()
 in (VAR XvarlistX)
end)
 in ( LrTable.NT 2, ( result, VAR1left, XvarlistX1right), rest671)
end
|  ( 4, ( ( _, ( MlyValue.XfunlistX XfunlistX1, _, XfunlistX1right))
 :: ( _, ( _, FUN1left, _)) :: rest671)) => let val  result = 
MlyValue.XdeclX (fn _ => let val  (XfunlistX as XfunlistX1) = 
XfunlistX1 ()
 in (FUN XfunlistX)
end)
 in ( LrTable.NT 2, ( result, FUN1left, XfunlistX1right), rest671)
end
|  ( 5, ( ( _, ( MlyValue.XlistofthdeclX XlistofthdeclX1, _, 
XlistofthdeclX1right)) :: ( _, ( _, THEORY1left, _)) :: rest671)) =>
 let val  result = MlyValue.XdeclX (fn _ => let val  (XlistofthdeclX
 as XlistofthdeclX1) = XlistofthdeclX1 ()
 in (THEORY XlistofthdeclX)
end)
 in ( LrTable.NT 2, ( result, THEORY1left, XlistofthdeclX1right), 
rest671)
end
|  ( 6, ( ( _, ( MlyValue.XlistofrulesX XlistofrulesX1, _, 
XlistofrulesX1right)) :: ( _, ( _, RULES1left, _)) :: rest671)) => let
 val  result = MlyValue.XdeclX (fn _ => let val  (XlistofrulesX as 
XlistofrulesX1) = XlistofrulesX1 ()
 in (RULES XlistofrulesX)
end)
 in ( LrTable.NT 2, ( result, RULES1left, XlistofrulesX1right), 
rest671)
end
|  ( 7, ( ( _, ( MlyValue.XstrategydeclX XstrategydeclX1, _, 
XstrategydeclX1right)) :: ( _, ( _, STRATEGY1left, _)) :: rest671)) =>
 let val  result = MlyValue.XdeclX (fn _ => let val  (XstrategydeclX
 as XstrategydeclX1) = XstrategydeclX1 ()
 in (STRATEGY XstrategydeclX)
end)
 in ( LrTable.NT 2, ( result, STRATEGY1left, XstrategydeclX1right), 
rest671)
end
|  ( 8, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  
result = MlyValue.XdeclX (fn _ => let val  (ID as ID1) = ID1 ()
 val  (XanylistX as XanylistX1) = XanylistX1 ()
 in (REST (ID::XanylistX))
end)
 in ( LrTable.NT 2, ( result, ID1left, XanylistX1right), rest671)
end
|  ( 9, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  
result = MlyValue.XanylistX (fn _ => let val  (ID as ID1) = ID1 ()
 val  (XanylistX as XanylistX1) = XanylistX1 ()
 in (ID::XanylistX)
end)
 in ( LrTable.NT 3, ( result, ID1left, XanylistX1right), rest671)
end
|  ( 10, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( MlyValue.XstringX XstringX1, XstringX1left, _)) :: rest671)
) => let val  result = MlyValue.XanylistX (fn _ => let val  (XstringX
 as XstringX1) = XstringX1 ()
 val  (XanylistX as XanylistX1) = XanylistX1 ()
 in (XstringX @ XanylistX)
end)
 in ( LrTable.NT 3, ( result, XstringX1left, XanylistX1right), rest671
)
end
|  ( 11, ( ( _, ( MlyValue.XanylistX XanylistX2, _, XanylistX2right))
 :: _ :: ( _, ( MlyValue.XanylistX XanylistX1, _, _)) :: ( _, ( _, 
LPAREN1left, _)) :: rest671)) => let val  result = MlyValue.XanylistX
 (fn _ => let val  XanylistX1 = XanylistX1 ()
 val  XanylistX2 = XanylistX2 ()
 in (("("::XanylistX1) @ (")"::XanylistX2))
end)
 in ( LrTable.NT 3, ( result, LPAREN1left, XanylistX2right), rest671)

end
|  ( 12, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, JOIN1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("-><-"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, JOIN1left, XanylistX1right), rest671)

end
|  ( 13, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, ARROW1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("->"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, ARROW1left, XanylistX1right), rest671)

end
|  ( 14, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, EQUAL1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("="::XanylistX)
end)
 in ( LrTable.NT 3, ( result, EQUAL1left, XanylistX1right), rest671)

end
|  ( 15, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, BAR1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("|"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, BAR1left, XanylistX1right), rest671)
end
|  ( 16, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, ARROWEQ1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("->="::XanylistX)
end)
 in ( LrTable.NT 3, ( result, ARROWEQ1left, XanylistX1right), rest671)

end
|  ( 17, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, COMMA1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in (","::XanylistX)
end)
 in ( LrTable.NT 3, ( result, COMMA1left, XanylistX1right), rest671)

end
|  ( 18, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, COLON1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in (":"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, COLON1left, XanylistX1right), rest671)

end
|  ( 19, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, VAR1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("VAR"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, VAR1left, XanylistX1right), rest671)
end
|  ( 20, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, FUN1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("FUN"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, FUN1left, XanylistX1right), rest671)
end
|  ( 21, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, RULES1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("RULES"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, RULES1left, XanylistX1right), rest671)

end
|  ( 22, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, CONTEXTSENSITIVE1left, _)) :: rest671)) => let val  
result = MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1
) = XanylistX1 ()
 in ("CONTEXTSENSITIVE"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, CONTEXTSENSITIVE1left, XanylistX1right),
 rest671)
end
|  ( 23, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, EQUATIONS1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("EQUATIONS"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, EQUATIONS1left, XanylistX1right), 
rest671)
end
|  ( 24, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, INNERMOST1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("INNERMOST"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, INNERMOST1left, XanylistX1right), 
rest671)
end
|  ( 25, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, OUTERMOST1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("OUTERMOST"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, OUTERMOST1left, XanylistX1right), 
rest671)
end
|  ( 26, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, STRATEGY1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("STRATEGY"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, STRATEGY1left, XanylistX1right), rest671
)
end
|  ( 27, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, THEORY1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("THEORY"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, THEORY1left, XanylistX1right), rest671)

end
|  ( 28, ( rest671)) => let val  result = MlyValue.XanylistX (fn _ =>
 ([]))
 in ( LrTable.NT 3, ( result, defaultPos, defaultPos), rest671)
end
|  ( 29, ( ( _, ( _, _, DQUOTE2right)) :: ( _, ( MlyValue.XidlistX 
XidlistX1, _, _)) :: ( _, ( _, DQUOTE1left, _)) :: rest671)) => let
 val  result = MlyValue.XstringX (fn _ => let val  (XidlistX as 
XidlistX1) = XidlistX1 ()
 in (XidlistX)
end)
 in ( LrTable.NT 4, ( result, DQUOTE1left, DQUOTE2right), rest671)
end
|  ( 30, ( ( _, ( MlyValue.XidlistX XidlistX1, _, XidlistX1right)) :: 
( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  result =
 MlyValue.XidlistX (fn _ => let val  (ID as ID1) = ID1 ()
 val  (XidlistX as XidlistX1) = XidlistX1 ()
 in (ID::XidlistX)
end)
 in ( LrTable.NT 5, ( result, ID1left, XidlistX1right), rest671)
end
|  ( 31, ( rest671)) => let val  result = MlyValue.XidlistX (fn _ => (
[]))
 in ( LrTable.NT 5, ( result, defaultPos, defaultPos), rest671)
end
|  ( 32, ( ( _, ( MlyValue.XlistofthdeclX XlistofthdeclX1, _, 
XlistofthdeclX1right)) :: _ :: ( _, ( MlyValue.XthdeclX XthdeclX1, _,
 _)) :: ( _, ( _, LPAREN1left, _)) :: rest671)) => let val  result = 
MlyValue.XlistofthdeclX (fn _ => let val  (XthdeclX as XthdeclX1) = 
XthdeclX1 ()
 val  (XlistofthdeclX as XlistofthdeclX1) = XlistofthdeclX1 ()
 in (XthdeclX::XlistofthdeclX)
end)
 in ( LrTable.NT 6, ( result, LPAREN1left, XlistofthdeclX1right), 
rest671)
end
|  ( 33, ( rest671)) => let val  result = MlyValue.XlistofthdeclX (fn
 _ => ([]))
 in ( LrTable.NT 6, ( result, defaultPos, defaultPos), rest671)
end
|  ( 34, ( ( _, ( MlyValue.XidlistX XidlistX1, _, XidlistX1right)) :: 
( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  result =
 MlyValue.XthdeclX (fn _ => let val  (ID as ID1) = ID1 ()
 val  (XidlistX as XidlistX1) = XidlistX1 ()
 in (THEORIES (ID,XidlistX))
end)
 in ( LrTable.NT 7, ( result, ID1left, XidlistX1right), rest671)
end
|  ( 35, ( ( _, ( MlyValue.XeqlistX XeqlistX1, _, XeqlistX1right)) :: 
( _, ( _, EQUATIONS1left, _)) :: rest671)) => let val  result = 
MlyValue.XthdeclX (fn _ => let val  (XeqlistX as XeqlistX1) = 
XeqlistX1 ()
 in (EQUATIONS XeqlistX)
end)
 in ( LrTable.NT 7, ( result, EQUATIONS1left, XeqlistX1right), rest671
)
end
|  ( 36, ( ( _, ( MlyValue.XeqlistX XeqlistX1, _, XeqlistX1right)) :: 
( _, ( MlyValue.XequationX XequationX1, XequationX1left, _)) :: 
rest671)) => let val  result = MlyValue.XeqlistX (fn _ => let val  (
XequationX as XequationX1) = XequationX1 ()
 val  (XeqlistX as XeqlistX1) = XeqlistX1 ()
 in (XequationX::XeqlistX)
end)
 in ( LrTable.NT 8, ( result, XequationX1left, XeqlistX1right), 
rest671)
end
|  ( 37, ( rest671)) => let val  result = MlyValue.XeqlistX (fn _ => (
[]))
 in ( LrTable.NT 8, ( result, defaultPos, defaultPos), rest671)
end
|  ( 38, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XequationX (fn _ => let val  XtermX1 = XtermX1
 ()
 val  XtermX2 = XtermX2 ()
 in ((XtermX1,XtermX2))
end)
 in ( LrTable.NT 9, ( result, XtermX1left, XtermX2right), rest671)
end
|  ( 39, ( ( _, ( MlyValue.XlistofrulesX XlistofrulesX1, _, 
XlistofrulesX1right)) :: ( _, ( MlyValue.XruleX XruleX1, XruleX1left,
 _)) :: rest671)) => let val  result = MlyValue.XlistofrulesX (fn _ =>
 let val  (XruleX as XruleX1) = XruleX1 ()
 val  (XlistofrulesX as XlistofrulesX1) = XlistofrulesX1 ()
 in (XruleX::XlistofrulesX)
end)
 in ( LrTable.NT 10, ( result, XruleX1left, XlistofrulesX1right), 
rest671)
end
|  ( 40, ( rest671)) => let val  result = MlyValue.XlistofrulesX (fn _
 => ([]))
 in ( LrTable.NT 10, ( result, defaultPos, defaultPos), rest671)
end
|  ( 41, ( ( _, ( MlyValue.XfunlistX XfunlistX1, _, XfunlistX1right))
 :: ( _, ( MlyValue.XfunX XfunX1, XfunX1left, _)) :: rest671)) => let
 val  result = MlyValue.XfunlistX (fn _ => let val  (XfunX as XfunX1)
 = XfunX1 ()
 val  (XfunlistX as XfunlistX1) = XfunlistX1 ()
 in (XfunX::XfunlistX)
end)
 in ( LrTable.NT 14, ( result, XfunX1left, XfunlistX1right), rest671)

end
|  ( 42, ( rest671)) => let val  result = MlyValue.XfunlistX (fn _ =>
 ([]))
 in ( LrTable.NT 14, ( result, defaultPos, defaultPos), rest671)
end
|  ( 43, ( ( _, ( MlyValue.XvarlistX XvarlistX1, _, XvarlistX1right))
 :: ( _, ( MlyValue.XvarX XvarX1, XvarX1left, _)) :: rest671)) => let
 val  result = MlyValue.XvarlistX (fn _ => let val  (XvarX as XvarX1)
 = XvarX1 ()
 val  (XvarlistX as XvarlistX1) = XvarlistX1 ()
 in (XvarX::XvarlistX)
end)
 in ( LrTable.NT 15, ( result, XvarX1left, XvarlistX1right), rest671)

end
|  ( 44, ( rest671)) => let val  result = MlyValue.XvarlistX (fn _ =>
 ([]))
 in ( LrTable.NT 15, ( result, defaultPos, defaultPos), rest671)
end
|  ( 45, ( ( _, ( MlyValue.ID ID2, _, ID2right)) :: _ :: ( _, ( 
MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  result = 
MlyValue.XfunX (fn _ => let val  ID1 = ID1 ()
 val  ID2 = ID2 ()
 in ((Fun.fromString ID1,Sort.fromString ID2))
end)
 in ( LrTable.NT 17, ( result, ID1left, ID2right), rest671)
end
|  ( 46, ( ( _, ( MlyValue.ID ID2, _, ID2right)) :: _ :: ( _, ( 
MlyValue.XsortlistX XsortlistX1, _, _)) :: _ :: ( _, ( MlyValue.ID ID1
, ID1left, _)) :: rest671)) => let val  result = MlyValue.XfunX (fn _
 => let val  ID1 = ID1 ()
 val  (XsortlistX as XsortlistX1) = XsortlistX1 ()
 val  ID2 = ID2 ()
 in ((Fun.fromString ID1,Sort.Proc (XsortlistX,Sort.fromString ID2)))

end)
 in ( LrTable.NT 17, ( result, ID1left, ID2right), rest671)
end
|  ( 47, ( ( _, ( MlyValue.ID ID1, ID1left, ID1right)) :: rest671)) =>
 let val  result = MlyValue.XvarX (fn _ => let val  (ID as ID1) = ID1
 ()
 in ((Var.fromString ID,Sort.null))
end)
 in ( LrTable.NT 16, ( result, ID1left, ID1right), rest671)
end
|  ( 48, ( ( _, ( MlyValue.ID ID2, _, ID2right)) :: _ :: ( _, ( 
MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  result = 
MlyValue.XvarX (fn _ => let val  ID1 = ID1 ()
 val  ID2 = ID2 ()
 in ((Var.fromString ID1, Sort.fromString ID2))
end)
 in ( LrTable.NT 16, ( result, ID1left, ID2right), rest671)
end
|  ( 49, ( ( _, ( MlyValue.ID ID1, ID1left, ID1right)) :: rest671)) =>
 let val  result = MlyValue.XsortlistX (fn _ => let val  (ID as ID1) =
 ID1 ()
 in ([Sort.fromString ID])
end)
 in ( LrTable.NT 18, ( result, ID1left, ID1right), rest671)
end
|  ( 50, ( ( _, ( MlyValue.XsortlistX XsortlistX1, _, XsortlistX1right
)) :: _ :: ( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let
 val  result = MlyValue.XsortlistX (fn _ => let val  (ID as ID1) = ID1
 ()
 val  (XsortlistX as XsortlistX1) = XsortlistX1 ()
 in ([Sort.fromString ID] @ XsortlistX)
end)
 in ( LrTable.NT 18, ( result, ID1left, XsortlistX1right), rest671)

end
|  ( 51, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XruleX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (NORMAL (XtermX1,XtermX2,[]))
end)
 in ( LrTable.NT 11, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 52, ( ( _, ( MlyValue.XcondlistX XcondlistX1, _, XcondlistX1right
)) :: _ :: ( _, ( MlyValue.XtermX XtermX2, _, _)) :: _ :: ( _, ( 
MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let val  
result = MlyValue.XruleX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 val  (XcondlistX as XcondlistX1) = XcondlistX1 ()
 in (NORMAL(XtermX1,XtermX2,XcondlistX))
end)
 in ( LrTable.NT 11, ( result, XtermX1left, XcondlistX1right), rest671
)
end
|  ( 53, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XruleX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (RELATIVE (XtermX1,XtermX2,[]))
end)
 in ( LrTable.NT 11, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 54, ( ( _, ( MlyValue.XcondlistX XcondlistX1, _, XcondlistX1right
)) :: _ :: ( _, ( MlyValue.XtermX XtermX2, _, _)) :: _ :: ( _, ( 
MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let val  
result = MlyValue.XruleX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 val  (XcondlistX as XcondlistX1) = XcondlistX1 ()
 in (RELATIVE (XtermX1,XtermX2,XcondlistX))
end)
 in ( LrTable.NT 11, ( result, XtermX1left, XcondlistX1right), rest671
)
end
|  ( 55, ( ( _, ( MlyValue.XcondX XcondX1, XcondX1left, XcondX1right))
 :: rest671)) => let val  result = MlyValue.XcondlistX (fn _ => let
 val  (XcondX as XcondX1) = XcondX1 ()
 in ([XcondX])
end)
 in ( LrTable.NT 12, ( result, XcondX1left, XcondX1right), rest671)

end
|  ( 56, ( ( _, ( MlyValue.XcondlistX XcondlistX1, _, XcondlistX1right
)) :: _ :: ( _, ( MlyValue.XcondX XcondX1, XcondX1left, _)) :: rest671
)) => let val  result = MlyValue.XcondlistX (fn _ => let val  (XcondX
 as XcondX1) = XcondX1 ()
 val  (XcondlistX as XcondlistX1) = XcondlistX1 ()
 in (XcondX::XcondlistX)
end)
 in ( LrTable.NT 12, ( result, XcondX1left, XcondlistX1right), rest671
)
end
|  ( 57, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XcondX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (ORIENT (XtermX1,XtermX2))
end)
 in ( LrTable.NT 13, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 58, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XcondX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (JOIN (XtermX1,XtermX2))
end)
 in ( LrTable.NT 13, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 59, ( ( _, ( MlyValue.ID ID1, ID1left, ID1right)) :: rest671)) =>
 let val  result = MlyValue.XtermX (fn _ => let val  (ID as ID1) = ID1
 ()
 in (Term.Fun (Atom.atom ID,[],Sort.null))
end)
 in ( LrTable.NT 19, ( result, ID1left, ID1right), rest671)
end
|  ( 60, ( ( _, ( _, _, RPAREN1right)) :: _ :: ( _, ( MlyValue.ID ID1,
 ID1left, _)) :: rest671)) => let val  result = MlyValue.XtermX (fn _
 => let val  (ID as ID1) = ID1 ()
 in (Term.Fun (Atom.atom ID,[],Sort.null))
end)
 in ( LrTable.NT 19, ( result, ID1left, RPAREN1right), rest671)
end
|  ( 61, ( ( _, ( _, _, RPAREN1right)) :: ( _, ( MlyValue.XtermlistX 
XtermlistX1, _, _)) :: _ :: ( _, ( MlyValue.ID ID1, ID1left, _)) :: 
rest671)) => let val  result = MlyValue.XtermX (fn _ => let val  (ID
 as ID1) = ID1 ()
 val  (XtermlistX as XtermlistX1) = XtermlistX1 ()
 in (Term.Fun (Atom.atom ID,XtermlistX,Sort.null))
end)
 in ( LrTable.NT 19, ( result, ID1left, RPAREN1right), rest671)
end
|  ( 62, ( ( _, ( MlyValue.XtermlistX XtermlistX1, _, XtermlistX1right
)) :: _ :: ( _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671
)) => let val  result = MlyValue.XtermlistX (fn _ => let val  (XtermX
 as XtermX1) = XtermX1 ()
 val  (XtermlistX as XtermlistX1) = XtermlistX1 ()
 in (XtermX::XtermlistX)
end)
 in ( LrTable.NT 20, ( result, XtermX1left, XtermlistX1right), rest671
)
end
|  ( 63, ( ( _, ( MlyValue.XtermX XtermX1, XtermX1left, XtermX1right))
 :: rest671)) => let val  result = MlyValue.XtermlistX (fn _ => let
 val  (XtermX as XtermX1) = XtermX1 ()
 in ([XtermX])
end)
 in ( LrTable.NT 20, ( result, XtermX1left, XtermX1right), rest671)

end
|  ( 64, ( ( _, ( _, INNERMOST1left, INNERMOST1right)) :: rest671)) =>
 let val  result = MlyValue.XstrategydeclX (fn _ => (STRAT_INNERMOST))
 in ( LrTable.NT 21, ( result, INNERMOST1left, INNERMOST1right), 
rest671)
end
|  ( 65, ( ( _, ( _, OUTERMOST1left, OUTERMOST1right)) :: rest671)) =>
 let val  result = MlyValue.XstrategydeclX (fn _ => (STRAT_OUTERMOST))
 in ( LrTable.NT 21, ( result, OUTERMOST1left, OUTERMOST1right), 
rest671)
end
|  ( 66, ( ( _, ( MlyValue.XcsstratlistX XcsstratlistX1, _, 
XcsstratlistX1right)) :: ( _, ( _, CONTEXTSENSITIVE1left, _)) :: 
rest671)) => let val  result = MlyValue.XstrategydeclX (fn _ => let
 val  (XcsstratlistX as XcsstratlistX1) = XcsstratlistX1 ()
 in (STRAT_CS XcsstratlistX)
end)
 in ( LrTable.NT 21, ( result, CONTEXTSENSITIVE1left, 
XcsstratlistX1right), rest671)
end
|  ( 67, ( ( _, ( MlyValue.XcsstratlistX XcsstratlistX1, _, 
XcsstratlistX1right)) :: _ :: ( _, ( MlyValue.XintlistX XintlistX1, _,
 _)) :: ( _, ( MlyValue.ID ID1, _, _)) :: ( _, ( _, LPAREN1left, _))
 :: rest671)) => let val  result = MlyValue.XcsstratlistX (fn _ => let
 val  (ID as ID1) = ID1 ()
 val  (XintlistX as XintlistX1) = XintlistX1 ()
 val  (XcsstratlistX as XcsstratlistX1) = XcsstratlistX1 ()
 in ((Atom.atom ID,XintlistX)::XcsstratlistX)
end)
 in ( LrTable.NT 22, ( result, LPAREN1left, XcsstratlistX1right), 
rest671)
end
|  ( 68, ( rest671)) => let val  result = MlyValue.XcsstratlistX (fn _
 => ([]))
 in ( LrTable.NT 22, ( result, defaultPos, defaultPos), rest671)
end
|  ( 69, ( ( _, ( MlyValue.XintlistX XintlistX1, _, XintlistX1right))
 :: ( _, ( MlyValue.XintX XintX1, XintX1left, _)) :: rest671)) => let
 val  result = MlyValue.XintlistX (fn _ => let val  (XintX as XintX1)
 = XintX1 ()
 val  (XintlistX as XintlistX1) = XintlistX1 ()
 in (XintX::XintlistX)
end)
 in ( LrTable.NT 23, ( result, XintX1left, XintlistX1right), rest671)

end
|  ( 70, ( rest671)) => let val  result = MlyValue.XintlistX (fn _ =>
 ([]))
 in ( LrTable.NT 23, ( result, defaultPos, defaultPos), rest671)
end
|  ( 71, ( ( _, ( MlyValue.ID ID1, ID1left, ID1right)) :: rest671)) =>
 let val  result = MlyValue.XintX (fn _ => let val  (ID as ID1) = ID1
 ()
 in (valOf (Int.fromString ID))
end)
 in ( LrTable.NT 24, ( result, ID1left, ID1right), rest671)
end
| _ => raise (mlyAction i392)
end
val void = MlyValue.VOID
val extract = fn a => (fn MlyValue.XstartX x => x
| _ => let exception ParseInternal
	in raise ParseInternal end) a ()
end
end
structure Tokens : MsTpdb_TOKENS =
struct
type svalue = ParserData.svalue
type ('a,'b) token = ('a,'b) Token.token
fun ID (i,p1,p2) = Token.TOKEN (ParserData.LrTable.T 0,(
ParserData.MlyValue.ID (fn () => i),p1,p2))
fun LPAREN (p1,p2) = Token.TOKEN (ParserData.LrTable.T 1,(
ParserData.MlyValue.VOID,p1,p2))
fun RPAREN (p1,p2) = Token.TOKEN (ParserData.LrTable.T 2,(
ParserData.MlyValue.VOID,p1,p2))
fun ARROW (p1,p2) = Token.TOKEN (ParserData.LrTable.T 3,(
ParserData.MlyValue.VOID,p1,p2))
fun EQUAL (p1,p2) = Token.TOKEN (ParserData.LrTable.T 4,(
ParserData.MlyValue.VOID,p1,p2))
fun ARROWEQ (p1,p2) = Token.TOKEN (ParserData.LrTable.T 5,(
ParserData.MlyValue.VOID,p1,p2))
fun JOIN (p1,p2) = Token.TOKEN (ParserData.LrTable.T 6,(
ParserData.MlyValue.VOID,p1,p2))
fun BAR (p1,p2) = Token.TOKEN (ParserData.LrTable.T 7,(
ParserData.MlyValue.VOID,p1,p2))
fun COMMA (p1,p2) = Token.TOKEN (ParserData.LrTable.T 8,(
ParserData.MlyValue.VOID,p1,p2))
fun COLON (p1,p2) = Token.TOKEN (ParserData.LrTable.T 9,(
ParserData.MlyValue.VOID,p1,p2))
fun DQUOTE (p1,p2) = Token.TOKEN (ParserData.LrTable.T 10,(
ParserData.MlyValue.VOID,p1,p2))
fun CONTEXTSENSITIVE (p1,p2) = Token.TOKEN (ParserData.LrTable.T 11,(
ParserData.MlyValue.VOID,p1,p2))
fun EQUATIONS (p1,p2) = Token.TOKEN (ParserData.LrTable.T 12,(
ParserData.MlyValue.VOID,p1,p2))
fun INNERMOST (p1,p2) = Token.TOKEN (ParserData.LrTable.T 13,(
ParserData.MlyValue.VOID,p1,p2))
fun OUTERMOST (p1,p2) = Token.TOKEN (ParserData.LrTable.T 14,(
ParserData.MlyValue.VOID,p1,p2))
fun RULES (p1,p2) = Token.TOKEN (ParserData.LrTable.T 15,(
ParserData.MlyValue.VOID,p1,p2))
fun STRATEGY (p1,p2) = Token.TOKEN (ParserData.LrTable.T 16,(
ParserData.MlyValue.VOID,p1,p2))
fun THEORY (p1,p2) = Token.TOKEN (ParserData.LrTable.T 17,(
ParserData.MlyValue.VOID,p1,p2))
fun VAR (p1,p2) = Token.TOKEN (ParserData.LrTable.T 18,(
ParserData.MlyValue.VOID,p1,p2))
fun FUN (p1,p2) = Token.TOKEN (ParserData.LrTable.T 19,(
ParserData.MlyValue.VOID,p1,p2))
fun EOF (p1,p2) = Token.TOKEN (ParserData.LrTable.T 20,(
ParserData.MlyValue.VOID,p1,p2))
end
end
