(******************************************************************************
 * file: rwtools/commands/spec.lex
 * description: lexer for specification file of provers
 * author: AOTO Takahito
 * 
 ******************************************************************************)

structure Tokens = Tokens;
type pos = int;
type svalue = Tokens.svalue;
type ('a,'b) token = ('a,'b) Tokens.token;
type lexresult = (svalue, pos) token;

val pos = ref 1;
val eof = fn () => (pos:=1; Tokens.EOF(!pos,!pos));

%%
%header (functor SpecLexFun (structure Tokens: Spec_TOKENS));
%s COMMENT;
%%
\n	     => (pos:=(!pos)+1; lex());
[\ \t]+	     => (lex() (* ϥå *));
\;    	     => (Tokens.SEMICOLON(!pos,!pos));
\:    	     => (Tokens.COLON(!pos,!pos));
\=    	     => (Tokens.EQ(!pos,!pos));
\+    	     => (Tokens.PLUS(!pos,!pos));
\*    	     => (Tokens.TIMES(!pos,!pos));
\->    	     => (Tokens.THINARROW(!pos,!pos));
\(    	     => (Tokens.LPAREN(!pos,!pos));
\,    	     => (Tokens.COM(!pos,!pos));
\)    	     => (Tokens.RPAREN(!pos,!pos));
\?    	     => (Tokens.QUESTION(!pos,!pos));
\[    	     => (Tokens.LBRACKET(!pos,!pos));
\]    	     => (Tokens.RBRACKET(!pos,!pos));
SORTS        => (Tokens.SORTS(!pos,!pos));
RULES        => (Tokens.RULES(!pos,!pos));
EQUATIONS    => (Tokens.EQUATIONS(!pos,!pos));
CONJECTURES  => (Tokens.CONJECTURES(!pos,!pos));
PRECEDENCE   => (Tokens.PRECEDENCE(!pos,!pos));
POLYNOMIALS  => (Tokens.POLYNOMIALS(!pos,!pos));
KBOSPECIFICATION => (Tokens.KBOSPECIFICATION(!pos,!pos));
[a-zA-Z][a-zA-Z0-9]* => (Tokens.ID(yytext,!pos,!pos));
[0-9]+	     => (Tokens.ID(yytext,!pos,!pos));
.      	     => (print ("Warning: ignoring bad character: " 
    	    	    	    	  ^ yytext ^ "at line(" ^ (Int.toString (!pos))
    	    	    	    	  ^ ")\n"); lex());
