functor TpdbLrValsFun(structure Token : TOKEN)
 : sig structure ParserData : PARSER_DATA
       structure Tokens : Tpdb_TOKENS
   end
 = 
struct
structure ParserData=
struct
structure Header = 
struct
(******************************************************************************
 * Copyright (c) 2012-2013, Toyama&Aoto Laboratory, Tohoku University
 * All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without 
 * modification, are permitted provided that the following conditions are met:
 * 
 *  1. Redistributions of source code must retain the above copyright notice, 
 *     this list of conditions and the following disclaimer.
 *  2. Redistributions in binary form must reproduce the above copyright 
 *     notice, this list of conditions and the following disclaimer in the 
 *     documentation and/or other materials provided with the distribution.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS IS" 
 * AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE 
 * IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE 
 * ARE DISCLAIMED. IN NO EVENT SHALL THE COPYRIGHT HOLDER OR CONTRIBUTORS BE 
 * LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR 
 * CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF 
 * SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS 
 * INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN 
 * CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) 
 * ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE 
 * POSSIBILITY OF SUCH DAMAGE.
 ******************************************************************************)
(******************************************************************************
 * file: rwtools/commands/tpdb.yacc
 * description: parser for TPDB (Termination Problem Data Base) format
 * author: AOTO Takahito
 * 
 ******************************************************************************)

local 
    open Term
    structure L = List
    structure LU = ListUtil
    structure PU = PrintUtil
in
datatype tpdbExpr = VAR of string list 
      		  | RULES of (term * term) option list
		  | THEORY of (string * fun_key list) list

datatype tpdb_thdecl = THEORIES of string * string list 
                     | EQUATIONS of (term * term) list

datatype tpdb_cond = ORIENT of (term * term)
                   | JOIN of (term * term)

datatype tpdb_rule = NORMAL of (term * term * tpdb_cond list)
                   | RELATIVE of (term * term * tpdb_cond list)

datatype tpdb_strat = STRAT_INNERMOST
                    | STRAT_OUTERMOST
                    | STRAT_CS of (fun_key * int list) list

datatype tpdb_decl = VAR of Atom.atom list 
		   | THEORY of tpdb_thdecl list
		   | CONDITIONTYPE of string
                   | RULES of tpdb_rule list 
                   | STRATEGY of tpdb_strat
                   | REST of string list

exception TpdbError of string

fun getCategory f xss = let val yss = List.mapPartial f xss 
			in if null yss 
			   then NONE
			   else SOME (LU.mapAppend (fn x=>x) yss)
			end
val getVAR = getCategory (fn top => case top of VAR xs => SOME xs | _ => NONE) 
val getRULES = getCategory (fn top => case top of RULES xs => SOME xs | _ => NONE)
val getTHEORY = getCategory (fn top => case top of THEORY xs => SOME xs | _ => NONE)
val getCONDITIONTYPE = getCategory (fn top => case top of CONDITIONTYPE xs => SOME [xs] | _ => NONE)
val getREST = getCategory (fn top => case top of REST xs => SOME xs | _ => NONE)

(* for the stragegy, only the fist one is considerd *)
fun getSTRATEGY (top::rest) = (case top of STRATEGY xs => SOME xs | _ => getSTRATEGY rest)
  | getSTRATEGY [] = NONE
						  
fun makeTrs vars rules = 
    let fun transTerm (t as (Fun (y,[],_))) = 
	    if ListUtil.member' Fun.equal y vars
	    then (Var ((y,0), Sort.null)) 
	    else t
	  | transTerm (Fun (y,ys,_)) = Fun (y, List.map transTerm ys, Sort.null)
    in
	List.map (fn (l,r) => (transTerm l, transTerm r)) rules
    end

local
    exception NonSupportedRule
in
fun checkRules xs ys = 
    let	fun getNormal ys = List.mapPartial 
                              (fn y => case y of 
                                NORMAL (l,r,xs) => 
    if null xs
                          then SOME (l,r)
                          else raise NonSupportedRule
                                | _ => NONE) ys
       fun getRelative ys = List.mapPartial 
                            (fn y => case y of 
                                RELATIVE (l,r,xs) => 
                          if null xs
                          then SOME (l,r)
                          else raise NonSupportedRule
                                | _ => NONE) ys
    in SOME (makeTrs xs (getNormal ys),
             makeTrs xs (getRelative ys))
    end
    handle NonSupportedRule => 
                 (PU.print "CONDITIONAL rules are not supported.\n"; 
                  raise (TpdbError "unsupported format"))
end

fun getRules decls =
    case (getVAR decls, getRULES decls) of
	(SOME xs, SOME ys) => checkRules xs ys
      | (NONE, SOME ys) => checkRules [] ys
      | (SOME xs,NONE) => checkRules xs []
      | (NONE,NONE) => checkRules [] []

fun checkDecl decls = 
    (if isSome (getTHEORY decls)
    then (PU.print "THEORY is not supported.\n"; 
          raise (TpdbError "unsupported format"))
    else if isSome (getCONDITIONTYPE decls)
    then (PU.print "CONDITIONTYPE is not supported.\n"; 
          raise (TpdbError "unsupported format"))
    else if isSome (getSTRATEGY decls)
    then (PU.print "STRATEGY is not supported.\n"; 
          raise (TpdbError "unsupported format"))
    else (case getREST decls of
	     SOME xs => (PU.print ("(ignored inputs)" ^  
                         (ListUtil.toStringSpace (fn x=>x) xs) ^ "\n");
			 getRules decls)
	   | NONE => getRules decls))
    handle TpdbError _ => NONE

end

(* 桼 *)

end
structure LrTable = Token.LrTable
structure Token = Token
local open LrTable in 
val table=let val actionRows =
"\
\\001\000\001\000\011\000\011\000\010\000\017\000\009\000\018\000\008\000\
\\019\000\007\000\020\000\006\000\000\000\
\\001\000\001\000\024\000\000\000\
\\001\000\001\000\024\000\003\000\084\000\000\000\
\\001\000\001\000\025\000\000\000\
\\001\000\001\000\049\000\014\000\048\000\000\000\
\\001\000\001\000\079\000\000\000\
\\001\000\003\000\012\000\000\000\
\\001\000\003\000\074\000\000\000\
\\001\000\003\000\086\000\000\000\
\\001\000\003\000\095\000\000\000\
\\001\000\003\000\100\000\000\000\
\\001\000\004\000\053\000\007\000\052\000\000\000\
\\001\000\004\000\109\000\006\000\108\000\008\000\107\000\000\000\
\\001\000\005\000\088\000\000\000\
\\001\000\012\000\085\000\000\000\
\\001\000\013\000\020\000\015\000\019\000\016\000\018\000\000\000\
\\001\000\021\000\000\000\000\000\
\\116\000\000\000\
\\117\000\000\000\
\\118\000\002\000\004\000\000\000\
\\119\000\000\000\
\\120\000\000\000\
\\121\000\000\000\
\\122\000\000\000\
\\123\000\000\000\
\\124\000\000\000\
\\125\000\000\000\
\\126\000\000\000\
\\127\000\000\000\
\\128\000\000\000\
\\129\000\000\000\
\\130\000\000\000\
\\131\000\000\000\
\\132\000\000\000\
\\133\000\000\000\
\\134\000\000\000\
\\135\000\000\000\
\\136\000\000\000\
\\137\000\000\000\
\\138\000\000\000\
\\139\000\000\000\
\\140\000\000\000\
\\141\000\000\000\
\\142\000\001\000\044\000\002\000\043\000\004\000\042\000\005\000\041\000\
\\007\000\040\000\008\000\039\000\009\000\038\000\010\000\037\000\
\\012\000\036\000\013\000\035\000\014\000\034\000\015\000\033\000\
\\016\000\032\000\017\000\031\000\018\000\030\000\019\000\029\000\
\\020\000\028\000\000\000\
\\143\000\000\000\
\\144\000\000\000\
\\145\000\001\000\014\000\000\000\
\\146\000\000\000\
\\147\000\002\000\016\000\000\000\
\\148\000\000\000\
\\149\000\000\000\
\\150\000\000\000\
\\151\000\001\000\024\000\000\000\
\\152\000\000\000\
\\153\000\000\000\
\\154\000\001\000\024\000\000\000\
\\155\000\009\000\094\000\000\000\
\\156\000\000\000\
\\157\000\009\000\093\000\000\000\
\\158\000\000\000\
\\159\000\010\000\110\000\000\000\
\\160\000\000\000\
\\161\000\000\000\
\\162\000\000\000\
\\163\000\000\000\
\\164\000\002\000\055\000\000\000\
\\165\000\000\000\
\\166\000\000\000\
\\167\000\000\000\
\\168\000\010\000\096\000\000\000\
\\169\000\000\000\
\\170\000\000\000\
\\171\000\000\000\
\\172\000\000\000\
\\173\000\002\000\051\000\000\000\
\\174\000\000\000\
\\175\000\001\000\092\000\000\000\
\\176\000\000\000\
\"
val actionRowNumbers =
"\019\000\017\000\000\000\006\000\
\\046\000\048\000\015\000\055\000\
\\003\000\043\000\019\000\020\000\
\\046\000\021\000\004\000\024\000\
\\071\000\070\000\074\000\011\000\
\\055\000\023\000\065\000\022\000\
\\043\000\025\000\043\000\043\000\
\\043\000\043\000\043\000\043\000\
\\043\000\043\000\046\000\043\000\
\\043\000\043\000\043\000\043\000\
\\043\000\043\000\043\000\018\000\
\\045\000\007\000\052\000\046\000\
\\072\000\005\000\001\000\001\000\
\\054\000\002\000\027\000\035\000\
\\042\000\041\000\036\000\040\000\
\\039\000\038\000\037\000\014\000\
\\034\000\032\000\029\000\033\000\
\\031\000\030\000\008\000\026\000\
\\048\000\013\000\052\000\050\000\
\\049\000\076\000\058\000\056\000\
\\009\000\069\000\066\000\044\000\
\\043\000\047\000\001\000\051\000\
\\076\000\010\000\077\000\001\000\
\\001\000\067\000\001\000\028\000\
\\053\000\075\000\074\000\012\000\
\\060\000\059\000\057\000\068\000\
\\073\000\001\000\001\000\001\000\
\\001\000\063\000\064\000\062\000\
\\061\000\016\000"
val gotoT =
"\
\\001\000\113\000\002\000\001\000\000\000\
\\000\000\
\\003\000\003\000\000\000\
\\000\000\
\\006\000\011\000\000\000\
\\007\000\013\000\000\000\
\\017\000\015\000\000\000\
\\011\000\021\000\012\000\020\000\015\000\019\000\000\000\
\\000\000\
\\004\000\025\000\005\000\024\000\000\000\
\\002\000\043\000\000\000\
\\000\000\
\\006\000\044\000\000\000\
\\000\000\
\\008\000\045\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\018\000\048\000\000\000\
\\000\000\
\\011\000\052\000\012\000\020\000\015\000\019\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\004\000\054\000\005\000\024\000\000\000\
\\000\000\
\\004\000\055\000\005\000\024\000\000\000\
\\004\000\056\000\005\000\024\000\000\000\
\\004\000\057\000\005\000\024\000\000\000\
\\004\000\058\000\005\000\024\000\000\000\
\\004\000\059\000\005\000\024\000\000\000\
\\004\000\060\000\005\000\024\000\000\000\
\\004\000\061\000\005\000\024\000\000\000\
\\004\000\062\000\005\000\024\000\000\000\
\\006\000\063\000\000\000\
\\004\000\064\000\005\000\024\000\000\000\
\\004\000\065\000\005\000\024\000\000\000\
\\004\000\066\000\005\000\024\000\000\000\
\\004\000\067\000\005\000\024\000\000\000\
\\004\000\068\000\005\000\024\000\000\000\
\\004\000\069\000\005\000\024\000\000\000\
\\004\000\070\000\005\000\024\000\000\000\
\\004\000\071\000\005\000\024\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\009\000\075\000\010\000\074\000\015\000\073\000\000\000\
\\006\000\076\000\000\000\
\\000\000\
\\000\000\
\\015\000\078\000\000\000\
\\015\000\079\000\000\000\
\\000\000\
\\015\000\081\000\016\000\080\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\007\000\085\000\000\000\
\\000\000\
\\009\000\087\000\010\000\074\000\015\000\073\000\000\000\
\\000\000\
\\000\000\
\\019\000\089\000\020\000\088\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\004\000\095\000\005\000\024\000\000\000\
\\000\000\
\\015\000\096\000\000\000\
\\000\000\
\\019\000\097\000\020\000\088\000\000\000\
\\000\000\
\\000\000\
\\013\000\101\000\014\000\100\000\015\000\099\000\000\000\
\\013\000\102\000\014\000\100\000\015\000\099\000\000\000\
\\000\000\
\\015\000\081\000\016\000\103\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\018\000\104\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\015\000\109\000\000\000\
\\015\000\110\000\000\000\
\\015\000\111\000\000\000\
\\013\000\112\000\014\000\100\000\015\000\099\000\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\\000\000\
\"
val numstates = 114
val numrules = 61
val s = ref "" and index = ref 0
val string_to_int = fn () => 
let val i = !index
in index := i+2; Char.ord(String.sub(!s,i)) + Char.ord(String.sub(!s,i+1)) * 256
end
val string_to_list = fn s' =>
    let val len = String.size s'
        fun f () =
           if !index < len then string_to_int() :: f()
           else nil
   in index := 0; s := s'; f ()
   end
val string_to_pairlist = fn (conv_key,conv_entry) =>
     let fun f () =
         case string_to_int()
         of 0 => EMPTY
          | n => PAIR(conv_key (n-1),conv_entry (string_to_int()),f())
     in f
     end
val string_to_pairlist_default = fn (conv_key,conv_entry) =>
    let val conv_row = string_to_pairlist(conv_key,conv_entry)
    in fn () =>
       let val default = conv_entry(string_to_int())
           val row = conv_row()
       in (row,default)
       end
   end
val string_to_table = fn (convert_row,s') =>
    let val len = String.size s'
        fun f ()=
           if !index < len then convert_row() :: f()
           else nil
     in (s := s'; index := 0; f ())
     end
local
  val memo = Array.array(numstates+numrules,ERROR)
  val _ =let fun g i=(Array.update(memo,i,REDUCE(i-numstates)); g(i+1))
       fun f i =
            if i=numstates then g i
            else (Array.update(memo,i,SHIFT (STATE i)); f (i+1))
          in f 0 handle General.Subscript => ()
          end
in
val entry_to_action = fn 0 => ACCEPT | 1 => ERROR | j => Array.sub(memo,(j-2))
end
val gotoT=Array.fromList(string_to_table(string_to_pairlist(NT,STATE),gotoT))
val actionRows=string_to_table(string_to_pairlist_default(T,entry_to_action),actionRows)
val actionRowNumbers = string_to_list actionRowNumbers
val actionT = let val actionRowLookUp=
let val a=Array.fromList(actionRows) in fn i=>Array.sub(a,i) end
in Array.fromList(List.map actionRowLookUp actionRowNumbers)
end
in LrTable.mkLrTable {actions=actionT,gotos=gotoT,numRules=numrules,
numStates=numstates,initialState=STATE 0}
end
end
local open Header in
type pos = int
type arg = unit
structure MlyValue = 
struct
datatype svalue = VOID | ntVOID of unit ->  unit
 | ID of unit ->  (string) | XintX of unit ->  (int)
 | XintlistX of unit ->  (int list)
 | XcsstratlistX of unit ->  ( ( Fun.ord_key * int list )  list)
 | XstrategydeclX of unit ->  (tpdb_strat)
 | XtermlistX of unit ->  (Term.term list)
 | XtermX of unit ->  (Term.term) | XcondX of unit ->  (tpdb_cond)
 | XcondlistX of unit ->  (tpdb_cond list)
 | XruleX of unit ->  (tpdb_rule)
 | XlistofrulesX of unit ->  (tpdb_rule list)
 | XequationX of unit ->  (Term.term*Term.term)
 | XeqlistX of unit ->  ( ( Term.term * Term.term )  list)
 | XthdeclX of unit ->  (tpdb_thdecl)
 | XlistofthdeclX of unit ->  (tpdb_thdecl list)
 | XidlistX of unit ->  (string list)
 | XstringX of unit ->  (string list)
 | XanylistX of unit ->  (string list)
 | XdeclX of unit ->  (tpdb_decl)
 | XspecX of unit ->  (tpdb_decl list)
 | XstartX of unit ->  ( ( (Term.term * Term.term) list * (Term.term * Term.term) list )  option)
end
type svalue = MlyValue.svalue
type result = 
 ( (Term.term * Term.term) list * (Term.term * Term.term) list )  option
end
structure EC=
struct
open LrTable
infix 5 $$
fun x $$ y = y::x
val is_keyword =
fn _ => false
val preferred_change : (term list * term list) list = 
nil
val noShift = 
fn (T 20) => true | _ => false
val showTerminal =
fn (T 0) => "ID"
  | (T 1) => "LPAREN"
  | (T 2) => "RPAREN"
  | (T 3) => "ARROW"
  | (T 4) => "EQUAL"
  | (T 5) => "DEQS"
  | (T 6) => "ARROWEQ"
  | (T 7) => "JOIN"
  | (T 8) => "BAR"
  | (T 9) => "COMMA"
  | (T 10) => "CONDITIONTYPE"
  | (T 11) => "DQUOTE"
  | (T 12) => "CONTEXTSENSITIVE"
  | (T 13) => "EQUATIONS"
  | (T 14) => "INNERMOST"
  | (T 15) => "OUTERMOST"
  | (T 16) => "RULES"
  | (T 17) => "STRATEGY"
  | (T 18) => "THEORY"
  | (T 19) => "VAR"
  | (T 20) => "EOF"
  | _ => "bogus-term"
local open Header in
val errtermvalue=
fn _ => MlyValue.VOID
end
val terms : term list = nil
 $$ (T 20) $$ (T 19) $$ (T 18) $$ (T 17) $$ (T 16) $$ (T 15) $$ (T 14)
 $$ (T 13) $$ (T 12) $$ (T 11) $$ (T 10) $$ (T 9) $$ (T 8) $$ (T 7)
 $$ (T 6) $$ (T 5) $$ (T 4) $$ (T 3) $$ (T 2) $$ (T 1)end
structure Actions =
struct 
exception mlyAction of int
local open Header in
val actions = 
fn (i392,defaultPos,stack,
    (()):arg) =>
case (i392,stack)
of  ( 0, ( ( _, ( MlyValue.XspecX XspecX1, XspecX1left, XspecX1right))
 :: rest671)) => let val  result = MlyValue.XstartX (fn _ => let val 
 (XspecX as XspecX1) = XspecX1 ()
 in (checkDecl (XspecX))
end)
 in ( LrTable.NT 0, ( result, XspecX1left, XspecX1right), rest671)
end
|  ( 1, ( ( _, ( MlyValue.XspecX XspecX1, _, XspecX1right)) :: _ :: (
 _, ( MlyValue.XdeclX XdeclX1, _, _)) :: ( _, ( _, LPAREN1left, _)) ::
 rest671)) => let val  result = MlyValue.XspecX (fn _ => let val  (
XdeclX as XdeclX1) = XdeclX1 ()
 val  (XspecX as XspecX1) = XspecX1 ()
 in (XdeclX::XspecX)
end)
 in ( LrTable.NT 1, ( result, LPAREN1left, XspecX1right), rest671)
end
|  ( 2, ( rest671)) => let val  result = MlyValue.XspecX (fn _ => ([])
)
 in ( LrTable.NT 1, ( result, defaultPos, defaultPos), rest671)
end
|  ( 3, ( ( _, ( MlyValue.XidlistX XidlistX1, _, XidlistX1right)) :: (
 _, ( _, VAR1left, _)) :: rest671)) => let val  result = 
MlyValue.XdeclX (fn _ => let val  (XidlistX as XidlistX1) = XidlistX1
 ()
 in (VAR (List.map Atom.atom (XidlistX)))
end)
 in ( LrTable.NT 2, ( result, VAR1left, XidlistX1right), rest671)
end
|  ( 4, ( ( _, ( MlyValue.XlistofthdeclX XlistofthdeclX1, _, 
XlistofthdeclX1right)) :: ( _, ( _, THEORY1left, _)) :: rest671)) =>
 let val  result = MlyValue.XdeclX (fn _ => let val  (XlistofthdeclX
 as XlistofthdeclX1) = XlistofthdeclX1 ()
 in (THEORY XlistofthdeclX)
end)
 in ( LrTable.NT 2, ( result, THEORY1left, XlistofthdeclX1right), 
rest671)
end
|  ( 5, ( ( _, ( MlyValue.ID ID1, _, ID1right)) :: ( _, ( _, 
CONDITIONTYPE1left, _)) :: rest671)) => let val  result = 
MlyValue.XdeclX (fn _ => let val  (ID as ID1) = ID1 ()
 in (CONDITIONTYPE ID)
end)
 in ( LrTable.NT 2, ( result, CONDITIONTYPE1left, ID1right), rest671)

end
|  ( 6, ( ( _, ( MlyValue.XlistofrulesX XlistofrulesX1, _, 
XlistofrulesX1right)) :: ( _, ( _, RULES1left, _)) :: rest671)) => let
 val  result = MlyValue.XdeclX (fn _ => let val  (XlistofrulesX as 
XlistofrulesX1) = XlistofrulesX1 ()
 in (RULES XlistofrulesX)
end)
 in ( LrTable.NT 2, ( result, RULES1left, XlistofrulesX1right), 
rest671)
end
|  ( 7, ( ( _, ( MlyValue.XstrategydeclX XstrategydeclX1, _, 
XstrategydeclX1right)) :: ( _, ( _, STRATEGY1left, _)) :: rest671)) =>
 let val  result = MlyValue.XdeclX (fn _ => let val  (XstrategydeclX
 as XstrategydeclX1) = XstrategydeclX1 ()
 in (STRATEGY XstrategydeclX)
end)
 in ( LrTable.NT 2, ( result, STRATEGY1left, XstrategydeclX1right), 
rest671)
end
|  ( 8, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  
result = MlyValue.XdeclX (fn _ => let val  (ID as ID1) = ID1 ()
 val  (XanylistX as XanylistX1) = XanylistX1 ()
 in (REST (ID::XanylistX))
end)
 in ( LrTable.NT 2, ( result, ID1left, XanylistX1right), rest671)
end
|  ( 9, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  
result = MlyValue.XanylistX (fn _ => let val  (ID as ID1) = ID1 ()
 val  (XanylistX as XanylistX1) = XanylistX1 ()
 in (ID::XanylistX)
end)
 in ( LrTable.NT 3, ( result, ID1left, XanylistX1right), rest671)
end
|  ( 10, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( MlyValue.XstringX XstringX1, XstringX1left, _)) :: rest671)
) => let val  result = MlyValue.XanylistX (fn _ => let val  (XstringX
 as XstringX1) = XstringX1 ()
 val  (XanylistX as XanylistX1) = XanylistX1 ()
 in (XstringX @ XanylistX)
end)
 in ( LrTable.NT 3, ( result, XstringX1left, XanylistX1right), rest671
)
end
|  ( 11, ( ( _, ( MlyValue.XanylistX XanylistX2, _, XanylistX2right))
 :: _ :: ( _, ( MlyValue.XanylistX XanylistX1, _, _)) :: ( _, ( _, 
LPAREN1left, _)) :: rest671)) => let val  result = MlyValue.XanylistX
 (fn _ => let val  XanylistX1 = XanylistX1 ()
 val  XanylistX2 = XanylistX2 ()
 in (("("::XanylistX1) @ (")"::XanylistX2))
end)
 in ( LrTable.NT 3, ( result, LPAREN1left, XanylistX2right), rest671)

end
|  ( 12, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, JOIN1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("-><-"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, JOIN1left, XanylistX1right), rest671)

end
|  ( 13, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, ARROW1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("->"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, ARROW1left, XanylistX1right), rest671)

end
|  ( 14, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, EQUAL1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("="::XanylistX)
end)
 in ( LrTable.NT 3, ( result, EQUAL1left, XanylistX1right), rest671)

end
|  ( 15, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, BAR1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("|"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, BAR1left, XanylistX1right), rest671)
end
|  ( 16, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, ARROWEQ1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("->="::XanylistX)
end)
 in ( LrTable.NT 3, ( result, ARROWEQ1left, XanylistX1right), rest671)

end
|  ( 17, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, COMMA1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in (","::XanylistX)
end)
 in ( LrTable.NT 3, ( result, COMMA1left, XanylistX1right), rest671)

end
|  ( 18, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, VAR1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("VAR"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, VAR1left, XanylistX1right), rest671)
end
|  ( 19, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, RULES1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("RULES"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, RULES1left, XanylistX1right), rest671)

end
|  ( 20, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, CONTEXTSENSITIVE1left, _)) :: rest671)) => let val  
result = MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1
) = XanylistX1 ()
 in ("CONTEXTSENSITIVE"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, CONTEXTSENSITIVE1left, XanylistX1right),
 rest671)
end
|  ( 21, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, EQUATIONS1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("EQUATIONS"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, EQUATIONS1left, XanylistX1right), 
rest671)
end
|  ( 22, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, INNERMOST1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("INNERMOST"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, INNERMOST1left, XanylistX1right), 
rest671)
end
|  ( 23, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, OUTERMOST1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("OUTERMOST"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, OUTERMOST1left, XanylistX1right), 
rest671)
end
|  ( 24, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, STRATEGY1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("STRATEGY"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, STRATEGY1left, XanylistX1right), rest671
)
end
|  ( 25, ( ( _, ( MlyValue.XanylistX XanylistX1, _, XanylistX1right))
 :: ( _, ( _, THEORY1left, _)) :: rest671)) => let val  result = 
MlyValue.XanylistX (fn _ => let val  (XanylistX as XanylistX1) = 
XanylistX1 ()
 in ("THEORY"::XanylistX)
end)
 in ( LrTable.NT 3, ( result, THEORY1left, XanylistX1right), rest671)

end
|  ( 26, ( rest671)) => let val  result = MlyValue.XanylistX (fn _ =>
 ([]))
 in ( LrTable.NT 3, ( result, defaultPos, defaultPos), rest671)
end
|  ( 27, ( ( _, ( _, _, DQUOTE2right)) :: ( _, ( MlyValue.XidlistX 
XidlistX1, _, _)) :: ( _, ( _, DQUOTE1left, _)) :: rest671)) => let
 val  result = MlyValue.XstringX (fn _ => let val  (XidlistX as 
XidlistX1) = XidlistX1 ()
 in (XidlistX)
end)
 in ( LrTable.NT 4, ( result, DQUOTE1left, DQUOTE2right), rest671)
end
|  ( 28, ( ( _, ( MlyValue.XidlistX XidlistX1, _, XidlistX1right)) :: 
( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  result =
 MlyValue.XidlistX (fn _ => let val  (ID as ID1) = ID1 ()
 val  (XidlistX as XidlistX1) = XidlistX1 ()
 in (ID::XidlistX)
end)
 in ( LrTable.NT 5, ( result, ID1left, XidlistX1right), rest671)
end
|  ( 29, ( rest671)) => let val  result = MlyValue.XidlistX (fn _ => (
[]))
 in ( LrTable.NT 5, ( result, defaultPos, defaultPos), rest671)
end
|  ( 30, ( ( _, ( MlyValue.XlistofthdeclX XlistofthdeclX1, _, 
XlistofthdeclX1right)) :: _ :: ( _, ( MlyValue.XthdeclX XthdeclX1, _,
 _)) :: ( _, ( _, LPAREN1left, _)) :: rest671)) => let val  result = 
MlyValue.XlistofthdeclX (fn _ => let val  (XthdeclX as XthdeclX1) = 
XthdeclX1 ()
 val  (XlistofthdeclX as XlistofthdeclX1) = XlistofthdeclX1 ()
 in (XthdeclX::XlistofthdeclX)
end)
 in ( LrTable.NT 6, ( result, LPAREN1left, XlistofthdeclX1right), 
rest671)
end
|  ( 31, ( rest671)) => let val  result = MlyValue.XlistofthdeclX (fn
 _ => ([]))
 in ( LrTable.NT 6, ( result, defaultPos, defaultPos), rest671)
end
|  ( 32, ( ( _, ( MlyValue.XidlistX XidlistX1, _, XidlistX1right)) :: 
( _, ( MlyValue.ID ID1, ID1left, _)) :: rest671)) => let val  result =
 MlyValue.XthdeclX (fn _ => let val  (ID as ID1) = ID1 ()
 val  (XidlistX as XidlistX1) = XidlistX1 ()
 in (THEORIES (ID,XidlistX))
end)
 in ( LrTable.NT 7, ( result, ID1left, XidlistX1right), rest671)
end
|  ( 33, ( ( _, ( MlyValue.XeqlistX XeqlistX1, _, XeqlistX1right)) :: 
( _, ( _, EQUATIONS1left, _)) :: rest671)) => let val  result = 
MlyValue.XthdeclX (fn _ => let val  (XeqlistX as XeqlistX1) = 
XeqlistX1 ()
 in (EQUATIONS XeqlistX)
end)
 in ( LrTable.NT 7, ( result, EQUATIONS1left, XeqlistX1right), rest671
)
end
|  ( 34, ( ( _, ( MlyValue.XeqlistX XeqlistX1, _, XeqlistX1right)) :: 
( _, ( MlyValue.XequationX XequationX1, XequationX1left, _)) :: 
rest671)) => let val  result = MlyValue.XeqlistX (fn _ => let val  (
XequationX as XequationX1) = XequationX1 ()
 val  (XeqlistX as XeqlistX1) = XeqlistX1 ()
 in (XequationX::XeqlistX)
end)
 in ( LrTable.NT 8, ( result, XequationX1left, XeqlistX1right), 
rest671)
end
|  ( 35, ( rest671)) => let val  result = MlyValue.XeqlistX (fn _ => (
[]))
 in ( LrTable.NT 8, ( result, defaultPos, defaultPos), rest671)
end
|  ( 36, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XequationX (fn _ => let val  XtermX1 = XtermX1
 ()
 val  XtermX2 = XtermX2 ()
 in ((XtermX1,XtermX2))
end)
 in ( LrTable.NT 9, ( result, XtermX1left, XtermX2right), rest671)
end
|  ( 37, ( ( _, ( MlyValue.XlistofrulesX XlistofrulesX1, _, 
XlistofrulesX1right)) :: ( _, ( MlyValue.XruleX XruleX1, XruleX1left,
 _)) :: rest671)) => let val  result = MlyValue.XlistofrulesX (fn _ =>
 let val  (XruleX as XruleX1) = XruleX1 ()
 val  (XlistofrulesX as XlistofrulesX1) = XlistofrulesX1 ()
 in (XruleX::XlistofrulesX)
end)
 in ( LrTable.NT 10, ( result, XruleX1left, XlistofrulesX1right), 
rest671)
end
|  ( 38, ( rest671)) => let val  result = MlyValue.XlistofrulesX (fn _
 => ([]))
 in ( LrTable.NT 10, ( result, defaultPos, defaultPos), rest671)
end
|  ( 39, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XruleX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (NORMAL (XtermX1,XtermX2,[]))
end)
 in ( LrTable.NT 11, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 40, ( ( _, ( MlyValue.XcondlistX XcondlistX1, _, XcondlistX1right
)) :: _ :: ( _, ( MlyValue.XtermX XtermX2, _, _)) :: _ :: ( _, ( 
MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let val  
result = MlyValue.XruleX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 val  (XcondlistX as XcondlistX1) = XcondlistX1 ()
 in (NORMAL(XtermX1,XtermX2,XcondlistX))
end)
 in ( LrTable.NT 11, ( result, XtermX1left, XcondlistX1right), rest671
)
end
|  ( 41, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XruleX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (RELATIVE (XtermX1,XtermX2,[]))
end)
 in ( LrTable.NT 11, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 42, ( ( _, ( MlyValue.XcondlistX XcondlistX1, _, XcondlistX1right
)) :: _ :: ( _, ( MlyValue.XtermX XtermX2, _, _)) :: _ :: ( _, ( 
MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let val  
result = MlyValue.XruleX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 val  (XcondlistX as XcondlistX1) = XcondlistX1 ()
 in (RELATIVE (XtermX1,XtermX2,XcondlistX))
end)
 in ( LrTable.NT 11, ( result, XtermX1left, XcondlistX1right), rest671
)
end
|  ( 43, ( ( _, ( MlyValue.XcondX XcondX1, XcondX1left, XcondX1right))
 :: rest671)) => let val  result = MlyValue.XcondlistX (fn _ => let
 val  (XcondX as XcondX1) = XcondX1 ()
 in ([XcondX])
end)
 in ( LrTable.NT 12, ( result, XcondX1left, XcondX1right), rest671)

end
|  ( 44, ( ( _, ( MlyValue.XcondlistX XcondlistX1, _, XcondlistX1right
)) :: _ :: ( _, ( MlyValue.XcondX XcondX1, XcondX1left, _)) :: rest671
)) => let val  result = MlyValue.XcondlistX (fn _ => let val  (XcondX
 as XcondX1) = XcondX1 ()
 val  (XcondlistX as XcondlistX1) = XcondlistX1 ()
 in (XcondX::XcondlistX)
end)
 in ( LrTable.NT 12, ( result, XcondX1left, XcondlistX1right), rest671
)
end
|  ( 45, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XcondX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (ORIENT (XtermX1,XtermX2))
end)
 in ( LrTable.NT 13, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 46, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XcondX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (JOIN (XtermX1,XtermX2))
end)
 in ( LrTable.NT 13, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 47, ( ( _, ( MlyValue.XtermX XtermX2, _, XtermX2right)) :: _ :: (
 _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671)) => let
 val  result = MlyValue.XcondX (fn _ => let val  XtermX1 = XtermX1 ()
 val  XtermX2 = XtermX2 ()
 in (JOIN (XtermX1,XtermX2))
end)
 in ( LrTable.NT 13, ( result, XtermX1left, XtermX2right), rest671)

end
|  ( 48, ( ( _, ( MlyValue.ID ID1, ID1left, ID1right)) :: rest671)) =>
 let val  result = MlyValue.XtermX (fn _ => let val  (ID as ID1) = ID1
 ()
 in (Term.Fun (Atom.atom ID,[],Sort.null))
end)
 in ( LrTable.NT 14, ( result, ID1left, ID1right), rest671)
end
|  ( 49, ( ( _, ( _, _, RPAREN1right)) :: _ :: ( _, ( MlyValue.ID ID1,
 ID1left, _)) :: rest671)) => let val  result = MlyValue.XtermX (fn _
 => let val  (ID as ID1) = ID1 ()
 in (Term.Fun (Atom.atom ID,[],Sort.null))
end)
 in ( LrTable.NT 14, ( result, ID1left, RPAREN1right), rest671)
end
|  ( 50, ( ( _, ( _, _, RPAREN1right)) :: ( _, ( MlyValue.XtermlistX 
XtermlistX1, _, _)) :: _ :: ( _, ( MlyValue.ID ID1, ID1left, _)) :: 
rest671)) => let val  result = MlyValue.XtermX (fn _ => let val  (ID
 as ID1) = ID1 ()
 val  (XtermlistX as XtermlistX1) = XtermlistX1 ()
 in (Term.Fun (Atom.atom ID,XtermlistX,Sort.null))
end)
 in ( LrTable.NT 14, ( result, ID1left, RPAREN1right), rest671)
end
|  ( 51, ( ( _, ( MlyValue.XtermlistX XtermlistX1, _, XtermlistX1right
)) :: _ :: ( _, ( MlyValue.XtermX XtermX1, XtermX1left, _)) :: rest671
)) => let val  result = MlyValue.XtermlistX (fn _ => let val  (XtermX
 as XtermX1) = XtermX1 ()
 val  (XtermlistX as XtermlistX1) = XtermlistX1 ()
 in (XtermX::XtermlistX)
end)
 in ( LrTable.NT 15, ( result, XtermX1left, XtermlistX1right), rest671
)
end
|  ( 52, ( ( _, ( MlyValue.XtermX XtermX1, XtermX1left, XtermX1right))
 :: rest671)) => let val  result = MlyValue.XtermlistX (fn _ => let
 val  (XtermX as XtermX1) = XtermX1 ()
 in ([XtermX])
end)
 in ( LrTable.NT 15, ( result, XtermX1left, XtermX1right), rest671)

end
|  ( 53, ( ( _, ( _, INNERMOST1left, INNERMOST1right)) :: rest671)) =>
 let val  result = MlyValue.XstrategydeclX (fn _ => (STRAT_INNERMOST))
 in ( LrTable.NT 16, ( result, INNERMOST1left, INNERMOST1right), 
rest671)
end
|  ( 54, ( ( _, ( _, OUTERMOST1left, OUTERMOST1right)) :: rest671)) =>
 let val  result = MlyValue.XstrategydeclX (fn _ => (STRAT_OUTERMOST))
 in ( LrTable.NT 16, ( result, OUTERMOST1left, OUTERMOST1right), 
rest671)
end
|  ( 55, ( ( _, ( MlyValue.XcsstratlistX XcsstratlistX1, _, 
XcsstratlistX1right)) :: ( _, ( _, CONTEXTSENSITIVE1left, _)) :: 
rest671)) => let val  result = MlyValue.XstrategydeclX (fn _ => let
 val  (XcsstratlistX as XcsstratlistX1) = XcsstratlistX1 ()
 in (STRAT_CS XcsstratlistX)
end)
 in ( LrTable.NT 16, ( result, CONTEXTSENSITIVE1left, 
XcsstratlistX1right), rest671)
end
|  ( 56, ( ( _, ( MlyValue.XcsstratlistX XcsstratlistX1, _, 
XcsstratlistX1right)) :: _ :: ( _, ( MlyValue.XintlistX XintlistX1, _,
 _)) :: ( _, ( MlyValue.ID ID1, _, _)) :: ( _, ( _, LPAREN1left, _))
 :: rest671)) => let val  result = MlyValue.XcsstratlistX (fn _ => let
 val  (ID as ID1) = ID1 ()
 val  (XintlistX as XintlistX1) = XintlistX1 ()
 val  (XcsstratlistX as XcsstratlistX1) = XcsstratlistX1 ()
 in ((Atom.atom ID,XintlistX)::XcsstratlistX)
end)
 in ( LrTable.NT 17, ( result, LPAREN1left, XcsstratlistX1right), 
rest671)
end
|  ( 57, ( rest671)) => let val  result = MlyValue.XcsstratlistX (fn _
 => ([]))
 in ( LrTable.NT 17, ( result, defaultPos, defaultPos), rest671)
end
|  ( 58, ( ( _, ( MlyValue.XintlistX XintlistX1, _, XintlistX1right))
 :: ( _, ( MlyValue.XintX XintX1, XintX1left, _)) :: rest671)) => let
 val  result = MlyValue.XintlistX (fn _ => let val  (XintX as XintX1)
 = XintX1 ()
 val  (XintlistX as XintlistX1) = XintlistX1 ()
 in (XintX::XintlistX)
end)
 in ( LrTable.NT 18, ( result, XintX1left, XintlistX1right), rest671)

end
|  ( 59, ( rest671)) => let val  result = MlyValue.XintlistX (fn _ =>
 ([]))
 in ( LrTable.NT 18, ( result, defaultPos, defaultPos), rest671)
end
|  ( 60, ( ( _, ( MlyValue.ID ID1, ID1left, ID1right)) :: rest671)) =>
 let val  result = MlyValue.XintX (fn _ => let val  (ID as ID1) = ID1
 ()
 in (valOf (Int.fromString ID))
end)
 in ( LrTable.NT 19, ( result, ID1left, ID1right), rest671)
end
| _ => raise (mlyAction i392)
end
val void = MlyValue.VOID
val extract = fn a => (fn MlyValue.XstartX x => x
| _ => let exception ParseInternal
	in raise ParseInternal end) a ()
end
end
structure Tokens : Tpdb_TOKENS =
struct
type svalue = ParserData.svalue
type ('a,'b) token = ('a,'b) Token.token
fun ID (i,p1,p2) = Token.TOKEN (ParserData.LrTable.T 0,(
ParserData.MlyValue.ID (fn () => i),p1,p2))
fun LPAREN (p1,p2) = Token.TOKEN (ParserData.LrTable.T 1,(
ParserData.MlyValue.VOID,p1,p2))
fun RPAREN (p1,p2) = Token.TOKEN (ParserData.LrTable.T 2,(
ParserData.MlyValue.VOID,p1,p2))
fun ARROW (p1,p2) = Token.TOKEN (ParserData.LrTable.T 3,(
ParserData.MlyValue.VOID,p1,p2))
fun EQUAL (p1,p2) = Token.TOKEN (ParserData.LrTable.T 4,(
ParserData.MlyValue.VOID,p1,p2))
fun DEQS (p1,p2) = Token.TOKEN (ParserData.LrTable.T 5,(
ParserData.MlyValue.VOID,p1,p2))
fun ARROWEQ (p1,p2) = Token.TOKEN (ParserData.LrTable.T 6,(
ParserData.MlyValue.VOID,p1,p2))
fun JOIN (p1,p2) = Token.TOKEN (ParserData.LrTable.T 7,(
ParserData.MlyValue.VOID,p1,p2))
fun BAR (p1,p2) = Token.TOKEN (ParserData.LrTable.T 8,(
ParserData.MlyValue.VOID,p1,p2))
fun COMMA (p1,p2) = Token.TOKEN (ParserData.LrTable.T 9,(
ParserData.MlyValue.VOID,p1,p2))
fun CONDITIONTYPE (p1,p2) = Token.TOKEN (ParserData.LrTable.T 10,(
ParserData.MlyValue.VOID,p1,p2))
fun DQUOTE (p1,p2) = Token.TOKEN (ParserData.LrTable.T 11,(
ParserData.MlyValue.VOID,p1,p2))
fun CONTEXTSENSITIVE (p1,p2) = Token.TOKEN (ParserData.LrTable.T 12,(
ParserData.MlyValue.VOID,p1,p2))
fun EQUATIONS (p1,p2) = Token.TOKEN (ParserData.LrTable.T 13,(
ParserData.MlyValue.VOID,p1,p2))
fun INNERMOST (p1,p2) = Token.TOKEN (ParserData.LrTable.T 14,(
ParserData.MlyValue.VOID,p1,p2))
fun OUTERMOST (p1,p2) = Token.TOKEN (ParserData.LrTable.T 15,(
ParserData.MlyValue.VOID,p1,p2))
fun RULES (p1,p2) = Token.TOKEN (ParserData.LrTable.T 16,(
ParserData.MlyValue.VOID,p1,p2))
fun STRATEGY (p1,p2) = Token.TOKEN (ParserData.LrTable.T 17,(
ParserData.MlyValue.VOID,p1,p2))
fun THEORY (p1,p2) = Token.TOKEN (ParserData.LrTable.T 18,(
ParserData.MlyValue.VOID,p1,p2))
fun VAR (p1,p2) = Token.TOKEN (ParserData.LrTable.T 19,(
ParserData.MlyValue.VOID,p1,p2))
fun EOF (p1,p2) = Token.TOKEN (ParserData.LrTable.T 20,(
ParserData.MlyValue.VOID,p1,p2))
end
end
