(* g_trans.sml
 *
 *)

signature G_TRANS = sig 

    val subtermsOfRules: (Term.term * Term.term) list -> TermSet.set
    val curry: Term.term -> Term.term
    val curryrule: (Term.term * Term.term) list -> (Term.term * Term.term) list
    val shallow: (Term.term * Term.term) list -> (Term.term * Term.term) list
    val plaisted: (Term.term * Term.term) list -> (Term.term * Term.term) list

end;

structure GTrans : G_TRANS = 
struct
   local 
       open Term
       structure L = List
       structure TS = TermSet
       structure TPS = TermPairSet
   in 
   
   exception GTransError;

   fun subtermsOfRules rs =
       List.foldl
	   (fn ((l,r),set) => TS.union (Term.setOfSubterms l,
					(TS.union (Term.setOfSubterms r,set))))
	   TS.empty
	   rs 

   (* ꡼Ѵ *)
   val appSymbol = Fun.fromString "@"

   fun curry (t as (Var _))  = t
     | curry (Fun (f,xs,_)) = 
       let
	   fun trans (t as (Var _)) = t
	     | trans (t as (Fun (f,[],_))) = t
	     | trans (Fun (f,t0::ts,_)) = 
	       Fun (appSymbol,
		    [trans (Fun (f,ts,Sort.null)), curry t0],
		    Sort.null)
       in 
	   trans (Fun (f,rev xs,Sort.null))
       end

   fun curryrule rs = map (fn (l,r) => (curry l,curry r)) rs


   fun shallow rs = 
       let (* 񤭴§κդ֤Ͽơ֥ *)
	   val initTable = L.foldl 
			   (fn ((l,r),tab) => TPS.add (TPS.add (tab,(l,l)),(r,r)))
			   TPS.empty
			   rs

	   (* depth 2 οʬ1 *)
	   fun selectDepth2Term (Var _) = NONE
	     | selectDepth2Term (t as (Fun (f,ts,_))) = 
	       case L.find (fn ti => Term.termDepth ti >= 2) ts of
		   SOME ti => if Term.termDepth t = 3
			      then SOME ti
			      else selectDepth2Term ti
		 | NONE => NONE

	   (* ʬ s 򤹤٤ c ֤ *)
	   fun replaceTerm (s,c) (t as (Var _)) = t
	     | replaceTerm (s,c) (t as (Fun (f,ts,ty))) = 
	       if Term.equal (s,t)
	       then c
	       else Fun (f, L.map (replaceTerm (s,c)) ts, ty)

	   (* ֤Ͽơ֥ *)
	   val constMap = ref TPS.empty

	   fun updateTable (s,c) table  = 
	       (constMap := TPS.add (!constMap, (c,s));
		TPS.map (fn (l,u) => (l, replaceTerm (s,c) u)) table)

	   (*  *)
	   val counter = ref 0 
	   fun newConst () = (counter := (!counter) + 1;
			      Fun (Atom.atom ("$c_" ^ (Int.toString (!counter))),[],Sort.null))

	   (* depth 3 ʾʬबޤޤʤʤޤǡơ֥򹹿 *)
	   fun saturateTable table = 
	       case TPS.find (fn (_,u) => Term.termDepth u > 2) table of
		   SOME (_,u) => (case selectDepth2Term u of
				      SOME s => saturateTable (updateTable (s,newConst ()) table)
				    | NONE => (print "shallow: unexpected error\n";
					       raise GTransError))
		 | NONE => table

	   val replacementTable = saturateTable initTable
       in
	   (L.map (fn (l,r) =>
		     (#2 (valOf (TPS.find (fn (s,_) => Term.equal (s,l)) replacementTable)),
		      #2 (valOf (TPS.find (fn (s,_) => Term.equal (s,r)) replacementTable))))
		 rs)
	   @ (TPS.foldr
		  (fn ((l,r),ss) => (l,r)::(r,l)::ss)
		  []
		  (!constMap))
       end


   fun plaisted rs = 
       let 
	   fun constOfTerm (Var _) = TS.empty
	     | constOfTerm (t as (Fun (_,[],_))) = TS.singleton t
	     | constOfTerm (Fun (f,t0::ts,ty)) =
	       L.foldl
		   (fn (ti,set) => TS.union (constOfTerm ti,set))
		   (constOfTerm t0)
		   ts

	   (* 񤭴§˽иν *)
	   val constSet = L.foldl 
			  (fn ((l,r),set) => TS.union (constOfTerm l,
						       TS.union (constOfTerm r,set)))
			  TS.empty
			  rs

	   (* 񤭴§˽иʬν *)
	   val subtermSet = L.foldl 
				(fn ((l,r),set) => TS.union (Term.setOfSubterms l,
							     TS.union (Term.setOfSubterms r,
								       set)))
				TS.empty
				rs

	   (* ադοǡ񤭴§ʬ䤹 *)
	   fun partitionRules rs = L.foldl
			      (fn ((l,r),(R11,R12,R21,R22))=>
				  if Term.equal (l,r)
				  then (R11,R12,R21,R22)
				  else case (Term.termDepth l,Term.termDepth r) of
					   (1,1) => (TPS.add (R11,(l,r)),R12,R21,R22)
					 | (1,2) => (R11,TPS.add (R12,(l,r)),R21,R22)
					 | (2,1) => (R11,R12,TPS.add (R21,(l,r)),R22)
					 | (2,2) => (R11,R12,R21,TPS.add (R22,(l,r)))
					 | _ => (print "plaised: unexpected error";
						 raise GTransError))
			      (TPS.empty,TPS.empty,TPS.empty,TPS.empty)
			      rs

	   (* Plaisted ѴεǼƥå *)
	   fun step (R11,R12,R21,R22)  =
	       let 
		   fun trans Init (Xset,Yset) =
		       ListXProd.foldX
			   (fn ((s,t),(l,r),set) =>
			       if Term.equal (t,l) 
				  andalso not (Term.equal (s,r))
			       then TPS.add (set,(s,r)) else set)
			   (TPS.listItems Xset, TPS.listItems Yset)
			   Init 
		   val R11a =  trans R11 (R11,R11)
		   val R12a =  trans (trans R12 (R11,R12)) (R12,R22)
		   val R21a =  trans (trans R21 (R21,R11)) (R22,R21)
		   val R22a =  trans R22 (R22,R22)
		   val R22b =  ListXProd.foldX
				   (fn ((s,t),(l,r),set) =>
				       let val u = Fun (appSymbol,[s,l],Sort.null)
					   val v = Fun (appSymbol,[t,r],Sort.null)
				       in if TS.member (subtermSet,u)
					     andalso TS.member (subtermSet,v)
					  then TPS.add (set,(u,v))
					  else set
				       end)
				   (TPS.listItems R11, TPS.listItems R11)
				   R22a 

		   val R22c =  ListXProd.foldX
				   (fn ((s,t),c,set) =>
				       let val u1 = Fun (appSymbol,[s,c],Sort.null)
					   val v1 = Fun (appSymbol,[t,c],Sort.null)
					   val u2 = Fun (appSymbol,[c,s],Sort.null)
					   val v2 = Fun (appSymbol,[c,t],Sort.null)
				       in if TS.member (subtermSet,u1)
					     andalso TS.member (subtermSet,v1)
					  then TPS.add
						   (if TS.member (subtermSet,u2)
						       andalso TS.member (subtermSet,v2)
						    then TPS.add (set,(u2,v2))
						    else set,
						    (u1,v1))
					  else if TS.member (subtermSet,u2)
						  andalso TS.member (subtermSet,v2)
					  then TPS.add (set,(u2,v2))
					  else set
				       end)
				   (TPS.listItems R11, TS.listItems constSet)
				   R22b
	       in
		   (R11a,R12a,R21a,R22c)
	       end

	   (* ˰¤ޤ Plaisted Ѵ򷫤֤ *)
	   fun saturate (R11,R12,R21,R22) = 
	       let val (R11',R12',R21',R22') = step (R11,R12,R21,R22)
	       in if TPS.equal (R11,R11')
		     andalso TPS.equal (R12,R12')
		     andalso TPS.equal (R21,R21')
		     andalso TPS.equal (R22,R22')
		  then (R11,R12,R21,R22)
		  else saturate (R11',R12',R21',R22')
	       end

	   (* Plaisted Ѵᥤ롼 *)
	   val (R11,R12,R21,R22) = saturate (partitionRules (L.filter (not o Term.equal) rs))
       in
	   (TPS.listItems R11)
	   @ (TPS.listItems R12) 
	   @ (TPS.listItems R21) 
	   @ (TPS.listItems R22) 
	   @ (L.map (fn x=>(x,x)) (TS.listItems subtermSet))
       end


   val rules = IOFotrs.rdRules
	       ["A -> F(B,B)",
		"B -> F(C,C)",
		"C -> F(A,A)",
		"C -> F(B,A)",
		"F(C,C) -> F(B,B)",
		"D -> F(F(B,B),E)",
		"D -> F(F(C,C),E)",
		"F(F(A,A),E) -> F(F(B,B),E)" ]
   val _ = print (Trs.prRules rules)
   val _ = print (Trs.prRules (curryrule rules))
   val _ = print (Trs.prRules (shallow (curryrule rules)))
   val _ = print (Trs.prRules (plaisted (shallow (curryrule rules))))

   end (* of local *)

end; (* of struct *)
