(******************************************************************************
 * Expanded Critical Pair for equation by Pham TuongVan
 * ecr.sml
  Based on
 * SML source file. Copyright (c) by Takahito Aoto 2007-2008.
 * cr.sml
 ******************************************************************************)

signature ECR = 
   sig
   val criticalPeaks : (Term.term * Term.term) list 
		       -> (Term.term * Term.term * Term.term) list
   val criticalPeaksKB : (Term.term * Term.term) 
			 -> (Term.term * Term.term) list 
			 -> (Term.term * Term.term * Term.term) list
   val insideCriticalPeaks : 
       (Term.term * Term.term) list -> (Term.term * Term.term * Term.term) list
   val insideCriticalPeaksBetweenDifferentRules: 
       (Term.term * Term.term) list -> (Term.term * Term.term * Term.term) list
   val insideCriticalPeaksBetweenDifferentRulesWithIndex: 
       (Term.term * Term.term) list -> ((int * int) * (Term.term * Term.term * Term.term)) list

   val outsideCriticalPeaks : 
       (Term.term * Term.term) list -> (Term.term * Term.term * Term.term) list
   val outsideCriticalPeaksInOneside : 
       (Term.term * Term.term) list -> (Term.term * Term.term * Term.term) list
   val outsideCriticalPeaksWithIndex: 
       (Term.term * Term.term) list -> ((int * int) * (Term.term * Term.term * Term.term)) list
   val outsideCriticalPeaksInOnesideWithIndex : 
       (Term.term * Term.term) list -> ((int * int) * (Term.term * Term.term * Term.term)) list

   val criticalPeaks2 : ((Term.term * Term.term) list) * ((Term.term * Term.term) list)
			-> (Term.term * Term.term * Term.term) list
   val insideCriticalPeaks2 : ((Term.term * Term.term) list) * ((Term.term * Term.term) list)
			       -> (Term.term * Term.term * Term.term) list
   val outsideCriticalPeaks2 : ((Term.term * Term.term) list) * ((Term.term * Term.term) list)
			       -> (Term.term * Term.term * Term.term) list

   val criticalPairs : (Term.term * Term.term) list -> (Term.term * Term.term) list
   val criticalPairsKB : (Term.term * Term.term) 
	   -> (Term.term * Term.term) list 
	   -> (Term.term * Term.term) list
(* Added by Pham TuongVan *)
(* Critical pair of a rule with some equations *)
   val criticalPeaksUKBR_ES : (Term.term * Term.term -> bool) 
	   -> (Term.term * Term.term)
	   -> (Term.term * Term.term) list
	   -> (Term.term * Term.term) list
(* Critical pair of an equation with some rules *)
   val criticalPeaksUKBE_RS : (Term.term * Term.term -> bool) 
	   -> (Term.term * Term.term)
	   -> (Term.term * Term.term) list
	   -> (Term.term * Term.term) list
(* Critical pair of an equation with some equation *)
   val criticalPeaksUKBE_ES : (Term.term * Term.term -> bool) 
	   -> (Term.term * Term.term)
	   -> (Term.term * Term.term) list
	   -> (Term.term * Term.term) list
(****End of Added Source ***)
   val insideCriticalPairs : 
       (Term.term * Term.term) list -> (Term.term * Term.term) list
   val insideCriticalPairsBetweenDifferentRules: 
       (Term.term * Term.term) list -> (Term.term * Term.term) list
   val insideCriticalPairsBetweenDifferentRulesWithIndex: 
       (Term.term * Term.term) list -> ((int * int) * (Term.term * Term.term)) list

   val outsideCriticalPairs : 
       (Term.term * Term.term) list -> (Term.term * Term.term) list
   val outsideCriticalPairsInOneside : 
       (Term.term * Term.term) list -> (Term.term * Term.term) list
   val outsideCriticalPairsWithIndex: 
       (Term.term * Term.term) list -> ((int * int) * (Term.term * Term.term)) list
   val outsideCriticalPairsInOnesideWithIndex: 
       (Term.term * Term.term) list -> ((int * int) * (Term.term * Term.term)) list

   val criticalPairs2 : ((Term.term * Term.term) list) * ((Term.term * Term.term) list)
			-> (Term.term * Term.term) list
   val insideCriticalPairs2 : ((Term.term * Term.term) list) * ((Term.term * Term.term) list)
			      -> (Term.term * Term.term) list
   val outsideCriticalPairs2 : ((Term.term * Term.term) list) * ((Term.term * Term.term) list)
			       -> (Term.term * Term.term) list

   val isLocalConfluent : (Term.term * Term.term) list -> bool
   val isLocalConfluentForNonTerminatingRules : (Term.term * Term.term) list -> bool
   val isNonOverlapping : (Term.term * Term.term) list -> bool
   val isOverlay : (Term.term * Term.term) list -> bool

   datatype ConfluenceResult = CR | NotCR | Unknown
   val report: ConfluenceResult -> ConfluenceResult

   val checkConfluenceConditions:
	   ((Term.term * Term.term) list -> bool) 
	   -> (Term.term * Term.term) list 
	   ->  ConfluenceResult


end;

structure ECr : ECR = 
   struct

   local 
       open Term
       open Trs
       open Rewrite
       open Subst
       structure VS = VarSet
       structure VM = VarMap
       structure FS = FunSet
       structure FM = FunMap
       structure SS = SortSet
       structure FIS = FunIntSet
       structure L = List
       structure LP = ListPair
       structure TS = TermSet
       fun mapAppend f xs = List.foldr (fn (x,ys) => List.@(f x, ys)) [] xs
       fun exceptNth n [] = []
	 | exceptNth n (x::xs) = 
	   if n = 0 
	   then xs
	   else (x::(exceptNth (n-1) xs))
   in

   val runDebug = ref false : bool ref
   fun debug f = if !runDebug then f () else ()


   (* we follow the definition of 
    "An Introduction to Knuth-Bendix Completion" by J.W.Klop and A.Middeldorp 
    *)
   local
       fun cpsub (alpha,beta) C (t,delta) =
	   case (cpsubWhole (alpha,beta) C (t,delta)) of
		   SOME ans => ans::(cpsubProperSubterm (alpha,beta) C (t,delta))
		 | NONE => cpsubProperSubterm (alpha,beta) C (t,delta)
       and cpsubWhole (alpha,beta) C (t,delta) =
	   case t of 
	       Var _ => NONE
	     | Fun (f,ts,ty) 
	       => case (unify t alpha) of
		      SOME sigma => SOME (applySubst sigma (C alpha), 
					  applySubst sigma (C beta), 
					  applySubst sigma delta)
		    | NONE => NONE
       and cpsubProperSubterm (alpha,beta) C (t,delta) =
	   case t of 
	       Var _ => []
	     | Fun (f,ts,ty) 
	       => cpsubArguments (alpha,beta) 
				 (fn xs => C (Fun (f,xs,ty)))
				 (ts,delta) 
       and cpsubArguments (alpha,beta) C ([],delta) = []
	 | cpsubArguments (alpha,beta) C (t::ts,delta) =
	   List.@(cpsub (alpha,beta) (fn x => C (x::ts)) (t,delta),
		  cpsubArguments (alpha,beta) (fn xs => C (t::xs)) (ts,delta))

(***********************************************************)
(*Inserted by PHAM TuongVan*)
	   (* find an cp of alpha = beta and gamma = delta as follow
		  find a substitution sigma such that alpha o sigma = t o sigma
		  and gamma = C[t]
		  if  ordab (beta o sigma, alpha o sigma) or orddg (delta o sigma, C[alpha] o sigma)
			  then NONE 
            else exist a cp of (C[beta o sigma], delta o sigma)
		*)
       fun ecpsub ordab orddg (alpha,beta) C (t,delta) =
		   case (ecpsubWhole ordab orddg (alpha,beta) C (t,delta)) of
			   SOME ans => ans::(ecpsubProperSubterm ordab orddg (alpha,beta) C (t,delta))
			 | NONE => ecpsubProperSubterm ordab orddg (alpha,beta) C (t,delta)
       and ecpsubWhole ordab orddg (alpha,beta) C (t,delta) =
		   case t of 
			   Var _ => NONE
	     | Fun (f,ts,ty) 
				   => case (unify t alpha) of
				   SOME sigma => 
					   if ordab(applySubst sigma beta, applySubst sigma alpha) orelse
						   orddg(applySubst sigma delta, applySubst sigma (C alpha)) then NONE
					   else SOME ((*applySubst sigma (C alpha),*) 
								  applySubst sigma (C beta), 
								  applySubst sigma delta)
				 | NONE => NONE
       and ecpsubProperSubterm ordab orddg (alpha,beta) C (t,delta) =
		   case t of 
			   Var _ => []
	     | Fun (f,ts,ty) 
				   => ecpsubArguments ordab orddg (alpha,beta) 
				   (fn xs => C (Fun (f,xs,ty)))
				   (ts,delta) 
       and ecpsubArguments ordab orddg (alpha,beta) C ([],delta) = []
		 | ecpsubArguments ordab orddg (alpha,beta) C (t::ts,delta) =
		   List.@(ecpsub ordab orddg (alpha,beta) (fn x => C (x::ts)) (t,delta),
				  ecpsubArguments ordab orddg (alpha,beta) (fn xs => C (t::xs)) (ts,delta))

(****End of Inserting Text by Pham TuongVan********)
   in
   (* $B0[$J$k5,B'4V$N%k!<%H0LCV$G$N4m81BP$N(B2$BEY7W;;$r$J$/$7$?(B *)
   fun criticalPeaks rs =
       let val rs' = renameRules rs
		   val nums = List.tabulate (List.length rs, fn i => i)   (* 0,...,n-1 *)
       in
		   mapAppend
		   (fn n => mapAppend
			(fn m => if n = m
						 then
							 let
								 val lr = List.nth (rs,n)
								 val [lr1,lr2] = renameRules [lr,lr]
							 in (* $B<+J,<+?H$K$D$$$F$O(B proper subterm $B$G$N=E$J$j$@$1(B *)
								 cpsubProperSubterm lr1 (fn x => x) lr2
							 end
			else if n < m
					 then (* $BAj0[$J$k5,B'$K$D$$$F$O(B $BG$0U$N(B subterm $B$G$N=E$J$j(B *)
						 cpsub (List.nth (rs',m)) (fn x => x) (List.nth (rs',n))
				 else (* m < n *)
					 (* m,n $B4V$N%k!<%H0LCV$O(B n < m $B$N$H$-$K$d$C$F$$$k$N$G$$$i$J$$(B *)
					 cpsubProperSubterm (List.nth (rs',m)) (fn x => x) (List.nth (rs',n)))
			nums)
		   nums


       end

  (*  r $B$H(B r::rs $B$N4V$N4m81BP=89g(B *)
   fun criticalPeaksKB r rs = 
       let val (r1::r2::rs') = renameRules (r::r::rs)
       in 
	   (* $B<+J,<+?H$K$D$$$F$O(B proper subterm $B$G$N=E$J$j$@$1(B *)
 	   L.@ (cpsubProperSubterm r1 (fn x => x) r2,
		(* rs $B$N5,B'$X$N(B r $B$N=E$J$j!$G$0U$N(B subterm $B$G(B *)
		L.@ (mapAppend
 			 (fn lr => cpsub lr (fn x => x) r1)
 			 rs',
		     (* r $B$X$N(B rs $B$N5,B'$N=E$J$j!$G$0U$N(B subterm $B$G(B *)
		     mapAppend
			 (fn lr => cpsub r1 (fn x => x) lr)
			 rs'))
       end

(**********Insert by Pham TuongVan***)
  (* l1 = r1, l2->r2 : ord, false
     r1=l1, l2->r2: ord, false
	 l2->r2, l1=r1: false, ord
	 l2->r2, r1=l1: false, ord *)
   
 
  (*  Critical pair (rule,equationS) *)
   fun criticalPeaksUKBR_ES ord r' es' = 
       let val (r::es) = renameRules (r'::es')
       in (* $B<+J,<+?H$N=E$J$$(B *)
		   L.@( 
			   (* ord, false, equation, rule *)
			   mapAppend (fn lr => ecpsub ord (fn _ => false) lr (fn x=>x) r) es,
			   L.@(
				   (* false, ord, rule, equation *)
				   mapAppend 
				   (fn lr => ecpsub (fn _ => false) ord r (fn x=>x) lr) es,
				   L.@(
					   (* ord, false, changed equation, rule *) 
					   mapAppend 
					   (fn (l0,r0) => ecpsub ord (fn _ => false) (r0,l0) (fn x => x) r) es,
					   (* false, ord, rule, changed equation *)
					   mapAppend 
					   (fn (l0,r0) => ecpsub (fn _ => false) ord r (fn x => x) (r0,l0)) es
					   )))
	   end
   
   (* Critical Pair (equation, RuleS)*)   
   fun criticalPeaksUKBE_RS ord e' rs' = 
       let val (e::rs) = renameRules (e'::rs')
		   fun rdb (a,b) = (b,a)
       in (* $B<+J,<+?H$N=E$J$$(B *)
		   L.@( 
			   (* ord, false, equation, rule *)
			   mapAppend (fn lr => ecpsub ord (fn _ => false) e (fn x=>x) lr) rs,
			   L.@(
				   (* false, ord, rule, equation *)
				   mapAppend 
				   (fn lr => ecpsub (fn _ => false) ord lr (fn x=>x) e) rs,
				   L.@(
					   (* ord, false, changed equation, rule *) 
					   mapAppend 
					   (fn lr => ecpsub ord (fn _ => false) (rdb e) (fn x => x) lr) rs,
					   (* false, ord, rule, changed equation *)
					   mapAppend 
					   (fn lr => ecpsub (fn _ => false) ord lr (fn x => x) (rdb e)) rs
					   )))
	   end
   (* Critical Pair (equation, equationS) *)

   fun criticalPeaksUKBE_ES ord e' es' = 
       let val (e1::e2::es) = renameRules (e'::e'::es')
		   fun rdb (a,b) = (b,a)
       in (* $B<+J,<+?H$N=E$J$j(B *)
		   (* e1 = e2 *)
		   L.@(ecpsubProperSubterm ord ord e1 (fn x => x) e2,		   		   
		   L.@(ecpsubProperSubterm ord ord e1 (fn x => x) (rdb e2),
		   L.@(ecpsubProperSubterm ord ord (rdb e1) (fn x => x) e2,
		   L.@(ecpsubProperSubterm ord ord (rdb e1) (fn x => x) (rdb e2),
		   
		   (* e2 = e1 *)
		   L.@(ecpsubProperSubterm ord ord e2 (fn x => x) e1,		   		   
		   L.@(ecpsubProperSubterm ord ord e2 (fn x => x) (rdb e1),
		   L.@(ecpsubProperSubterm ord ord (rdb e2) (fn x => x) e1,
		   L.@(ecpsubProperSubterm ord ord (rdb e2) (fn x => x) (rdb e1),
			   
		   (* And the other e1 = e2 *)

		   L.@((* ord, ord, equation, equation *)
			   mapAppend (fn lr => ecpsub ord ord e1 (fn x=>x) lr) es,		  
		   L.@((* ord, ord, equation, noitauqe *)
			   mapAppend (fn (l0,r0) => ecpsub ord ord e1 (fn x=>x) (r0,l0)) es,
		   L.@((* ord, ord, noitauqe, equation *)
			   mapAppend (fn lr => ecpsub ord ord (rdb e1) (fn x=>x) lr) es,		    
		   L.@((* ord, ord, noitauqe, noitauqe *)
			   mapAppend (fn (l0,r0) => ecpsub ord ord (rdb e1) (fn x=>x) (r0,l0)) es,		  

		   (* And the other e2 = e1 *)

		   L.@((* ord, ord, equation, equation *)
			   mapAppend (fn lr => ecpsub ord ord lr (fn x=>x) e1) es,		  
		   L.@((* ord, ord, equation, noitauqe *)
			   mapAppend (fn (l0,r0) => ecpsub ord ord (r0,l0) (fn x=>x) e1) es,
		   L.@((* ord, ord, noitauqe, equation *)
			   mapAppend (fn lr => ecpsub ord ord lr (fn x=>x) (rdb e1)) es,		    
			   (* ord, ord, noitauqe, noitauqe *)
			   mapAppend (fn (l0,r0) => ecpsub ord ord (r0,l0) (fn x=>x) (rdb e1)) es
			   )))))))))))))))
			  
	   end;

(****End of insert by Pham TuongVan***)

   fun insideCriticalPeaks rs = 

       let val rs' = renameRules rs
	   val nums = List.tabulate (List.length rs, fn i => i)   (* 0,...,n-1 *)
       in 
 	   L.@(mapAppend 
		   (* $BAj0[$J$k5,B'$K$D$$$F$O(B renaming $BI,MW$J$7(B *)
		   (fn n => mapAppend
				(fn lr => cpsubProperSubterm lr (fn x => x) (List.nth (rs',n)))
				(exceptNth n rs'))
		   nums,
	       (* $B<+J,<+?H$K$D$$$F$O(B renaming $B$r$7$F$+$i(B *)
	       mapAppend (fn lr => let val [lr1,lr2] = renameRules [lr,lr]
				   in cpsubProperSubterm lr1 (fn x => x) lr2
				   end)
			 rs')
       end

       (* outside CP $B$O(B $BBP>NE*$J$N$G!$JRB&$@$1$GNI$$>l9g$O$3$A$i(B *)
   fun outsideCriticalPeaksInOneside rs = 
       let val rs' = renameRules rs
	   fun loop [] = []
	     | loop (x::ys) = 
	       L.@(List.mapPartial (fn y => cpsubWhole x (fn i => i) y) ys,
		   loop ys)
       in 
	   loop rs'
       end

(*    fun outsideCriticalPeaks rs =  *)
(*        let val rs' = renameRules rs *)
(* 	   val nums = List.tabulate (List.length rs, fn i => i)   (\* 0,...,n-1 *\) *)
(*        in  *)
(*  	   mapAppend  *)
(* 		   (fn n => List.mapPartial *)
(* 				(fn lr => cpsubWhole lr (fn x => x) (List.nth (rs',n))) *)
(* 				(exceptNth n rs')) *)
(* 		   nums *)
(*        end *)

    fun outsideCriticalPeaks rs =  
	let val cps = outsideCriticalPeaksInOneside rs
	in cps @ (L.map (fn (x,y,z) => (x,z,y)) cps)
	end

   fun insideCriticalPeaksBetweenDifferentRules rs = 
       let val rs' = renameRules rs
	   val nums = List.tabulate (List.length rs, fn i => i)   (* 0,...,n-1 *)
       in 
 	   mapAppend 
	   (fn n => mapAppend
			(fn m => cpsubProperSubterm (List.nth (rs',m))
						    (fn x => x) (List.nth (rs',n)))
			(exceptNth n nums))
	   nums
       end;

   fun insideCriticalPeaksBetweenDifferentRulesWithIndex rs = 
       let val rs' = renameRules rs
	   val nums = List.tabulate (List.length rs, fn i => i)   (* 0,...,n-1 *)
       in 
 	   mapAppend 
	   (fn n => mapAppend
			(fn m => L.map (fn cp => ((m,n),cp))
				       (cpsubProperSubterm (List.nth (rs',m))
							   (fn x => x) (List.nth (rs',n))))
			(exceptNth n nums))
	   nums
       end;

   fun outsideCriticalPeaksInOnesideWithIndex rs = 
       let val rs' = renameRules rs
	   val nums = List.tabulate (List.length rs, fn i => i)   (* 0,...,n-1 *)
	   fun loop [] = []
	     | loop (n::ms) = 
	       L.@(List.mapPartial
				(fn m => case cpsubWhole (List.nth (rs',m))
							 (fn x => x) (List.nth (rs',n))
					  of SOME cp => SOME ((m,n),cp)
					   | NONE => NONE) ms,
		   loop ms)
       in 
	   loop nums
       end

(*    fun outsideCriticalPeaksWithIndex rs =  *)
(*        let val rs' = renameRules rs *)
(* 	   val nums = List.tabulate (List.length rs, fn i => i)   (\* 0,...,n-1 *\) *)
(*        in  *)
(*  	   mapAppend  *)
(* 		   (fn n => List.mapPartial *)
(* 				(fn m => case cpsubWhole (List.nth (rs',m)) *)
(* 							 (fn x => x) (List.nth (rs',n)) *)
(* 					  of SOME cp => SOME ((m,n),cp) *)
(* 					   | NONE => NONE) *)
(* 				(exceptNth n nums)) *)
(* 		   nums *)
(*        end *)

   fun outsideCriticalPeaksWithIndex rs = 
       let val cps = outsideCriticalPeaksInOnesideWithIndex rs
	in cps @ (L.map (fn ((m,n),(x,y,z)) => ((n,m),(x,z,y))) cps)
	end


  (* 2$B$D$N0[$J$k=q$-49$(%7%9%F%`$N85$N4m81D:(B *)
   fun criticalPeaks2 (rs1,rs2)  = 
       let val (rs1',rs2') = renameRuleSet (rs1,rs2)
       in 
 	   mapAppend 
		   (fn lr1' => mapAppend
				(fn lr2' => cpsub lr1' (fn x => x) lr2')
				rs2')
		   rs1'
       end

   fun insideCriticalPeaks2 (rs1,rs2)  = 
       let val (rs1',rs2') = renameRuleSet (rs1,rs2)
       in 
 	   mapAppend 
		   (fn lr1' => mapAppend
				(fn lr2' => cpsubProperSubterm lr1' (fn x => x) lr2')
				rs2')
		   rs1'
       end

   fun outsideCriticalPeaks2 (rs1,rs2)  = 
       let val (rs1',rs2') = renameRuleSet (rs1,rs2)
       in
 	   mapAppend 
		   (fn lr1' => List.mapPartial
				(fn lr2' => cpsubWhole lr1' (fn x => x) lr2')
				rs2')
		   rs1'
       end

   fun criticalPairs rs = L.map (fn (x,y,z) => (y,z)) (criticalPeaks rs)

   fun criticalPairsKB r rs = L.map (fn (x,y,z) => (y,z)) (criticalPeaksKB r rs)

   fun insideCriticalPairs rs = 
       L.map (fn (x,y,z) => (y,z)) (insideCriticalPeaks rs)

   fun insideCriticalPairsBetweenDifferentRules rs = 
       L.map (fn (x,y,z) => (y,z)) (insideCriticalPeaksBetweenDifferentRules rs)

   fun insideCriticalPairsBetweenDifferentRulesWithIndex rs = 
       L.map (fn (mn,(x,y,z)) => (mn,(y,z))) (insideCriticalPeaksBetweenDifferentRulesWithIndex rs)

   fun outsideCriticalPairsInOneside rs = 
       L.map (fn (x,y,z) => (y,z)) (outsideCriticalPeaksInOneside rs)

   fun outsideCriticalPairs rs = 
       L.map (fn (x,y,z) => (y,z)) (outsideCriticalPeaks rs)

   fun outsideCriticalPairsInOnesideWithIndex rs = 
       L.map (fn (mn,(x,y,z)) => (mn,(y,z))) (outsideCriticalPeaksInOnesideWithIndex rs)

   fun outsideCriticalPairsWithIndex rs = 
       L.map (fn (mn,(x,y,z)) => (mn,(y,z))) (outsideCriticalPeaksWithIndex rs)

   fun criticalPairs2 (rs1,rs2) = L.map (fn (x,y,z) => (y,z)) (criticalPeaks2 (rs1,rs2))
   fun insideCriticalPairs2 (rs1,rs2) = L.map (fn (x,y,z) => (y,z)) (insideCriticalPeaks2 (rs1,rs2))
   fun outsideCriticalPairs2 (rs1,rs2) = L.map (fn (x,y,z) => (y,z)) (outsideCriticalPeaks2 (rs1,rs2))
   end

   fun isJoinableCps rs cps =
       List.all (fn (u,v) => Term.equal (linf rs u,  linf rs v)) 
		cps

   fun isLocalConfluent rs =
       isJoinableCps rs (criticalPairs rs)	 

   fun isLocalConfluentForNonTerminatingRules rs =
       let val cps = criticalPairs rs
	   fun isjoinable (u,v) 
	     = not (TS.isEmpty 
		    (TermSet.intersection (developOneStepReductSet rs u,
					   developOneStepReductSet rs v)))
       in L.all isjoinable (criticalPairs rs)
       end

   fun isNonOverlapping rs = 
       null (criticalPairs rs)

   fun isOverlay rs = 
       null (insideCriticalPeaks rs)

   (* Huet $B$N(B Parallel-Closed $B>r7o(B *)
   fun isParallelClosedCps rs cps =
       List.all (fn (t1,t2) => 
		    if Term.equal (t1,t2) 
		    then true
	            else TS.member (Rewrite.parallelOneStepReductSet rs t1, t2))
		cps

   (* $B30;3$N>r7o!'4J0WHG(B *)
   fun isToyamaClosedCps rs inCps outCps =
       (List.all (fn (t1,t2) => 
		     Term.equal (t1,t2) 
		     orelse TS.member (Rewrite.parallelOneStepReductSet rs t1, t2))
		 inCps)
       andalso
       (List.all (fn (t1,t2) => 
		    Term.equal (t1,t2) 
		    orelse 		    
	            not (TS.isEmpty (TS.intersection (Rewrite.parallelOneStepReductSet rs t1,  
						      Rewrite.parallelOneStepReductSet rs t2))))
		 outCps)

   (* Oostrom$B>r7o!'4J0WHG(B *)
   fun isOostromClosedCps rs inCps outCps =
       (List.all (fn (t1,t2) => 
		     Term.equal (t1,t2) 
		     orelse TS.member (Rewrite.developOneStepReductSet rs t1, t2))
		 inCps)
       andalso
       (List.all (fn (t1,t2) => 
			    Term.equal (t1,t2) 
			    orelse 		    
			    not (TS.isEmpty (TS.intersection (Rewrite.developOneStepReductSet rs t1,  
							      Rewrite.developOneStepReductSet rs t2))))
			outCps)

  (* $B9gN.@-$N8!>Z7k2L(B *)
   datatype ConfluenceResult = CR | NotCR | Unknown

   fun report CR = (print ": CR\n"; CR)
     | report NotCR = (print ": not CR\n"; NotCR)
     | report Unknown = (print ": Can't judge\n"; Unknown)



  (* $B9gN.@-$N4pK\H=Dj(B *)
   fun checkConfluenceConditions isTerminating rs = 
       let val inCps = insideCriticalPairs rs
	   val outCps = outsideCriticalPairsInOneside rs
	   val _ = print "Check Termination...\n"
       in
	   if isTerminating rs
	   then let val _ = print "Terminating"
		    val joinableCps = isJoinableCps rs (inCps @ outCps)
		    val _ = if joinableCps
			    then print ", WCR"
			    else print ", not WCR"
		in
		    if joinableCps then report CR else report NotCR
		end
	   else 
	       let val _ = print "unknown Terminating"
	       in
		   if Trs.areLeftLinearRules rs
		   then let val _ = print ", Left-Linear"
(*			    val _ = print (Trs.prRules rs) *)
(*			    val _ = print ("In  CPs:\n" ^ (prEqs inCps)) *)
(*			    val _ = print ("Out CPs:\n" ^ (prEqs outCps)) *)
			    val oostromClosedCps = isOostromClosedCps rs inCps outCps
			    val _ = if oostromClosedCps
				    then print ", Oostrom"
				    else print ", unknown Oostrom"
			in
			    if oostromClosedCps 
			    then report CR 
			    else report Unknown
			end
		   else let val _ = print ", not Left-Linear"
			in report Unknown
			end
	       end
       end




   end (* of local *)

   end; (* of structure Cr *)










