(******************************************************************************
 * file: rwtools/kbcomp/kb.sml
 * description: Knuth-Bendix Completion Procedure
 * author: MICHIMATA Junichi
 * author: AOTO Takahito
 * 
 * Copyright (c) 2008, Toyama&Aoto Laboratory, Tohoku University
 ******************************************************************************)

signature KB = 
sig
        val kb : (Term.term * Term.term -> bool) -> Trs.equations -> bool
end;

structure Kb : KB = struct

   exception KbError;

   (*   RuleSet$B$+$i(B $B9`%5%$%:$N(Bmax$B$,:G$b>.$5$$5,B'(Br$B$rA*$S!$AH(B( r, RuleSet - {r} ) $B$rJV$94X?t(B  *)
   fun orientation ord (rs, []) = (print "no equation to expand\n";  raise KbError)
     |  orientation ord (rs, es) =
	let

            (* $B9`%5%$%:(B($BD9(B,$BC;(B)$B$HBP1~$9$k9`$r4^$s$@%j%9%H$r%=!<%H$7JV$9(B *)
            fun makeVarNumlist rs =
		let 
                    fun withSize (l,r) = let val llen = Term.termSize l
					     val rlen = Term.termSize r
					 in if rlen > llen
					    then ((rlen,llen),(r,l))
					    else ((llen,rlen),(l,r))
					 end
		in
                    ListMergeSort.sort 
			(fn (((n1,m1),_),((n2,m2),_)) => n1 > n2 orelse 
							 (n1 = n2 andalso m1 > m2))
			(List.map (fn lr => withSize lr) rs)
		end

	   (* $B$D$$$G$K!$%i%U$K=EJ#$r=|5n(B *)
	    local fun removesub [] ans = (rev ans)
		    | removesub (x::xs) [] = removesub xs [x]
		    | removesub ((ls,s)::xs) ((lt,t)::ys) = 
		      if ls = lt (* $B:8JU$H1&JU%5%$%:$,F1$8$J$i(B *)
			 andalso (Subst.identicalModuloRenamingRule s t)
		      then removesub xs ((lt,t)::ys)
		      else removesub xs ((ls,s)::(lt,t)::ys)
	    in
	    fun removeDuplication xs =  removesub xs []
	    end

            (*  $B9`%5%$%:(B($B:8JU$H1&JU$N(Bmax)$B$N>.$5$$=g$K$_$F!$:G=i$K=g=x$KIU$/Ey<0$H$=$l0J30$KJ,3d$9$k(B *)
            fun select ord [] done = (print "No orientable equations\n"; raise KbError)
              | select ord ((l,r)::es) done = 
		if ord (l,r) then ((l,r), (rev done) @ es )
		else if ord (r,l) then  ((r,l), (rev done) @ es)
		else select ord es ((l,r)::done)
                     
            val sorted = List.map (#2) (removeDuplication (makeVarNumlist es))

            val ((a,b),rest) = select ord sorted []
	in
            ((a,b), rs , rest)
	end                   


   (* rs $B$N1&JU$r(B r::rs $B$rMQ$$$F@55,2=$9$k(B *)
   fun composition (r, rs, es) =
       let
           fun comp (a,b) = (a, Rewrite.linf (r::rs) b)
       in
           (r, List.map comp rs, es)
       end


   (* $B=q$-49$(5,B'(B r $B$H(B $B=q$-49$(5,B'=89g(B r::rs $B$N4V$N4m81BP$rEy<0$KDI2C$9$k(B *)
   fun deduction (r, rs, es) = 
       (r, rs, List.@(Cr.criticalPairsKB r rs, es)) 

(*    only a litte improvement *)
(*    local *)
(*        fun appendRules [] ys = ys *)
(*          | appendRules (x::xs) ys = *)
(*    	if List.exists (fn y=> Subst.identicalModuloRenamingRule x y) ys *)
(*    	then appendRules xs ys *)
(*    	else x::appendRules xs ys *)
(*    in *)
(*    fun deduction (r, rs, es) = *)
(*        (r, rs, appendRules (Cr.criticalPairsKB r rs) es) *)
(*    end *)

   (* rs $B$+$i!$:8JU$,(B r $B$G=q$-49$(2DG=$J$b$N$r<h$j=|$/(B *)
   fun collapse (r, rs, es) = 
       let
           fun collaplist (l,_) rs =
               let
                   fun subcol l (Term.Var _) = false
                     | subcol l (m as Term.Fun( _ , ts , _) ) =
                       case Subst.match l m of
                           SOME _ => true
			 | NONE   => subcollist l ts
                   and subcollist _    []   = false
                     | subcollist l (t::ts) = (subcol l t) orelse (subcollist l ts) 
               in
                   List.filter (fn (m,_) => not (subcol l m)) rs
               end
       in
           (r, collaplist r rs, es)
       end

   (* $BEy<0=89g$NN>JU$r@55,2=$9$k(B *)
   fun simplification (rs, es) = 
       let
           fun simpl (l,r) = (Rewrite.linf rs l, Rewrite.linf rs r)
       in
           (rs, List.map simpl es)
       end

   (* $BEy<0=89g$+$i!$N>JU$,F1$8Ey<0$r:o=|$9$k(B *)
   fun deletion (rs, es) = (rs, List.filter (not o Term.equal) es)

(****   this makes slower ****)
(*    fun unique [] = [] *)
(*      | unique ((l,r)::rs) =  *)
(*        let *)
(*            fun ordTermPair (x,y) = case Term.compare (x,y) of *)
(* 			   EQUAL => [x,y] *)
(* 			 | GREATER => [y,x] *)
(* 			 | LESS => [x,y] *)
(*            val [l',r'] = let val t = Term.Fun (Fun.fromString "",ordTermPair (l,r),Sort.null) *)
(* 			 in case Subst.renameTermToStd t of *)
(* 				Term.Fun (_,ts,_) => ts | _ => [l,r] *)
(* 			 end *)
(* 	   val rest = unique rs *)
(*        in if List.exists (fn (x,y) => Term.equal (l',x) *)
(* 				   andalso Term.equal (r',y)) rest *)
(* 	  then rest *)
(* 	  else (l',r')::rest *)
(*        end *)


   (* $B40Hw2=$N5"G<%9%F%C%W(B *)
   fun kbstep grter (rs, es) =
       (deletion
	o simplification
	o (fn (r, rs, es) => (r::rs, es))
	o collapse
	o deduction
	o composition 
	o (orientation grter))  (rs, es)
       

   (* $B40Hw2=$N%a%$%s%k!<%A%s(B *)
   fun kb grter es =
       let 
	   val _ = print "Start Knuth-Bendix Completion Procedure with the Given Order...\n"
	   val _ = print "Initial set of equations:\n"
	   val _ = print (Trs.prEqs es)
	   fun kbsub n grter (rs, es) = 
	       let val _ = print ("Step " ^ (Int.toString n) ^ "\n")
		   val (rs',es') = kbstep grter (rs,es)
		   val _ = print (Trs.prRules rs')
		   val _ = print (Trs.prEqs es')
	       in
		   if null es'
		   then (print ("Succeed in " ^ (Int.toString n) ^ " steps \n");
			 print (Trs.prRules rs'); 
			 true)
		   else  kbsub (n+1) grter (rs',es')
	       end
       in
	   kbsub 1 grter ([], es)
	   handle KbError => (print "Failed:\n"; false)
       end

end




