(******************************************************************************
 * file: rwtools/kbcomp/kb_prover.sml
 * description: theorem prover based on Knuth-Bendix Completion Procedures
 * author: Pham Tuong Van
 * 
 * Copyright (c) 2008, Toyama&Aoto Laboratory, Tohoku University
 ******************************************************************************)

signature KB_PROVER = 
sig
	exception KB
	val kbProver : (Term.term * Term.term -> bool) -> Trs.equations -> Trs.equation -> bool
end;

structure KbProver : KB_PROVER = 

struct

    exception KB;
    (*   RuleSet$B$+$i!"=g=x$,$D$/5,B'$NFb!$N>JU$NJQ?t$,:G$b>/$J$$5,B'(Br$B$rA*$S!$AH(B( r, RuleSet - {r} ) $B$rJV$94X?t(B  *)
   fun orientation ord (rs, [], goal) = (print "No equation to expand\n";  raise KB)
    |  orientation ord (rs, es, goal) =
	   let

	       (* $B9`$NAH$ND9$$J}$N%5%$%:$r4^$s$@O"7k%j%9%H$rJV$9(B *)
	       fun makeVarNumlist rs =
		   let 
		       fun maxSize (l,r) =Int.max (Term.termSize l, Term.termSize r)
		   in
		       ListMergeSort.sort 
			   (fn ((n1,_),(n2,_)) => n1 > n2)
			   (List.map (fn lr => (maxSize lr, lr)) rs)
		   end
		   
		   (*   $BN>JU$NJQ?t$,:G$b>/$J$$5,B'$H$=$N;D$j$rJV$94X4X?t(B *)
	       fun leastvarbhs ord [] done = (print "No orientable equations. Goal is unsolvable\n"; raise KB)
		 | leastvarbhs ord ((l,r)::es) done = 
		   if ord (l,r) then ((l,r), (rev done) @ es )
		   else if ord (r,l) then  ((r,l), (rev done) @ es)
		   else leastvarbhs ord es ((l,r)::done)
			
               val sorted = List.map (#2) (makeVarNumlist es)

               val ((a,b),rest) = leastvarbhs ord sorted []
	   in
		  ((a,b), rs , rest, goal)
	   end		   


    (* rs $B$N1&JU$r(B r::rs $B$rMQ$$$F@55,2=$9$k(B *)
    fun composition (r, rs, es, goal) =
		let
			fun comp (a,b) = (a, Rewrite.linf (r::rs) b)
		in
			(r, List.map comp rs, es, goal)
		end


    (* $B=q$-49$(5,B'(B r $B$H(B $B=q$-49$(5,B'=89g(B r::rs $B$N4V$N4m81BP$rEy<0$KDI2C$9$k(B *)
    fun deduction (r, rs, es, goal) = 
           (r, rs, List.@(Cr.criticalPairsKB r rs, es), goal)

    (* rs $B$+$i!$:8JU$,(B *)
    fun collapse (r, rs, es, goal) = 
		let
			fun collaplist (l,_) rs =
				let
					fun subcol l (Term.Var _) = false
					  | subcol l (m as Term.Fun( _ , ts , _) ) =
						case Subst.match l m of
							SOME _ => true
						  | NONE   => subcollist l ts
					and subcollist _    []   = false
					  | subcollist l (t::ts) = (subcol l t) orelse (subcollist l ts) 
				in
					List.filter (fn (m,_) => not (subcol l m)) rs
				end
	    in
              (r, collaplist r rs, es, goal)
		end

    (* $BEy<0=89g$NN>JU$r@55,2=$9$k(B *)
    fun simplification (rs, es, goal) = 
		let
			fun simpl (l,r) = (Rewrite.linf rs l, Rewrite.linf rs r)
		in
			(rs, List.map simpl es, simpl goal)
		end

    (* $BEy<0=89g$+$i!$N>JU$,F1$8Ey<0$r:o=|$9$k(B *)
    fun deletion (rs, es, goal) = if Term.equal goal then (print "Goal is proved TRUE. "; raise KB)
								  else(rs, List.filter (not o Term.equal) es, goal)

    (* $B40Hw2=$N5"G<%9%F%C%W(B *)
    fun kbsub _ (rs, [], goal) = (print (Trs.prRules rs); rs)
      | kbsub grter (rs, es, goal) = kbsub grter (kbstep grter (rs, es, goal))
    and kbstep grter (rs, es, goal) =
    (* Remove comment to make program display KB result step by step *)
	let (*val _ = print (Trs.prRules rs)
	    val _ = print (Trs.prEqs es)*)
	in
       (deletion
         (simplification
          ((fn (r, rs, es, goal) => (r::rs, es, goal))
           (collapse
            (deduction
             (composition (orientation grter (rs, es, goal))))))))
	end

    (* EquationSet$B7?0z?t(B es $B$N!$N>JU$,Ey$7$$$b$N$r<h$j=|$/(B *)
    fun kbProver grter es goal =
		let
			val result = kbsub grter ([], List.filter (not o Term.equal) es, goal)
			fun simpl (l,r) = (Rewrite.linf result l, Rewrite.linf result r)
			val sgoal = simpl goal;
			val _ = if Term.equal sgoal then 
				    print  "TRUE.\n" else 
				print ("FALSE. Final form: " ^ (Trs.prEq sgoal))
		in true
		end
		handle KB => ((print "Finish proving\n"); false)
			  
end;


