(* SML source file. Copyright (c) by Takahito Aoto 2008.
 * test_kb.sml
 *)

signature SigTestKBP=  
  sig
      val kTest: int -> (Term.term * Term.term) list
  end;

structure TKP : SigTestKBP =
    struct

    fun time f arg=
	let
	    val timer = Timer.startCPUTimer ()
	    val start = TimeUtil.checkTime timer 
	    val result = f arg
	    val stop = TimeUtil.checkTime timer 
	    val _ = print (" ("  
			   ^ (TimeUtil.reportTime (start,stop)) 
			   ^ " msec.)\n")	
	in
	    result
	end

    fun kTest 1 = 
		time (KBP.kb 
			  (Order.qlposCheck
		       (Order.comparePrec [Atom.atom "I", Atom.atom "D", Atom.atom "E"])
		       (fn _ => (fn xs => xs))
		       Order.GT))
		(IOFotrs.rdEqs
		 ["D(E, ?x) = ?x",
		  "D(I(?x), ?x) = E",
		   "D(D(?x, ?y), ?z) = D(?x, D(?y, ?z))"])
		(IOFotrs.rdEq "D(?x, E) = ?x")
	  |	kTest 2 = 
		time (KBP.kb 
			  (Order.qlposCheck
		       (Order.comparePrec [Atom.atom "I", Atom.atom "D", Atom.atom "E"])
		       (fn _ => (fn xs => xs))
		       Order.GT))
		(IOFotrs.rdEqs
		 ["D(E, ?x) = ?x",
		  "D(I(?x), ?x) = E",
		   "D(D(?x, ?y), ?z) = D(?x, D(?y, ?z))"])
		(IOFotrs.rdEq "I(I(?x)) = ?x")
	  |	kTest 3 = 
		time (KBP.kb 
			  (Order.qlposCheck
		       (Order.comparePrec [Atom.atom "I", Atom.atom "D", Atom.atom "E"])
		       (fn _ => (fn xs => xs))
		       Order.GT))
		(IOFotrs.rdEqs
		 ["D(E, ?x) = ?x",
		  "D(I(?x), ?x) = E",
		   "D(D(?x, ?y), ?z) = D(?x, D(?y, ?z))"])
		(IOFotrs.rdEq "D(E,I(I(?x))) = I(I(?x))")
	  |	kTest 4 = 
		time (KBP.kb 
			  (Order.qlposCheck
		       (Order.comparePrec [Atom.atom "I", Atom.atom "D", Atom.atom "E"])
		       (fn _ => (fn xs => xs))
		       Order.GT))
		(IOFotrs.rdEqs
		 ["D(E, ?x) = ?x",
		  "D(I(?x), ?x) = E",
		   "D(D(?x, ?y), ?z) = D(?x, D(?y, ?z))"])
		(IOFotrs.rdEq "D(?x, E) = ?x")
	  |	kTest 5 = 
		time (KBP.kb 
			  (Order.qlposCheck
		       (Order.comparePrec [Atom.atom "I", Atom.atom "D", Atom.atom "E"])
		       (fn _ => (fn xs => xs))
		       Order.GT))
		(IOFotrs.rdEqs
		 ["D(E, ?x) = ?x",
		  "D(I(?x), ?x) = E",
		   "D(D(?x, ?y), ?z) = D(?x, D(?y, ?z))"])
		(IOFotrs.rdEq "D(D(?x,?y),D(I(?y),I(I(I(?x))))) = E")
(* Test 6, KB Completion Generate infinite Rules *)
	  |	kTest 6 = 
		time (KBP.kb 
			  (Order.qlposCheck
		       (Order.comparePrec [Atom.atom "I", Atom.atom "D", Atom.atom "E"])
		       (fn _ => (fn xs => xs))
		       Order.GT))
		(IOFotrs.rdEqs
		 ["D(?x, ?x) = ?x",
		  "D(D(?x, ?y), ?z) = D(?x, D(?y, ?z))"])
		(IOFotrs.rdEq "D(D(D(?x, D(D(?y, ?z), ?y)), ?x), D(D(?y, ?z), ?x)) = D(D(?x, ?y), D(?y, D(D(?x,?y), D(?z, ?x))))")
      | kTest _ = [];


end
