(* Unfailing Knuth-Bendix Completion Prover *)
(* 2008.Dec.10 Pham Tuong Van *)
signature SigUKBProver = 
sig
	exception UPE
	
	(* Step 1, 2: select s = t and orient it, if ok then true, (s,t)... else false (s,t)... *)
	val orientation : (term * term -> bool)
		-> (term * term) list  * (term * term) list * (term * term) list * (term * term)
		-> bool * (term * term) * (term * term) list  * (term * term) list * (term * term) list * (term * term) 

	(* Step 4.1: If Orientable, then add criticalpair of (rule, selected equation) and (rule, rules) *)
	val deductionIfOrientable : (term * term -> bool)
		-> (term * term) * (term * term) list * (term * term) list * (term * term) list * (term * term)
		-> (term * term) * (term * term) list * (term * term) list * (term * term) list * (term * term)

	(* Step 4.2: If Unorientable, then add criticalpair of (equ, selected equation) and (equ, rules) *)             
	val deductionIfUnorientable : (term * term -> bool)
		-> (term * term) * (term * term) list * (term * term) list * (term * term) list * (term * term)
		-> (term * term) * (term * term) list * (term * term) list * (term * term) list * (term * term)             

	(* Step 5: Normalize right hand side of rules *)
	val composition : 
		(term * term) * (term * term) list * (term * term) list * (term * term) list * (term * term)
		-> (term * term) * (term * term) list * (term * term) list * (term * term) list * (term * term)		

    (* Step 6: Simplify selected equation *) 
	val simplifyE_S : 
		(term * term) list * (term * term) list * (term * term) list * (term * term)
		-> (term * term) list * (term * term) list * (term * term) list * (term * term)		
		
	(* Step 8: Normalize Equations Set and Goal *)
	val normalization :
		(term * term) list * (term * term) list * (term * term) list * (term * term)
		-> (term * term) list * (term * term) list * (term * term) list * (term * term)

	(* Step 9: Delete trivial and check Goal *)
	val deletion :
		(term * term) list * (term * term) list * (term * term) list * (term * term)
		-> (term * term) list * (term * term) list * (term * term) list * (term * term)
		
    (* Step 5.1: Delete all rules in Rules Set which new orienting rule is an intance of *)
	val collapsion :
		(term * term) * (term * term) list * (term * term) list * (term * term) list * (term * term)
		-> (term * term) * (term * term) list * (term * term) list * (term * term) list * (term * term)		

	(* Step 7: Simplify lhs is finished in deduction *)

	(* Step 3: Divide into two case *)
	val caseDivision :
		(term * term -> bool)
		-> bool * (term * term) * (term * term) list  * (term * term) list * (term * term) list * (term * term)
		-> (term * term) list * (term * term) list * (term * term) list * (term * term) 

	(* Malfuntion operation *)
		
	val ukpsub :
		(term * term -> bool)
		-> (term * term) list * (term * term) list * (term * term) list * (term * term) 
		-> (term * term) list

	val ukpstep :
		(term * term -> bool)
		-> (term * term) list * (term * term) list * (term * term) list * (term * term) 
		-> (term * term) list * (term * term) list * (term * term) list * (term * term) 
		
	(* Automatic operation *)

	val ukp :
		(term * term -> bool)
		-> (term * term) list 
		-> (term * term) 
		-> bool
		

end;

structure UP : SigUKBProver = struct
	
    open Term
	open Trs
	open Rewrite
	open Subst
	open ECr
	structure L = List
		
    exception UPE;
	
    (*   RuleSet$B$+$i!"=g=x$,$D$/5,B'$NFb!$N>JU$NJQ?t$,:G$b>/$J$$5,B'(Br$B$rA*$S!$AH(B( r, RuleSet - {r} ) $B$rJV$94X?t(B  *)
   fun orientation ord (rs, e_s, [], goal) = (print "No equation to expand\n";  raise UPE)
    |  orientation ord (rs, e_s, ess, goal) =
	   let
	       (* $B9`$NAH$ND9$$J}$N%5%$%:$r4^$s$@O"7k%j%9%H$rJV$9(B *)
	       fun makeVarNumlist rs =
		   let 
		       fun maxSize (l,r) =Int.max (termSize l, termSize r)
		   in
		       ListMergeSort.sort 
			   (fn ((n1,_),(n2,_)) => n1 > n2)
			   (L.map (fn lr => (maxSize lr, lr)) rs)
		   end
		   
		   (*   $BN>JU$NJQ?t$,:G$b>/$J$$5,B'$H$=$N;D$j$rJV$94X4X?t(B *)
	       fun leastvarbhs ord [] done = (print "No orientable equations. Goal is unsolvable\n"; raise UPE)
			 | leastvarbhs ord ((l,r)::ess) done = 
			   (* Orientable *)
			   if ord (l,r) then (true, (l,r), (rev done) @ ess )
			   else if ord (r,l) then  (true, (r,l), (rev done) @ ess)
					(* Unorientable *)
					else (false, (l,r), (rev done) @ ess)
						
		   val sorted = L.map (#2) (makeVarNumlist ess)
			   
		   val (isor,(a,b),rest) = leastvarbhs ord sorted []
	   in
		   (isor,(a,b), rs, e_s, rest, goal)
	   end		   
   
   (* Append all critical pair of r with rs, and r with e_s to ess *)
   fun deductionIfOrientable ord (r, rs, e_s, ess, goal) = 
	   (r, rs, e_s, L.@(L.@(criticalPairsKB r rs, criticalPairsUKBR_ES ord r e_s), ess), goal)
	  
   (* Append all critical pair of e with rs, and e with e_s to ess *)
   fun deductionIfUnorientable ord (e, rs, e_s, ess, goal) = 
	   (e, rs, e_s, L.@(L.@(criticalPairsUKBE_RS ord e rs, criticalPairsUKBE_ES ord e e_s), ess), goal)

   (* Normalize right hand side of rules *)
   fun composition (r, rs, e_s, ess, goal) =
	   (r, L.map (fn (a,b) => (a, linf (r::rs) b)) rs, e_s, ess, goal)
	   
   (* Simplify selected equation *)
   fun simplifyE_S (rs, e_s, ess, goal) =
	   let
		   fun equaStatusLinf R (l,r) = 
		   let 
			   val (stat1, nf1) = statusLinf R l
			   val (stat2, nf2) = statusLinf R r
		   in (stat1 orelse stat2, (nf1,nf2))
		   end
		   
		   fun dividing R e_sl es [] = (e_sl,es)
			 | dividing R e_sl es (lr::lrs) =
			   case equaStatusLinf R lr of
				   (true, nfs) => dividing R e_sl (nfs::es) lrs
				 | (false, _) => dividing R (lr::e_sl) es lrs
					   
		   val (ne_s, nes) = dividing rs [] [] e_s

	   in (rs, ne_s, L.@(nes, ess), goal)
	   end

   (* Normalize Equations Set, and goal*)
   fun normalization (rs, e_s, ess, (gl,gr)) =
	   (rs, e_s, L.map (fn (l,r) => (linf rs l, linf rs r)) ess, (linf rs gl, linf rs gr))

   (* Delete trivial and check goal*)
   fun deletion (rs, e_s, ess, goal) = 
	   if equal goal then (print "Goal is proved TRUE. "; raise UPE)
	   else (rs, e_s, L.filter (not o equal) ess, goal)

    (* delete all rules in rs of which r is an instance *)
   fun collapsion (r, rs, e_s, ess, goal) = 
		let
			fun collaplist (l,_) rs =
				let
					fun subcol l (Var _) = false
					  | subcol l (m as Fun( _ , ts , _) ) =
						case match l m of
							SOME _ => true
						  | NONE   => subcollist l ts
					and subcollist _    []   = false
					  | subcollist l (t::ts) = (subcol l t) orelse (subcollist l ts) 
				in
					L.filter (fn (m,_) => not (subcol l m)) rs
				end
	    in
              (r, collaplist r rs, e_s, ess, goal)
		end

   (* divide orientation in two cases *)
   fun caseDivision ord (false, er, rs, e_s, ess, goal) =
	   (fn (a, b, c, d, e) => (b, a::c, d, e)) (
	   deductionIfUnorientable ord (er, rs, e_s, ess, goal))
	 | caseDivision ord (true,  er, rs, e_s, ess, goal) = 
	   simplifyE_S
	   ((fn (a, b, c, d, e) => (a::b, c, d, e)) 
		(composition
		 (deductionIfOrientable ord (er, rs, e_s, ess, goal)
		  )))
	   
   
   (* Final procedure *)	   
   fun ukpsub _ (rs, e_s, [], goal) = (print (prRules rs); rs)
	 | ukpsub grter (rs, e_s, es, goal) = ukpsub grter (ukpstep grter (rs, e_s, es, goal))
   and ukpstep grter (rs, e_s, es, goal) =
	   (* Remove comment to make program display KB result step by step *)
	   let val _ = print (prRules rs)
		   val _ = print (prEqs e_s)
           val _ = print (prEqs es)
	   in
		   deletion
		   (normalization
		   (caseDivision grter 
		   (orientation grter (rs, e_s, es, goal))))
	   end

   	(* EquationSet$B7?0z?t(B es $B$N!$N>JU$,Ey$7$$$b$N$r<h$j=|$/(B *)
   fun ukp grter ess goal =
	   let
		   val result = ukpsub grter ([], [], L.filter (not o equal) ess, goal)
		   fun simpl (l,r) = (linf result l, linf result r)
		   val sgoal = simpl goal;
		   val _ = if equal sgoal then 
			   print  "TRUE.\n" else print ("FALSE. Final form: " ^ (prEq sgoal))
	   in (equal sgoal)
	   end

   handle UPE => ((print "Finish proving\n"); raise UPE)
	   

end;


