structure Pat_Test : sig
  val newAlgorithmTest : IntInf.int -> unit
  val complementTest : IntInf.int -> unit
  val main : string * string list -> OS.Process.status
end = struct

val mkTerm = let
  val env = IOMstrs.rdDecls ["tt:B", "ff:B"];
  fun rdSortedTerm str = case Term.attachSortToTerm env (IOMstrs.rdTerm str) of
			      SOME t => t
  in rdSortedTerm end
  
fun runAgcpComplementAlgorithm c n ps = Gcr.patCovered 
  [mkTerm "tt", mkTerm "ff"]
  (Pat_Complete.create_agcp_input mkTerm c n ps)
    
fun timeAlgorithm alg desc nmax = let
  val timeout = Time.fromSeconds 60;
  fun mainLoop c n = if n > nmax then if c = 15 then () else mainLoop (c + 1) 1 else let
    val _ = SMLofNJ.Internals.GC.doGC 100000;
    val configs = [c,c,c];
    val t = Timer.startRealTimer ();
    val bs = map (fn c => alg c n []) configs;
    val b = List.all (fn b => b) bs;
    val totalTime = Timer.checkRealTimer t;
    val time = Time.fromMilliseconds(Time.toMilliseconds(totalTime) div IntInf.fromInt (length configs));
    val _ = print (desc ^ "\tc:\t" ^ IntInf.toString c ^ "\tn:\t" ^ IntInf.toString(n) ^ "\tav-time:\t" ^ Time.toString(time) ^ "\tseconds\n");
    in if Time.<= (time,timeout) then mainLoop c (n + 1) else mainLoop c (nmax + 1) end      
  in mainLoop 0 1 
  end
  
val complementTest = timeAlgorithm runAgcpComplementAlgorithm "complement algorithm (AGCP)"
val newAlgorithmTest = timeAlgorithm Pat_Complete.pat_complete_alg_test "our decision procedure (SML)"

fun help () = raise Fail "invoke with arguments {complement|our} n"
  
fun main (prog_name : string, args) = 
   if length args = 2 then 
    let 
      val [algS,nS] = args
      val alg = if algS = "our" then newAlgorithmTest 
      	  else if algS = "complement" then complementTest 
      	  else help ()
      val SOME n = IntInf.fromString nS
      val _ = alg n
    in
      OS.Process.success
    end
   else help ()
  
end; (* of struct *)