(******************************************************************************
 * file: rwtools/sprint/dc.sml
 * description: divergence critic
 * author: AOTO Takahito
 * 
 * Copyright (c) 2007-2009, Toyama&Aoto Laboratory, Tohoku University
 ******************************************************************************)

signature DC = 
  sig
  datatype aterm = 
	   IVar of Term.var_key * Term.sort_key  (* wave $B$NCf(B *) 
	 | OVar of Term.var_key * Term.sort_key  (* wave $B$N30(B *) 
	 | IFun of Term.fun_key * aterm list * Term.sort_key (* wave $B$NCf(B *) 
	 | OFun of Term.fun_key * aterm list * Term.sort_key (* wave $B$N30(B *) 

  val prAterm : aterm -> string

  val differenceMatch: Term.term 
		       -> Term.term 
		       -> (aterm * Term.var_key VarMap.map) list

  val speculateLemma: Term.term * Term.term 
		      -> Term.term * Term.term 
		      -> (Term.term * Term.term) option

  val speculateSoundLemma: Term.term * Term.term 
			   -> Term.term * Term.term 
			   -> (Term.term * Term.term) option
  end;

structure Dc : DC = 
   struct

   local
       structure F = Fun
       structure V = Var
       structure VM = VarMap
       structure A = Atom
       structure L = List
       structure S = Sort
       fun mapAppend f xs = List.foldr (fn (x,ys) => List.@(f x, ys)) [] xs
       fun member x ys = isSome (List.find (fn y => x = y) ys)
       fun notMember x ys = not (member x ys)
       fun listMinus (xs,ys) =  L.filter (fn x => notMember x ys) xs
       fun deleteDuplication [] =  []
	 | deleteDuplication (x::xs) =  if member x xs then deleteDuplication xs
					else x::(deleteDuplication xs)
       open Term
   in

(***************************************)
(************** $B:9J,%^%C%A(B ***************)
(***************************************)

   datatype aterm = 
	    IVar of var_key * sort_key  (* wave $B$NCf(B *) 
	  | OVar of var_key * sort_key  (* wave $B$N30(B *) 
	  | IFun of fun_key * aterm list * sort_key (* wave $B$NCf(B *) 
	  | OFun of fun_key * aterm list * sort_key (* wave $B$N30(B *) 

   fun topBar (OVar _) = true
     | topBar (IVar _) = false
     | topBar (OFun _) = true
     | topBar (IFun _) = false

   fun topBox (OVar _) = false
     | topBox (IVar _) = true
     | topBox (OFun _) = false
     | topBox (IFun _) = true

   fun sortOfATerm (OVar (_,ty)) = ty
     | sortOfATerm (IVar (_,ty)) = ty
     | sortOfATerm (OFun (_,_,ty)) = ty
     | sortOfATerm (IFun (_,_,ty)) = ty

  (* Anotated term $B$+$iJ8;zNs$X(B *)
  (* wave $B$NCf$N%7%s%\%k$K$O(B*$B$rIU$1$FI=<((B *)
   fun prAterm (OVar (x,_)) = 
       let val s = A.toString (#1 x)
	   val n = #2 x
       in  if n = 0 
	   then ("?"^s) 
	   else ("?"^s^(Int.toString n))
       end
     | prAterm (IVar (x,_)) = 
       let val s = A.toString (#1 x)
	   val n = #2 x
       in  if n = 0 
	   then ("?"^s ^ "*") 
	   else ("?"^s^(Int.toString n)^ "*") 
       end
     | prAterm (OFun (f,ts,_)) = 
       if (null ts) 
       then (A.toString f)
       else ((A.toString f) ^ "(" ^ (prAterm (L.hd ts)) ^ (prAterm' (L.tl ts)) ^ ")")
     | prAterm (IFun (f,ts,_)) = 
       if (null ts) 
       then (A.toString f) ^ "*"
       else ((A.toString f) ^ "*" ^ "(" ^ (prAterm (L.hd ts)) ^ (prAterm' (L.tl ts)) ^ ")")
   and prAterm' [] = ""
     | prAterm' (t0::ts) = "," ^ (prAterm t0) ^ (prAterm' ts)


   (* annotation $B$N=|5n(B *)
   fun erase (IVar (x,ty)) = Var (x,ty)
     | erase (OVar (x,ty)) = Var (x,ty)
     | erase (IFun (f,ts,ty)) = Fun (f,map erase ts,ty)
     | erase (OFun (f,ts,ty)) = Fun (f,map erase ts,ty)

   (* Box annotation $B$NIU2C(B *)
   fun annotateBox (Var (x,ty)) = IVar (x,ty)
     | annotateBox (Fun (f,ts,ty)) = IFun (f,map annotateBox ts,ty)

   (* Bar annotation $B$NIU2C(B *)
   fun annotateBar (Var (x,ty)) = OVar (x,ty)
     | annotateBar (Fun (f,ts,ty)) = OFun (f,map annotateBar ts,ty)

   (* $B@5$7$/(B annotation $B$,IU$$$F$$$k$+!'(Bbox $B$NCf$K$OCzEY(B 1$B$D(B bar $B$,$"$k(B *)
   fun wellAnnotated (OVar _) = true
     | wellAnnotated (IVar _) = false  (* bar $B$,4^$^$l$F$$$J$$(B *)
     | wellAnnotated (OFun (f,ts,ty)) = List.all wellAnnotated ts
     | wellAnnotated (IFun (f,ts,ty)) = containsOneBarList ts
   and containsOneBar (OVar _) = true
     | containsOneBar (IVar _) = false
     | containsOneBar (OFun (f,ts,ty)) = List.all wellAnnotated ts
     | containsOneBar (IFun (f,ts,ty)) = containsOneBarList ts
   and containsNoBar (OVar _) = false
     | containsNoBar (IVar _) = true
     | containsNoBar (OFun (f,ts,ty)) = false
     | containsNoBar (IFun (f,ts,ty)) = List.all containsNoBar ts
   and containsOneBarList [] = false
     | containsOneBarList (t::ts) = 
       ((containsOneBar t) andalso (List.all containsNoBar ts))
       orelse ((containsNoBar t) andalso (containsOneBarList ts))
   and containsNoBox (OVar _) = true
     | containsNoBox (IVar _) = false
     | containsNoBox (OFun (f,ts,ty)) = List.all containsNoBox ts
     | containsNoBox (IFun (f,ts,ty)) = false


   (* skelton $B$NCj=P(B($B@.8y$9$l$P(B)$B!$(BwellAnnotation term $B$H2>Dj$9$k(B *)
   fun skelton t =
       let exception notWellAnnotated
	   fun skel (IVar (x,ty)) = raise notWellAnnotated
	     | skel (OVar (x,ty)) = Var (x,ty)
	     | skel (IFun (f,ts,ty)) = skelList ts 
	     | skel (OFun (f,ts,ty)) = Fun (f,map skel ts,ty)
	   and skelList [] = raise notWellAnnotated
	     | skelList (t::ts) = 
	       skel t (* $B:G:8$N(B skelton $B$r<h$j=P$9!$;D$j$OHs%A%'%C%/(B *)
	       handle notWellAnnotated => skelList ts
       in
	   SOME (skel t)
	   handle notWellAnnotated => NONE
       end

   (*  maximal differnce match *)     
   fun differenceMatch skel term = 
       let fun dm sub (Var (x,ty1), Var (y,ty2)) =
	       (case VM.find (sub,x) of
		   SOME z => if (Var.equal (z,y))
			     then [(OVar (y,ty2),sub)]
			     else []
		 | NONE => if (S.equal (ty1, ty2))
			   then [(OVar (y,ty2), VM.insert(sub,x,y))]
			   else [])
	     | dm sub (s1 as Var _, Fun (g,ts,ty)) = 
	       L.map (fn (us,sub') => (IFun (g,us,ty), sub')) (dmOne sub (s1,ts))
	     | dm sub (s1 as Fun (f,ss,ty1), Fun (g,ts,ty2)) = 
	       (* box $B$,$h$j>e$K$J$k>H9g$rM%@h(B *)
	       L.@(L.map (fn (us,sub') => (IFun (g,us,ty2), sub')) 
			 (dmOne sub (s1,ts)),
		   if Fun.equal (f,g)
		   then L.map (fn (us, sub') => (OFun (g,us,ty2), sub'))
			      (dmList sub (ListPair.zip (ss,ts)))
		   else [])
	     | dm sub (Fun _, Var _) = []
	   and dmList sub [] = [([], sub)]
	     | dmList sub (p::ps) =
	       mapAppend (fn (u1,sub') => L.map (fn (us, sub'') => (u1::us, sub''))
						  (dmList sub' ps))
			   (dm sub p)
	   and dmOne sub (skel, []) = []
	     | dmOne sub (skel, t1::ts) =
	       L.@(L.map (fn (u1, sub') => (u1::(map annotateBox ts), sub'))
			 (dm sub (skel, t1)),   (* $B$h$j:8$N>H9g$rM%@h(B *)
		   L.map (fn (us, sub'') => ((annotateBox t1)::us, sub''))
			 (dmOne sub (skel, ts)))
       in
	   dm VM.empty (skel, term)
       end

(*  C1[C2*[u]] => (C1, C2, u) *)
   fun decompose (OVar _) = NONE
     | decompose (IVar _) = NONE
     | decompose (OFun (f,ts,ty)) = 
       (case (decomposeList ts) of
	   SOME (alpha, beta, u) => SOME (fn x => OFun (f,alpha x,ty), beta, u)
	 | NONE => NONE)
     | decompose (IFun (f,ts,ty)) = 
       case (decompose2List ts) of 
	   SOME (beta,u) => SOME (fn x => x, fn y => IFun (f,beta y,ty), u)
	 | NONE => NONE
   and decomposeList [] = NONE
     | decomposeList (t::ts) = 
       if containsNoBox t 
       then case (decomposeList ts) of
		SOME (alpha, beta,u) => SOME (fn x => t::(alpha x), beta, u)
	      | NONE => NONE
       else case (decompose t) of
		SOME (alpha, beta,u) => if List.all containsNoBox ts 
				      then SOME (fn x => (alpha x)::ts, beta,u)
				      else NONE
	      | NONE => NONE
   and decompose2 (u as (OVar (_,ty))) = SOME (fn x => x, u)
     | decompose2 (IVar _) = NONE
     | decompose2 (u as (OFun (f,ts,ty))) = 
       if List.all containsNoBox ts
       then SOME (fn x => x, u)
       else NONE
     | decompose2 (IFun (f,ts,ty)) = 
       case (decompose2List ts) of
	   SOME (beta, u) => SOME (fn x => IFun (f,beta x,ty), u)
	 | NONE => NONE
   and decompose2List [] = NONE
     | decompose2List (t::ts) = 
       if containsNoBar t 
       then case (decompose2List ts) of
		SOME (beta,u) => SOME (fn x => t::(beta x), u)
	      | NONE => NONE
       else case (decompose2 t) of
		SOME (beta,u) => if List.all containsNoBar ts 
			     then SOME (fn x => (beta x)::ts, u)
			     else NONE
	      | NONE => NONE


   (*  $BH/;64UDjK!$K$h$kJdBj@8@.!$(B $B7?%A%'%C%/$O$^$@(B *)    
   (*  l1,r2 : skelton,  l2,r2: term *)    
   fun speculateLemma (l1,r1) (l2,r2) = 
       let 
	   val m  = Term.maxVarIndexInTerms [l2,r2]
	   val x = (A.atom "x", m + 1)
(*	   val v = IVar (x, S.null) *)  (* fresh variable *)
(*	   val v = IVar (x, S.fromString "A")   *) (* fresh variable *)

	   val diffs = differenceMatch  
			  (Fun (A.atom "$eq", [l1,r1],S.null))
			  (Fun (A.atom "$eq", [l2,r2],S.null))
	   fun getOVarSym (OVar (y,_)) = SOME y
	     | getOVarSym _ = NONE

	   (* $B4pK\E*$JH/;64UDjK!$N$_(B *)
	   fun speculateLemmaSub0 (OFun (_,[la,ra],_), _) = 
	       case (decompose la, decompose ra) of
		   (SOME (C_1,C_2,u1), SOME(C_3,C_4,u2)) 
		   => let val s1 = sortOfATerm u1
			  val s2 = sortOfATerm u2
			  val v = IVar (x, s1)
		      in if Sort.equal (s1,s2)
			 then
			     if topBox ra 
			     then SOME (erase (C_1 (C_2 v)), erase (C_4 (C_1 v)))
			     else if topBox la
			     then SOME (erase (C_2 (C_3 v)), erase (C_3 (C_4 v)))
			     else NONE
			 else NONE
		      end
		 | _ => NONE

       (* Sideway Rippling $B$b4^$`H/;64UDjK!(B *)
	   fun speculateLemmaSub (OFun (_,[la,ra],_), _) = 
	       (case (decompose la, decompose ra) of
		   (SOME (C_1,C_2,u1), SOME(C_3,C_4,u2)) 
		   => 
		   let 
		       (* val _ = print ("aa" ^ "\n"
					 ^ (prAterm (C_1 v)) ^ "\n"
					 ^ (prAterm (C_2 v)) ^ "\n"
					 ^ (prAterm (C_3 v)) ^ "\n"
					 ^ (prAterm (C_4 v)) ^ "\n") ; *)
		       val s1 = sortOfATerm u1
		       val s2 = sortOfATerm u2
		       val v = IVar (x, s1)
		   in if not (Sort.equal (s1,s2))
		      then NONE
		      else
			  (* divergence critic *)
	       		  if topBox ra 
			  then SOME (erase (C_1 (C_2 v)), erase (C_4 (C_1 v)))
			  else if topBox la
			  then SOME (erase (C_2 (C_3 v)), erase (C_3 (C_4 v)))
			  (* accumlater $B$N$"$k(B sideway rippling *)
			  else 
			      (let 
				   val vts1 = Term.varSubterms (erase (C_1 v))
				   val vts3 = Term.varSubterms (erase (C_3 v))
 			       (* val _ = print ("ab" ^ "\n")  *)
			       in case L.find 
					   (fn (Var (sy,ty)) => 
					       Term.equal (Subst.applySubst 
							    (VM.singleton (sy, 
									   erase (C_4 (IVar (sy,ty)))))
							    (erase u2),
							erase (C_4 u2)))
					(termListMinus (vts1,vts3)) of
				   SOME (y as Var (sy,ty)) 
				   => SOME (erase (C_1 (C_2 v)), 
					    Subst.applySubst 
						(VM.singleton (sy, erase (C_4 (IVar (sy,ty)))))
						(erase (C_1 v)))
				 | _
				   => (case L.find 
						(fn (Var (sy,ty)) => 
						    Term.equal 
							(Subst.applySubst 
							     (VM.singleton (sy, 
									    erase (C_2 (IVar (sy,ty)))))
							     (erase u1),
							 erase (C_2 u1)))
						(termListMinus (vts3,vts1)) of
					   SOME (Var (sy,ty)) 
					   => SOME (erase (C_3 (C_4 v)), 
						    Subst.applySubst 
							(VM.singleton (sy, erase (C_2 (IVar (sy,ty)))))
							(erase (C_3 v)))
					 | _ => NONE)
			       end)
		   end
		 (* accumlater $B$N$J$$(B sideway rippling *)
		  | (SOME (C_1,C_2,u1), NONE)
		    => 
		    let (* val _ = print ("bb" ^ (prAterm ra) ^ (prAterm la)) *)
			val s1 = sortOfATerm u1
			val v = IVar (x, s1)
		    in
			if containsNoBox ra
			then SOME (erase (C_1 (C_2 v)), erase (C_1 v))
			else NONE
		    end
		  | (NONE, SOME (C_3,C_4,u2))
		   => 
		   let (* val _ = print ("cc" ^ (prAterm la)) *)
		       val s2 = sortOfATerm u2
		       val v = IVar (x, s2)
		   in
		       if containsNoBox la
		       then SOME (erase (C_3 (C_4 v)), erase (C_3 v))
		       else NONE
		   end
		  | (NONE,NONE) => ((* print "dd\n"; *) NONE))
				   
	     | speculateLemmaSub _ = NONE
				     
 	   val ans = L.mapPartial speculateLemmaSub diffs 

(*	   val ans = L.mapPartial speculateLemmaSub0 diffs    *)
       (* $BV:DE$/$sO@J8MQ(B *)							 

       in
		   if null (ans) 
			   then NONE
		   else ((* print ("DC: " ^ Int.toString (length ans)); *) SOME (hd ans))
       end


   (*  $B7rA4H/;64UDjK!$K$h$kJdBj@8@.(B *)    
   fun speculateSoundLemma (l1,r1) (l2,r2) = 
       let 
	   (* val _ = print ("Speculate: " ^ (Trs.prEq (l1,r1)) ^ "\n") *)
	   (* val _ = print ("       and " ^ (Trs.prEq (l2,r2)) ^ "\n") *)

	   val diffs = differenceMatch  
			  (Fun (A.atom "$eq", [l1,r1],S.null))
			  (Fun (A.atom "$eq", [l2,r2],S.null))

	   fun isIVar (IVar _) = true
	     | isIVar _ = false

	   fun modification (OFun (_,[la,ra],_), _) = 
	       let 
(* 		   val _ = print ((prAterm la) ^ ":"); *)
(* 		   val _ = print ((prAterm ra) ^ "\n"); *)
		   val l1' = valOf (skelton la)
		   val r1' = valOf (skelton ra)
		   val l2' = erase (la)
		   val r2' = erase (ra)
	       in
		   case (decompose la, decompose ra) of
		       (SOME (C_1,C_2,u1), SOME(C_3,C_4,v1)) =>
	       	       if topBox ra
		       then let val v2 = valOf (Rewrite.rootRewrite [(r1',l1')] (erase v1))
			    in SOME (l2', erase (C_4 (annotateBox v2)))
			    end
		       else if topBox la
		       then let val u2 = valOf (Rewrite.rootRewrite [(l1',r1')] (erase u1))
			    in SOME (erase (C_2 (annotateBox u2)), r2')
			    end
		       else if isIVar v1
		       then let val r3 = valOf (Rewrite.rootRewrite [(r1',l2')] r2')
			    in SOME (l2', r3)
			    end
		       else if isIVar u1
		       then let val l3 = valOf (Rewrite.rootRewrite [(l1',r1')] l2')
			    in SOME (l3, r2')
			    end
		       else NONE
		     | (NONE, SOME (C_3,C_4,v1)) 
		       => if topBox ra 
			  then let val v2 = valOf (Rewrite.rootRewrite [(r1',l1')] (erase v1))
			       in SOME (l2, erase (C_4 (annotateBox v2)))
			       end
			  else if isIVar v1
			  then let val r3 = valOf (Rewrite.rootRewrite [(r1',l1')] r2')
			       in SOME (l2',r3)
			       end
			  else if containsNoBox la
			  then let val l3 = valOf (Rewrite.rootRewrite [(l1',r1')] l2')
			       in SOME (l3,r2')
			       end 
			  else NONE
		     | (SOME (C_1,C_2,u1), NONE)
		       => if topBox la
			  then let val u2 = valOf (Rewrite.rootRewrite [(l1',r1')] (erase u1))
			       in SOME (erase (C_2 (annotateBox u2)), r2')
			       end
			  else if isIVar u1
			  then let val l3 = valOf (Rewrite.rootRewrite [(l1',r1')] l2')
			       in SOME (l3,r2')
			       end
			  else if containsNoBox ra
			  then let val r3 = valOf (Rewrite.rootRewrite [(r1',l1')] r2')
			       in SOME (l2',r3)
			       end 
			  else NONE
		     | (NONE,NONE) => NONE
	       end

	   exception Found of (Term.term * Term.term)

       in
	   (L.app (fn df => case modification df of
				SOME e' => raise (Found e')
			      | NONE => ())
		  diffs; NONE)
	   handle (Found e') => ( (* print ("Found?: " ^  (Trs.prEq e') ^ "\n"); *) SOME e')
       end


   end (* of local *)
   end; (* of structure *)
