(******************************************************************************
 * file: rwtools/sprint/sg.sml
 * description: sound generalization 
 * author: AOTO Takahito
 * 
 * Copyright (c) 2007-2009, Toyama&Aoto Laboratory, Tohoku University
 ******************************************************************************)

signature SG = 
  sig
  val transitiveClosure:  SortPairSet.set -> SortPairSet.set option
(*  val hasMonomorphicSig: (Term.term * Term.term) list -> bool *)
  type argTables = {RA:int FunMap.map, 
		    DP:int FunMap.map, 
		    UP:int FunMap.map,
		    DCP:int FunMap.map,
		    UCP:int FunMap.map}
  val nullTables: argTables
  val getMonomorphicSortSet: Term.decl list -> Trs.trs -> SortSet.set * argTables

  val upPosLemmas: argTables -> Trs.trs -> (Term.term * Term.term) list
  val soundGeneralization: argTables -> (Term.term * Term.term) -> (Term.term * Term.term) option
  val soundGeneralization2: argTables
			    -> (Term.term * Term.term -> bool) 
			    -> (Term.term * Term.term -> bool) 
			    -> (Term.term * Term.term) 
			    -> (Term.term * Term.term) option
  val termPartition: (Term.sort_key list * Term.sort_key) FunMap.map
		     -> Trs.trs -> argTables -> Term.term 
		     -> (Term.position * Term.term * Term.term) list
  end;

structure Sg : SG = 
struct

local
    structure A = Atom
    structure AT = AtomTable  (* predefined in SML/NJ lib *)
    structure F = Fun
    structure FM = FunMap
    structure FS = FunSet
    structure V = Var
    structure VM = VarMap
    structure VS = VarSet
    structure S = Sort
    structure SPS = SortPairSet
    structure SS = SortSet
    structure L = List
    structure LP = ListPair
    fun mapAppend f xs = List.foldr (fn (x,ys) => List.@(f x, ys)) [] xs
    fun replaceNth (xs,i,y) = (List.take (xs,i)) @ [y] @ (List.drop (xs,i+1))
    open Term;
in

type argTables = {RA:int FunMap.map, 
		  DP:int FunMap.map, 
		  UP:int FunMap.map,
		  DCP:int FunMap.map,
		  UCP:int FunMap.map}

val nullTables = {RA = FM.empty,
		  DP = FM.empty,
		  UP = FM.empty,
		  DCP = FM.empty,
		  UCP = FM.empty}

fun transitiveClosure precset =
    let
	fun transCls ([], yset) = SOME yset
	  | transCls ((a,b)::xs, yset) =
	    let fun addNeighbour (a,b) (x,y) = 
		    if S.equal (x, b) then SOME (a,y) 
		    else if S.equal (y,a) then SOME (x,b) 
		    else NONE
	    in
		if SPS.member (yset, (a,b))
		then transCls (xs, yset)
		else if SPS.member (yset, (b,a))
		then NONE
		else case (transCls (List.mapPartial (addNeighbour (a,b)) 
						     (SPS.listItems yset),
				     yset))
		      of SOME zset => transCls (xs, SPS.add (zset, (a,b)))
		       | NONE => NONE
	    end
    in
	transCls (List.filter (fn (x,y) => not (S.equal (x,y)))
			      (SPS.listItems precset), 
		  SPS.empty)
    end
  

(*    fun getMonomorphicSortSet (decls:Term.decl list) (trs:Trs.trs) =  *)
(*        let val rs = #Rules trs *)
(* 	   val sortSet = Trs.sortSetInRules rs *)
(* (\*	   val sortedFunSet = Trs.signatureInRules rs *\) *)
(* (\* 	   val _ = print (PrintUtil.prList (Atom.toString o #1) (FM.listItemsi sortedFunSet)) *\) *)
(* 	   val funSet = #FSymSet trs *)
(* 	   val dFunSet = #DSymSet trs *)
(* 	   val cFunSet = #CSymSet trs *)

(* 	   val sortedFunSet = L.foldr (fn (x,fm) => FM.insert (fm,#sym x, *)
(* 							       case (#sort x) of *)
(* 								   Sort.Base a => ([],Sort.Base a) *)
(* 								 | Sort.Proc (tys,ty) => (tys,ty))) *)
(* 				      FM.empty decls *)


(* 	   val sortedCFunSet =  *)
(* 	       FM.filteri (fn (key,_) => FS.member (cFunSet,key)) sortedFunSet *)
(* 	   val sortedConstList =  *)
(* 	       FM.listItemsi *)
(* 		   (FM.filteri (fn (_,(tys,_)) => null tys) sortedFunSet) *)
(* (\* 	   val _ = print (PrintUtil.prList (Atom.toString o #1) sortedConstList) *\) *)
(* 	   val sortedNonConstCFunList =  *)
(* 	       FM.listItemsi *)
(* 		   (FM.filteri (fn (_,(tys,_)) => not (null tys)) sortedCFunSet) *)
(* 	   fun hasDuplication [] = false *)
(* 	     | hasDuplication (x::xs) =  *)
(* 	       (List.exists (fn y => S.equal (x,y)) xs) *)
(* 	       orelse (hasDuplication xs) *)
(* 	   fun hasOneOccurrence x [] = false *)
(* 	     | hasOneOccurrence x (y::ys) =  *)
(* 	       if S.equal (x,y)  *)
(* 	       then not (List.exists (fn z => S.equal (x,z)) ys) *)
(* 	       else hasOneOccurrence x ys *)
(* 	   val mprecs = FM.foldr (fn ((tys,ty), precs)  *)
(* 				     =>  SPS.addList(precs, *)
(* 						     List.map (fn ty2 => (ty2,ty)) tys)) *)
					       
(* 				 SPS.empty *)
(* 				 sortedCFunSet *)
(* 	 (\*   val cps = Cr.criticalPairs rs *\) *)
(*        in *)
(* 	   SS.filter (fn ty => *)
(* 			 (length (List.filter (fn (_,(_,ty2)) => Sort.equal (ty,ty2)) *)
(* 					      sortedConstList) *)
(* 			  = 1) *)
(* 			 andalso (List.all  *)
(* 				      (fn (_,(tys,ty2)) => not (Sort.equal (ty2,ty)) *)
(* 							   orelse (hasOneOccurrence ty2 tys)) *)
(* 				      sortedNonConstCFunList)) *)
(* 		     sortSet *)
(*        end *)


(*    fun hasMonomorphicSig rs = *)
(*        let val sortSet = Trs.sortSetInRules rs *)
(* 	   val sortedFunSet = Trs.signatureInRules rs *)
(* (\* 	   val _ = print (PrintUtil.prList (Atom.toString o #1) (FM.listItemsi sortedFunSet)) *\) *)
(* 	   val (funSet,dFunSet,cFunSet,_) = Trs.fdcSetInRules rs *)
(* 	   val sortedCFunSet =  *)
(* 	       FM.filteri (fn (key,_) => FS.member (cFunSet,key)) sortedFunSet *)
(* 	   val sortedConstList =  *)
(* 	       FM.listItemsi *)
(* 		   (FM.filteri (fn (_,(tys,_)) => null tys) sortedFunSet) *)
(* (\* 	   val _ = print (PrintUtil.prList (Atom.toString o #1) sortedConstList) *\) *)
(* 	   val sortedNonConstCFunList =  *)
(* 	       FM.listItemsi *)
(* 		   (FM.filteri (fn (_,(tys,_)) => not (null tys)) sortedCFunSet) *)
(* 	   fun hasDuplication [] = false *)
(* 	     | hasDuplication (x::xs) =  *)
(* 	       (List.exists (fn y => S.equal (x,y)) xs) *)
(* 	       orelse (hasDuplication xs) *)
(* 	   fun hasOneOccurrence x [] = false *)
(* 	     | hasOneOccurrence x (y::ys) =  *)
(* 	       if S.equal (x,y)  *)
(* 	       then not (List.exists (fn z => S.equal (x,z)) ys) *)
(* 	       else hasOneOccurrence x ys *)
(* 	   val mprecs = FM.foldr (fn ((tys,ty), precs)  *)
(* 				     =>  SPS.addList(precs, *)
(* 						     List.map (fn ty2 => (ty2,ty)) tys)) *)
					       
(* 				 SPS.empty *)
(* 				 sortedCFunSet *)
(* 	   val cps = Cr.criticalPairs rs *)
(*        in *)
(* 	   isSome (transitiveClosure mprecs) *)
(* 	   andalso not (hasDuplication (List.map (fn (_,(_,ty)) => ty)  sortedConstList)) *)
(* 	   andalso (List.all  *)
(* 		       (fn (_,(tys,ty)) => hasOneOccurrence ty tys) *)
(* 		       sortedNonConstCFunList) *)
(* 	   andalso Trs.areLeftLinearRules rs *)
(* 	   andalso Cr.isLocalConfluent rs *)
(*        end *)


   fun makeDpMap raMap rules = 
       let
	   val sigMap = Trs.signatureInRules rules
 	   val (funSet,dFunSet,cFunSet,_) = Trs.fdcSetInRules rules
	   val dFunWithArityList = 
	       FM.listItems
		   (FM.mapPartiali (fn (f,(tys,_)) => 
				       if FS.member (dFunSet, f)
				       then SOME (f,length tys)
				       else NONE)
				   sigMap)

	   val dpPosCandidates =
	       L.filter
		   (fn (f,i)
		       => L.all (fn (l as (Fun (g,ts,_)),r) =>
				    (not (Fun.equal (f,g)))
				    orelse case L.nth (ts,i-1) of
					       Var (x,_) => VS.member (linearVarSetInTerm r,x)
							    andalso VS.member (linearVarSetInTerm l,x)
					     | _ => false)
				rules)
		   (mapAppend
			(fn (f,n) => L.tabulate (n, fn i => (f,i+1)))
			dFunWithArityList)

	   val dpMapCands = foldr (fn ((f,i),maps) =>
				     mapAppend
				     (fn m => case FM.find (m,f) of
						  SOME j => [m, FM.insert(m,f,i)]
						| NONE=> [FM.insert(m,f,i),m])
				     maps)
				 [FM.empty]
				 dpPosCandidates

	   fun isCompatibleDpMap dpMap =
	       let
 		   fun checkDpMapRule dpMap (f,n) (l as Fun (g,ts,_), r) = 
		       not (Fun.equal (f,g))
		       orelse
		       let
			   fun check (f,n) (u, r as (Var _)) = Term.equal (u,r)
			     | check (f,n) (u, r as (Fun (g,ts,_))) =
			       Term.equal (u,r)
			       orelse  if (Fun.equal (f,g)) then
					   check (f,n) (u, L.nth(ts,n-1))
				       else
					   case FM.find (raMap,g) of
					       SOME i =>  check (f,n) (u, L.nth(ts,i-1))
					     | NONE => case FM.find (dpMap,g) of
							   SOME i =>  check (f,n) (u, L.nth(ts,i-1))
							 | NONE => false
		       in
			   case L.nth (ts,n-1) of
 			       Var (x,_) => check (f,n) (L.nth (ts,n-1), r)
		             | _ => false
		       end
	       in
		   L.all 
		       (fn (f,n) => L.all (checkDpMapRule dpMap (f,n)) rules)
		       (FM.listItemsi dpMap)
	       end

	   fun selectMaximal [] (_,m) = m
	     | selectMaximal (m2::ms) (len1,m1) = 
	       let val len2 = FM.numItems m2
	       in if len2 > len1
		  then selectMaximal ms (len2,m2)
		  else selectMaximal ms (len1,m1)
	       end

       in
	   case (L.filter isCompatibleDpMap dpMapCands) of
	       [] => FM.empty
	     | (m::ms) => selectMaximal ms (FM.numItems m,m)
       end


(* 	   local *)
(* 	       exception NeedComputeFirst of Fun.ord_key *)
(* 	       exception GaveUp *)
(* 	       exception CheckLater *)
(* 	   in *)
(* 	   fun checkDpMapRule n (l as Fun (f,ts,_), r) dpMap stack dfarlist = *)
(* 	       let *)
(* 		   val _ = print ("Fun&Arg: " ^ (Fun.toString f) ^ ", ") *)
(* 		   val _ = print ((Int.toString n) ^ "\n") *)
(* 		   fun check (f,n) (u, r as (Var _)) = Term.equal (u,r) *)
(* 		     | check (f,n) (u, r as (Fun (g,ts,_))) = *)
(* 		       Term.equal (u,r)  *)
(* 		       orelse  if (Fun.equal (f,g)) then *)
(* 				   check (f,n) (u, L.nth(ts,n-1)) *)
(* 			       else *)
(* 				   case FM.find (raMap,g) of *)
(* 				       SOME i =>  check (f,n) (u, L.nth(ts,i-1)) *)
(* 				     | NONE => case FM.find (dpMap,g) of *)
(* 						   SOME i =>  check (f,n) (u, L.nth(ts,i-1)) *)
(* 						 | NONE => if L.exists  *)
(* 								  (fn (h,_) => Fun.equal (h,g)) *)
(* 								  (stack @ dfarlist) *)
(* 							   then raise (NeedComputeFirst g) *)
(* 							   else false *)
(* 	       in *)
(* 		   case L.nth (ts,n-1) of  *)
(*  		       Var (x,_) => VS.member (linearVarSetInTerm r,x) *)
(* 				    andalso VS.member (linearVarSetInTerm l,x) *)
(* 		                    andalso (check (f,n) (L.nth (ts,n-1), r)) *)
(*  		     | _ => false  *)
(* 	       end *)
	       
(* 	   val MaxForMutualDef = 3 *)

(* 	   fun computeDpForMutualDefinition dpMap stack dfarlist = *)
(* 	       let val xss = L.map (fn (f,n) => L.tabulate (n, fn i => (f,i))) *)
(* 				   stack *)
(* 	           fun checkFunsInRules [] = () *)
(* 	             | checkFunsInRules ((l as Fun (g,ts,_), r)::rs)  =  *)
(* 		       case L.partition (fn (h,i) => Fun.equal (g,h)) stack of *)
(* 			   ([(h,i)],_) => checkFunsInTerms (L.take (ts,i)) rs *)
(* 			 | _ => checkFunsInRules rs *)
(* 	           and checkFunsInTerms [] rs = checkFunsInRules rs *)
(* 	             | checkFunsInTerms (t::ts) rs = checkFunsInTerm t rs ts *)
(* 	           and checkFunsInTerm (t as Fun (g,us,_)) rs ts =  *)
(* 		       case FM.find (raMap,g) of *)
(* 			   SOME i => checkFunsInTerm (L.nth (ts,i-1)) rs ts *)
(* 			 | NONE => case FM.find (dpMap,g) of *)
(* 				       SOME i => checkFunsInTerm (L.nth (ts,i-1)) rs ts *)
(* 				     | NONE => case L.partition (fn (h,i) => Fun.equal (g,h)) stack of *)
(* 						   ([(h,i)],_) *)
(* 						   => checkFunsInTerms ((L.take (us,i)) @ ts)  rs *)
(* 						 | _ (\* g doesn't appear in stack *\)  *)
(* 						   => if L.exists (fn (h,_) => Fun.equal (g,h)) dfarlist *)
(* 						      then raise CheckLater *)
(* 						      else checkFunsInTerms ts rs *)

(* 		   val _ = checkFunsInRules rules   (\* dfarlist $B$K$"$k4X?t>pJs$rMQ$$$J$$$+%A%'%C%/(B *\) *)
(* 	       in *)
(* 		   if (length stack) > MaxForMutualDef *)
(* 		   then raise GaveUp *)
(* 		   else raise GaveUp *)
(* 	       end *)

(* 	   fun computeDpRecursively dpMap [] [] =  dpMap *)
(* 	     | computeDpRecursively dpMap stack [] =  computeDpRecursively dpMap [] stack *)
(* 	     | computeDpRecursively dpMap stack ((f,n)::dfarlist) = *)
(* 	       (if n = 0 then (\* f $B$N(B downward position $BB8:_$;$:(B *\) *)
(* 		   computeDpRecursively dpMap stack dfarlist *)
(* 	       else if (L.all *)
(* 			   (fn (l as Fun (g,_,_),r)  *)
(* 			       => (not (Fun.equal (f,g))) *)
(* 				  orelse checkDpMapRule n (l,r) dpMap stack dfarlist) *)
(* 			   rules) then *)
(* 		   computeDpRecursively (FM.insert (dpMap, f, n)) []  (dfarlist @ stack) *)
(* 	       else (\* n $B$O(B f $B$N(B dp $B$N8uJd$G$O$J$$(B *\) *)
(* 	       	   computeDpRecursively dpMap stack ((f,n-1)::dfarlist)) *)
(* 	       (\* f $B0J30$N$N(B dp $B$N8uJd$G$O$J$$(B *\) *)
(* 	       handle (NeedComputeFirst g)  *)
(* 		      => if (L.exists (fn (h,_) => Fun.equal (g,h)) stack) then *)
(* 			     (print "mutual defintion found\n"; *)
(* 			     computeDpForMutualDefinition dpMap ((f,n)::stack) dfarlist *)
(* 			     handle CheckLater => computeDpRecursively dpMap [] (dfarlist @ ((f,n)::stack))) *)
(* 			     handle GaveUp => (print "Gave up computing down positions"; *)
(* 					       print " for mutually defined functions\n"; *)
(* 					       computeDpRecursively dpMap [] dfarlist) *)
(* 			 else *)
(* 			     computeDpRecursively dpMap ((f,n)::stack) dfarlist *)
(* 	   end (\* of local *\) *)
			    
(*        in *)
(* 	   computeDpRecursively FM.empty [] dFunWithArityList *)
(*        end *)


   fun makeUpMap raMap rules = 
       let
	   val sigMap = Trs.signatureInRules rules
 	   val (funSet,dFunSet,cFunSet,_) = Trs.fdcSetInRules rules
	   val cFunSigMap = 
	       FM.filteri (fn (key,_) => FS.member (cFunSet,key)) sigMap
	   val cConstSigMap = 
	       FM.filteri (fn (_,(tys,_)) => null tys) cFunSigMap

	   val dFunWithArityList = 
	       FM.listItems
		   (FM.mapPartiali (fn (f,(tys,_)) => 
				       if FS.member (dFunSet, f)
				       then SOME (f,length tys)
				       else NONE)
				   sigMap)

	   fun isCConstOfSpecifiedType (Var _,_) = false
	     | isCConstOfSpecifiedType (Fun (c,ts,ty1), ty2) = 
	       (null ts) 
	       andalso S.equal (ty1,ty2)
	       andalso FM.inDomain (cConstSigMap, c)


	   fun isAConstructorTerm t = FS.isSubset (funSetInTerm t,cFunSet) 

	   local
	       exception NotCContext
	   in
	   fun isContextPreserving ((l as Fun (f,ts,ty)), i, r) =
	       let
		   val lp =  L.nth (ts, i-1)
		   fun divLp (x as Var _) = (fn y => y, x)
		     | divLp (Fun (g,ss,ty')) = 
		       case FM.find (raMap,g) of
			   SOME j => let val (hMap,u) = divLp (L.nth (ss,j-1))
				     in (fn y => Fun (g,replaceNth (ss,j-1,hMap y),ty'), u)
				     end
			 | NONE => raise NotCContext
		   val (cMap,xVar) = divLp lp
	       in
		   Term.equal (r, cMap (Fun (f,replaceNth (ts,i-1,xVar),ty)))
	       end (* of let *)
	       handle NotCContext => false
	   end (* of local *)

	   val upPosCandidates = 
	       L.filter 
		   (fn (f,i) 
		       => L.all 
			       (fn (l as Fun (g,ts,ty),r) 
				   => (not (Fun.equal (f,g))) 
				      orelse isCConstOfSpecifiedType (L.nth (ts,i-1), ty)
				      orelse (isAConstructorTerm (L.nth (ts,i-1))
					      andalso isContextPreserving (l, i, r)))
			       rules)
		   (mapAppend
			(fn (f,n) => L.tabulate (n, fn i => (f,i+1)))
			dFunWithArityList)

	   val upMapCands = foldr (fn ((f,i),maps) => 
				     mapAppend
				     (fn m => case FM.find (m,f) of
						  SOME j => [m, FM.insert(m,f,i)]
						| NONE=> [FM.insert(m,f,i)])
				     maps)
				 [FM.empty]
				 upPosCandidates

	   fun selectOne [] = FM.empty 
	     | selectOne (m::ms) = m

	   in
	   selectOne upMapCands
       end


   fun makeCpMaps raMap upMap dpMap monoSortSet rules = 
       let
	   val sigMap = Trs.signatureInRules rules
 	   val (funSet,dFunSet,cFunSet,_) = Trs.fdcSetInRules rules
	   val cFunSigMap = 
	       FM.filteri (fn (key,_) => FS.member (cFunSet,key)) sigMap
	   val cConstSigMap = 
	       FM.filteri (fn (_,(tys,ty)) => null tys
					      andalso SS.member (monoSortSet,ty))
			  cFunSigMap

	   val dFunWithArityList = 
	       FM.listItems
		   (FM.mapPartiali (fn (f,(tys,_)) => 
				       if FS.member (dFunSet, f)
				       then SOME (f,length tys)
				       else NONE)
				   sigMap)

	   fun isCConstOfSpecifiedType (Var _,_) = false
	     | isCConstOfSpecifiedType (Fun (c,ts,ty1), ty2) = 
	       (null ts) 
	       andalso S.equal (ty1,ty2)
	       andalso FM.inDomain (cConstSigMap, c)

	   fun reflectiveVariable (Var (x,ty)) = SOME (Var (x,ty))
	     | reflectiveVariable (Fun (f,ts,_)) = 
	       case FM.find (raMap,f) of
		   SOME j => reflectiveVariable (L.nth (ts,j-1))
		 | NONE => NONE

	   fun isContextEliminating map (lxp,Var _) = false
	     | isContextEliminating map (lxp, r as Fun (g,ss,ty)) =
	       Term.equal (lxp,r)
	       orelse
	       case FM.find (map, g) of 
		   NONE => false
		 | SOME q => isContextEliminating map (lxp,L.nth (ss,q-1))

	   fun posCandidates map = 
	       L.filter 
		   (fn (f,i) 
		       => L.all 
			       (fn (l as Fun (g,ts,ty),r) 
				   => (not (Fun.equal (f,g))) 
				      orelse (isCConstOfSpecifiedType (L.nth (ts,i-1),
								       sortOfTerm (L.nth (ts,i-1)))
					      andalso isCConstOfSpecifiedType (r, sortOfTerm r))
				      orelse 
				      case reflectiveVariable (L.nth (ts,i-1)) of
					  SOME (Var (x,xty)) => 
					  VS.member (linearVarSetInTerm l,x)
					  andalso VS.member (linearVarSetInTerm r,x)
					  andalso 
					  let val lxp = 
						  Fun (f, replaceNth (ts,i-1,Var (x,xty)), ty) 
					  in isContextEliminating map (lxp,r)
					  end
					| NONE => false)
			       rules)
		   (mapAppend
			(fn (f,n) => L.tabulate (n, fn i => (f,i+1)))
			dFunWithArityList)

	   val rdpMap  = FM.unionWith (fn (x,y) => x) (dpMap, raMap)

       in
	  (L.foldr (fn ((f,i),m) => FM.insert(m,f,i))
		  FM.empty
		  (posCandidates upMap),
	   L.foldr (fn ((f,i),m) => FM.insert(m,f,i))
		  FM.empty
		  (posCandidates rdpMap))
       end



   local 
       exception NotMonomorphic
       fun uniquePosition pred x ys =
	   let fun uniquePositionSub x [] i = NONE
		 | uniquePositionSub x (y::ys) i = 
		   if pred (x,y) 
		   then if not (List.exists (fn z => pred (x,z)) ys)
			then SOME i
			else NONE
		   else uniquePositionSub x ys (i+1)
	   in
	       uniquePositionSub x ys 0
	   end
   in
   fun getMonomorphicSortSet (decls:Term.decl list) (trs:Trs.trs) = 
       let val rs = #Rules trs
	   val sortSet = Trs.sortSetInRules rs
	   val funSet = #FSymSet trs
	   val dFunSet = #DSymSet trs
	   val cFunSet = #CSymSet trs

	   val sortedFunSet = L.foldr (fn (x,fm) => FM.insert (fm,#sym x,
							       case (#sort x) of
								   Sort.Base a => ([],Sort.Base a)
								 | Sort.Proc (tys,ty) => (tys,ty)))
				      FM.empty decls


	   val sortedCFunSet = 
	       FM.filteri (fn (key,_) => FS.member (cFunSet,key)) sortedFunSet
	   val sortedConstList = 
	       FM.listItemsi
		   (FM.filteri (fn (_,(tys,_)) => null tys) sortedFunSet)

	   val sortedNonConstCFunList = 
	       FM.listItemsi
		   (FM.filteri (fn (_,(tys,_)) => not (null tys)) sortedCFunSet)
	   fun hasDuplication [] = false
	     | hasDuplication (x::xs) = 
	       (List.exists (fn y => S.equal (x,y)) xs)
	       orelse (hasDuplication xs)
	   fun hasOneOccurrence x [] = false
	     | hasOneOccurrence x (y::ys) = 
	       if S.equal (x,y) 
	       then not (List.exists (fn z => S.equal (x,z)) ys)
	       else hasOneOccurrence x ys
	   val mprecs = FM.foldr (fn ((tys,ty), precs) 
				     =>  SPS.addList(precs,
						     List.map (fn ty2 => (ty2,ty)) tys))
					       
				 SPS.empty
				 sortedCFunSet

	   val monoSortSet = 
	       SS.filter (fn ty =>
			     (length (List.filter (fn (_,(_,ty2)) => Sort.equal (ty,ty2))
						  sortedConstList)
			      = 1)
			     andalso (List.all 
					  (fn (_,(tys,ty2)) => not (Sort.equal (ty2,ty))
							       orelse (hasOneOccurrence ty2 tys))
					  sortedNonConstCFunList))
			 sortSet

	   val _ =  print ("Monomorphic sorts: " ^ 
			   (ListUtil.toStringCommaCurly Sort.toString (SortSet.listItems monoSortSet))
			   ^ "\n")

	   val raMap =
	       List.foldr
		   (fn ((c,(tys,ty)), map)
		       => let val pOp = uniquePosition S.equal ty tys
			  in if isSome pOp 
			     then FM.insert (map, c, (valOf pOp) + 1)
			     else raise NotMonomorphic
			  end)
	  	   FM.empty
		   sortedNonConstCFunList
	   val _ = (print " reflective positions:";
		    List.app (fn (f,i) => print (" " ^ (Atom.toString f) ^ "/" ^ (Int.toString i)))
			     (FunMap.listItemsi raMap);
		    print "\n")

	   val dpMap = makeDpMap raMap rs
	   val _ = (print " downward positions:";
		    List.app (fn (f,i) => print (" " ^ (Atom.toString f) ^ "/" ^ (Int.toString i)))
			     (FunMap.listItemsi dpMap);
		    print "\n")

	   val upMap = makeUpMap raMap rs
	   val _ = (print " upward positions:";
		    List.app (fn (f,i) => print (" " ^ (Atom.toString f) ^ "/" ^ (Int.toString i)))
			     (FunMap.listItemsi upMap);
		    print "\n")

	   val (dcpMap,ucpMap) = makeCpMaps raMap upMap dpMap monoSortSet rs
	   val _ = (print " down-contextual positions:";
		    List.app (fn (f,i) => print (" " ^ (Atom.toString f) ^ "/" ^ (Int.toString i)))
			     (FunMap.listItemsi dcpMap);
		    print "\n")
	   val _ = (print " up-contextual positions:";
		    List.app (fn (f,i) => print (" " ^ (Atom.toString f) ^ "/" ^ (Int.toString i)))
			     (FunMap.listItemsi ucpMap);
		    print "\n")
       in
	   (monoSortSet,
	    { RA = raMap,    (* partial map from FunSet to Nat *)
	      DP = dpMap,
 	      UP = upMap,
 	      DCP = dcpMap,
 	      UCP = ucpMap 
	   })
       end
   end


   fun upPosLemmas (argTables:argTables) (trs:Trs.trs) =
       let val upMap = #UP argTables
	   val rules = #Rules trs
	   val sigMap = Trs.signatureInRules rules
 	   val cFunSet = #CSymSet trs
	   fun botOfType ty = 
	       case L.find (fn (key,(tys,ty2)) => 
			       (FS.member (cFunSet,key))
			       andalso (null tys)
			       andalso (S.equal (ty,ty2)))
			   (FM.listItemsi sigMap) of
		   SOME (key,(_,ty3)) => SOME (Fun (key,[],ty3))
		 | NONE => NONE
	   fun constLem (l as Fun (g,ts,ty),p,q) = 
	       case botOfType ty of
		   SOME bot => let val null = Atom.atom ""
				   val m = maxVarIndexInTerm l
				   val xVar = Var ((Atom.atom "x",m+1),ty)
			       in
				   SOME (Fun (g,replaceNth(replaceNth (ts,p-1,xVar),q-1,bot), ty),
					 xVar)
			       end
		 | NONE => NONE
	   fun makeLem (f,p) rs = 
	       case (L.mapPartial 
			 (fn (l as Fun (g,ts,ty),r) 
			     => if (Fun.equal (f,g)) 
				   andalso (Term.isVar r)
				then case (L.find (fn (u,_) => Term.equal (u,r))
						  (LP.zip (ts, L.tabulate (length ts, fn j => j+1))))
				      of SOME (u,q) => SOME (l,p,q)
				       | NONE => NONE
				else NONE)
			 rs) of
		   [(l,p,q)] => constLem (l,p,q)
		 |  _ => NONE
       in
	   L.mapPartial (fn (f,i) => makeLem (f,i) rules)
			(FM.listItemsi upMap)
       end


   (* make  [<p, t/p, top(t,p)> | p \in TP(t) ] *)
   fun topSeq (argTables:argTables) (t as Var _) = [([],t,t)]
     | topSeq (argTables:argTables) (t as Fun (f,ts,ty)) = 
       case FM.find (#RA argTables,f) of
	   SOME i => ([],t,t)::
		     (map (fn (is,s,u) => (i::is,
					   s,
					   Fun (f, replaceNth (ts,i-1,u), ty)))
			  (topSeq argTables (L.nth (ts,i-1))))
	 | NONE => 
	   case FM.find (#UP argTables,f) of
	       SOME i => ([],t,t)::
			 (map (fn (is,s,u) => (i::is,
					       s,
					       u))
			      (topSeq argTables (L.nth (ts,i-1))))
	     | NONE => 
	       case FM.find (#DCP argTables,f) of
		   SOME i => ([],t,t)::
			     (map (fn (is,s,u) => (i::is,
						   s,
						   Fun (f, replaceNth (ts,i-1,u), ty)))
				  (botSeq argTables (L.nth (ts,i-1))))
		 | NONE => 
		   case FM.find (#UCP argTables,f) of
		       SOME i => ([],t,t)::
				 (map (fn (is,s,u) => (i::is,
						       s,
						       Fun (f, replaceNth (ts,i-1,u), ty)))
				      (topSeq argTables (L.nth (ts,i-1))))
		     | NONE => [([],t,t)]

   (* make  [<p, t/p, bot(t,p)> | p \in BP(t) ] *)

   and botSeq (argTables:argTables) (t as Var _) = [([],t,t)]
     | botSeq (argTables:argTables) (t as Fun (f,ts,ty)) = 
       case FM.find (#RA argTables,f) of
	   SOME i => ([],t,t)::
		     (map (fn (is,s,u) => (i::is,
					   s,
					   u))
			  (botSeq argTables (L.nth (ts,i-1))))
	 | NONE => 
	   case FM.find (#DP argTables,f) of
	       SOME i => ([],t,t)::
			 (map (fn (is,s,u) => (i::is,
					       s,
					       u))
			      (botSeq argTables (L.nth (ts,i-1))))
	     | NONE => 
	       case FM.find (#DCP argTables,f) of
		   SOME i => ([],t,t)::
			     (map (fn (is,s,u) => (i::is,
						   s,
						   Fun (f, replaceNth (ts,i-1,u), ty)))
				  (topSeq argTables (L.nth (ts,i-1))))
		 | NONE => 
		   case FM.find (#UCP argTables,f) of
		       SOME i => ([],t,t)::
				 (map (fn (is,s,u) => (i::is,
						       s,
						       Fun (f, replaceNth (ts,i-1,u), ty)))
				      (botSeq argTables (L.nth (ts,i-1))))
		     | NONE => [([],t,t)]


   fun generalizeByTop argTables (l,r) =
       let
(* 	   val _ = print ("generalizeByTop "  ^ (IOMstrs.prTerm l) ^ "=" ^ (IOMstrs.prTerm r) ^ "\n") *)
	   val tpseqL = topSeq argTables l
(* 	   val _ = print "generalizeByTop (i)\n" *)
	   val tpseqR = topSeq argTables r
(* 	   val _ = print "generalizeByTop (ii)\n" *)
	   fun generalizeSub ([],_) = NONE
	     | generalizeSub (p::ps, []) = generalizeSub (ps, tpseqR)
	     | generalizeSub ((p,lp,toplp)::ps, (q,rq,toprq)::qs) = 
	       ((* print ((IOMstrs.prTerm lp) ^ "*" ^ (IOMstrs.prTerm rq) ^ "\n"); *)
(* 		print ((IOMstrs.prTerm toplp) ^ "**" ^ (IOMstrs.prTerm toprq) ^ "\n"); *)
		if Term.equal (lp, rq)
		   andalso Term.equal (toplp,toprq)
		then SOME (p, q, sortOfTerm lp)
		else generalizeSub ((p,lp,toplp)::ps, qs))
       in 
	   case generalizeSub (tpseqL, tpseqR) of
	       SOME (ps,qs,ty) => let val null = Atom.atom ""
				      val m = maxVarIndexInTerms [l,r]
				      val xVar = Var ((Atom.atom "x",m+1),ty)
				      val l' = valOf (replaceSubterm l ps xVar)
				      val r' = valOf (replaceSubterm r qs xVar)
				  in SOME (l',r')
				  end
	     | NONE => NONE
       end

   fun generalizeByBot argTables (l,r) =
       let
(* 	   val _ = print "generalizeByBot\n" *)
	   val btseqL = botSeq argTables l
	   val btseqR = botSeq argTables r
	   fun generalizeSub ([],_) = NONE
	     | generalizeSub (p::ps, []) = generalizeSub (ps, btseqR)
	     | generalizeSub ((p,lp,botlp)::ps, (q,rq,botrq)::qs) = 
(* print ("<" ^ (Term.toString lp) ^ ", "); *)
(* print ((Term.toString rq) ^ ">\n"); *)
	       if Term.equal (lp, rq)
		   andalso Term.equal (botlp,botrq)
		then SOME (p, q, sortOfTerm lp)
		else generalizeSub ((p,lp,botlp)::ps, qs)
       in 
	   case generalizeSub (btseqL, btseqR) of
	       SOME (ps,qs,ty) => let val null = Atom.atom ""
				      val m = maxVarIndexInTerms [l,r]
				      val xVar = Var ((Atom.atom "x",m+1),ty)
				      val l' = valOf (replaceSubterm l ps xVar)
				      val r' = valOf (replaceSubterm r qs xVar)
				  in SOME (l',r')
				  end
	     | NONE => NONE
       end


   fun soundGeneralization (argTables:argTables) (l,r) =
       let val null = Atom.atom ""
	   fun properGen (l',r') = 
	       case Subst.match (Fun (null,[l,r],Sort.null)) (Fun (null,[l',r'],Sort.null)) of
		   SOME _ => false
		 | NONE => true
       in
	   case generalizeByTop argTables (l,r) of 
	       SOME (l',r') => (case generalizeByBot argTables (l',r') of 
				    SOME (l'',r'') => if properGen (l'',r'') 
						      then SOME (l'',r'') 
						      else NONE
				  | NONE => if properGen (l',r') 
					    then SOME (l',r') 
					    else NONE)
	     | NONE => (case generalizeByBot argTables (l,r) of 
			   SOME (l',r') => if properGen (l',r') 
					   then SOME (l',r') 
					   else NONE
			 | NONE => NONE)
       end

   (* make  [<p, t/p, fn x => top(t[x]_p,p)> | p \in TP(t) ] *)
   fun topSeq2 (argTables:argTables) (t as Var _) = [([],t,fn x=> x)]
     | topSeq2 (argTables:argTables) (t as Fun (f,ts,ty)) = 
       case FM.find (#RA argTables,f) of
	   SOME i => ([],t,fn x => x)::
		     (map (fn (is,s,u) => (i::is,
					   s,
					   fn x => Fun (f, replaceNth (ts,i-1,u x), ty)))
			  (topSeq2 argTables (L.nth (ts,i-1))))
	 | NONE => 
	   case FM.find (#UP argTables,f) of
	       SOME i => ([],t,fn x => x)::
			 (map (fn (is,s,u) => (i::is,
					       s,
					       u))
			      (topSeq2 argTables (L.nth (ts,i-1))))
	     | NONE => 
	       case FM.find (#DCP argTables,f) of
		   SOME i => ([],t,fn x => x)::
			     (map (fn (is,s,u) => (i::is,
						   s,
						   fn x => Fun (f, replaceNth (ts,i-1,u x), ty)))
				  (botSeq2 argTables (L.nth (ts,i-1))))
		 | NONE => 
		   case FM.find (#UCP argTables,f) of
		       SOME i => ([],t,fn x => x)::
				 (map (fn (is,s,u) => (i::is,
						       s,
						       fn x => Fun (f, replaceNth (ts,i-1,u x), ty)))
				      (topSeq2 argTables (L.nth (ts,i-1))))
		     | NONE => [([],t,fn x => x)]

   (* make  [<p, t/p, bot(t,p)> | p \in BP(t) ] *)

   and botSeq2 (argTables:argTables) (t as Var _) = [([],t,fn x => x)]
     | botSeq2 (argTables:argTables) (t as Fun (f,ts,ty)) = 
       case FM.find (#RA argTables,f) of
	   SOME i => ([],t,fn x => x) ::
		     (map (fn (is,s,u) => (i::is,
					   s,
					   u))
			  (botSeq2 argTables (L.nth (ts,i-1))))
	 | NONE => 
	   case FM.find (#DP argTables,f) of
	       SOME i => ([],t,fn x => x)::
			 (map (fn (is,s,u) => (i::is,
					       s,
					       u))
			      (botSeq2 argTables (L.nth (ts,i-1))))
	     | NONE => 
	       case FM.find (#DCP argTables,f) of
		   SOME i => ([],t,fn x=>x)::
			     (map (fn (is,s,u) => (i::is,
						   s,
						   fn x => Fun (f, replaceNth (ts,i-1,u x), ty)))
				  (topSeq2 argTables (L.nth (ts,i-1))))
		 | NONE => 
		   case FM.find (#UCP argTables,f) of
		       SOME i => ([],t,fn x => x)::
				 (map (fn (is,s,u) => (i::is,
						       s,
						       fn x => Fun (f, replaceNth (ts,i-1,u x), ty)))
				      (botSeq2 argTables (L.nth (ts,i-1))))
		     | NONE => [([],t,fn x =>x)]


  (* Simply-C $B$rMxMQ$7$?3HD%HG(B; top $B$b(B top(l[x])!$B!$(Btop(r[x])! $B$GHf3S(B*)
   fun generalizeByTop2 argTables isEqual isSame (l,r) =
       let
	   val null = Atom.atom ""
	   val m = maxVarIndexInTerms [l,r]
	   val tpseqL = topSeq2 argTables l
	   val tpseqR = topSeq2 argTables r
	   fun generalizeSub ([],_) = NONE
	     | generalizeSub (p::ps, []) = generalizeSub (ps, tpseqR)
	     | generalizeSub ((p,lp,toplxp)::ps, (q,rq,toprxq)::qs) = 
		if Term.equal (lp, rq)
		   andalso isSame (toplxp (Var ((Atom.atom "x",m+1), Sort.null)),
				   toprxq (Var ((Atom.atom "x",m+1), Sort.null)))
		then (print ("SG (a)" ^ (Term.toString lp) ^ ", " ^ (Term.toString rq) ^ " at " );
		      print ((Term.prPosition p) ^ ", " ^ (Term.prPosition q) ^ "\n" );
		      SOME (p, q, sortOfTerm lp))
		else if Term.equal (lp, toplxp lp)
			andalso Term.equal (rq, toprxq rq)
			andalso isEqual (lp,rq)
		then (print ("SG (b)" ^ (Term.toString lp) ^ ", " ^ (Term.toString rq) ^ " at " );
		      print ((Term.prPosition p) ^ ", " ^ (Term.prPosition q) ^ "\n" );
		      SOME (p, q, sortOfTerm lp))
		else generalizeSub ((p,lp,toplxp)::ps, qs)
       in 
	   case generalizeSub (tpseqL, tpseqR) of
	       SOME (ps,qs,ty) => let val xVar = Var ((Atom.atom "x",m+1),ty)
				      val l' = valOf (replaceSubterm l ps xVar)
				      val r' = valOf (replaceSubterm r qs xVar)
				      val _ = print ((Trs.prEq (l',r')) ^ "\n")
				  in SOME (l',r')
				  end
	     | NONE => NONE
       end

   fun generalizeByBot2 argTables isEqual isSame (l,r) =
       let
	   val null = Atom.atom ""
	   val m = maxVarIndexInTerms [l,r]
	   val btseqL = botSeq2 argTables l
	   val btseqR = botSeq2 argTables r
	   fun generalizeSub ([],_) = NONE
	     | generalizeSub (p::ps, []) = generalizeSub (ps, btseqR)
	     | generalizeSub ((p,lp,botlxp)::ps, (q,rq,botrxq)::qs) = 
		if Term.equal (lp, rq)
		   andalso isSame (botlxp (Var ((Atom.atom "x",m+1), Sort.null)),
				   botrxq (Var ((Atom.atom "x",m+1), Sort.null)))
		then SOME (p, q, sortOfTerm lp)
		else if Term.equal (lp, botlxp lp)
			andalso Term.equal (rq,botrxq rq)
			andalso isEqual (lp,rq)
		then SOME (p, q, sortOfTerm lp)
		else generalizeSub ((p,lp,botlxp)::ps, qs)
       in 
	   case generalizeSub (btseqL, btseqR) of
	       SOME (ps,qs,ty) => let val xVar = Var ((Atom.atom "x",m+1),ty)
				      val l' = valOf (replaceSubterm l ps xVar)
				      val r' = valOf (replaceSubterm r qs xVar)
				  in SOME (l',r')
				  end
	     | NONE => NONE
       end


   fun soundGeneralization2 (argTables:argTables) isEqual isSame (l,r) =
       let val null = Atom.atom ""
	   fun properGen (l',r') = 
	       case Subst.match (Fun (null,[l,r],Sort.null)) (Fun (null,[l',r'],Sort.null)) of
		   SOME _ => false
		 | NONE => true
       in
	   case generalizeByTop2 argTables isEqual isSame (l,r) of 
	       SOME (l',r') => (case generalizeByBot2 argTables isEqual isSame (l',r') of 
				    SOME (l'',r'') => if properGen (l'',r'') 
						      then SOME (l'',r'') 
						      else NONE
				  | NONE => if properGen (l',r') 
					    then SOME (l',r') 
					    else NONE)
	     | NONE => (case generalizeByBot2 argTables isEqual isSame (l,r) of 
			   SOME (l',r') => if properGen (l',r') 
					   then SOME (l',r') 
					   else NONE
			 | NONE => NONE)
       end



   exception NoBottomFound

   fun botOfSort sigMap (trs:Trs.trs) ty =
       let val cFunSet = #CSymSet trs
	   val map = FM.filteri (fn (f,(xs,x)) => null xs 
						  andalso S.equal (x,ty)
						  andalso FS.member (cFunSet, f))
				sigMap
       in
	   case (FM.firsti map)
		of SOME (c,_) => Fun (c,[],ty)
		 | NONE => raise NoBottomFound
       end
 	   


   (* make  [<p,top(t,p),ntp(t,p)> | p \in TP(t) ] *)
   fun divByTp sigMap (trs:Trs.trs)  (argTables:argTables) (t as Var (x,ty)) = 
       [([],t,botOfSort sigMap trs ty)]
     | divByTp sigMap (trs:Trs.trs) (argTables:argTables) (t as Fun (f,ts,ty)) = 
       case FM.find (#RA argTables,f) of
	   SOME i => ([],t,botOfSort sigMap trs ty)::
		     (map (fn (is,u,s) => (i::is,
					   Fun (f, replaceNth (ts,i-1,u), ty),
					   s))
			  (divByTp sigMap trs argTables (L.nth (ts,i-1))))
	 | NONE => 
	   case FM.find (#UP argTables,f) of
	       SOME i => ([],t,botOfSort sigMap trs ty)::
			 (map (fn (is,u,s) => (i::is,
					       u,
					       Fun (f, replaceNth (ts,i-1,s), ty)))
			      (divByTp sigMap trs argTables (L.nth (ts,i-1))))
	     | NONE => 
	       case FM.find (#DCP argTables,f) of
		   SOME i => ([],t,botOfSort sigMap trs ty)::
			     (map (fn (is,u,s) => (i::is,
						   Fun (f, replaceNth (ts,i-1,u), ty),
						   Fun (f, replaceNth (ts,i-1,s), ty)))
				  (divByBp sigMap trs argTables (L.nth (ts,i-1))))
		 | NONE => 
		   case FM.find (#UCP argTables,f) of
		       SOME i => ([],t,botOfSort sigMap trs ty)::
				 (map (fn (is,u,s) => (i::is,
						       Fun (f, replaceNth (ts,i-1,u), ty),
						       Fun (f, replaceNth (ts,i-1,s), ty)))
				      (divByTp sigMap trs argTables (L.nth (ts,i-1))))
		     | NONE => [([],t,botOfSort sigMap trs ty)]

   (* make  [<p, t/p, bot(t,p)> | p \in BP(t) ] *)

   and divByBp sigMap (trs:Trs.trs)  (argTables:argTables) (t as Var (x,ty)) = 
       [([],t,botOfSort sigMap trs ty)]
     | divByBp sigMap (trs:Trs.trs) (argTables:argTables) (t as Fun (f,ts,ty)) = 
       case FM.find (#RA argTables,f) of
	   SOME i => ([],t,botOfSort sigMap trs ty)::
		     (map (fn (is,u,s) => (i::is,
					   u,
					   Fun (f, replaceNth (ts,i-1,s), ty)))
			  (divByBp sigMap trs argTables (L.nth (ts,i-1))))
	 | NONE => 
	   case FM.find (#DP argTables,f) of
	       SOME i => ([],t,botOfSort sigMap trs ty)::
			 (map (fn (is,u,s) => (i::is,
					       u,
					       Fun (f, replaceNth (ts,i-1,s), ty)))
			      (divByBp sigMap trs argTables (L.nth (ts,i-1))))
	     | NONE => 
	       case FM.find (#DCP argTables,f) of
		   SOME i => ([],t,botOfSort sigMap trs ty)::
			     (map (fn (is,u,s) => (i::is,
						   Fun (f, replaceNth (ts,i-1,u), ty),
						   Fun (f, replaceNth (ts,i-1,s), ty)))
				  (divByTp sigMap trs argTables (L.nth (ts,i-1))))
		 | NONE => 
		   case FM.find (#UCP argTables,f) of
		       SOME i => ([],t,botOfSort sigMap trs ty)::
				 (map (fn (is,u,s) => (i::is,
						       Fun (f, replaceNth (ts,i-1,u), ty),
						       Fun (f, replaceNth (ts,i-1,s), ty)))
				      (divByBp sigMap trs argTables (L.nth (ts,i-1))))
		     | NONE => [([],t,botOfSort sigMap trs ty)]


   fun termPartition sigMap (trs:Trs.trs)  (argTables:argTables) t =
       let val bot =  botOfSort sigMap trs (Term.sortOfTerm t)
	   val cand1 = divByTp sigMap (trs:Trs.trs)  (argTables:argTables) t
	   val cand2 = L.map (fn (is,x,y)=>(is,y,x)) 
			     (divByBp sigMap (trs:Trs.trs)  (argTables:argTables) t)
	   val cand = let val f = Rewrite.linf (#Rules trs)
		      in
			  L.map (fn (i,s,t) => (i,f s, f t)) (cand1 @ cand2)
		      end
       in
	   L.filter (fn (_,x,y) => not (Term.equal (bot,x))
			       andalso not (Term.equal (bot,y))
			       andalso not (Term.equal (t,x))
(* 			       andalso not (Term.isVar x)  (\* $BJQ?t$O=|30$7$J$$$H(BSG$B$GL58B%k!<%W(B???*\) *)
(* 			       andalso not (Term.isVar y)   (\* $BJQ?t$O=|30$7$J$$$H(BSG$B$GL58B%k!<%W(B???*\) *)
			       andalso not (Term.equal (t,y))) cand
       end


   end (* of local *)


   end; (* of structure *)


