(******************************************************************************
 * file: rwtools/sprint/test_rip.sml
 * description: tests of on rewriting induction
 * author: AOTO Takahito
 * 
 * Copyright (c) 2008, Toyama&Aoto Laboratory, Tohoku University
 ******************************************************************************)

signature TEST_RIP =  
  sig
      val riCheck: Sort.ord_key list
		   -> Term.decl list
		   -> (Term.term * Term.term) list
                   -> (Term.term * Term.term) list
                   -> (int * Atom.atom * Order.rpo_status * int list) list
(*                   -> (Fun.ord_key * Poly.poly) list  *)
                   -> bool
      val riTest: int -> bool
  end;

structure TestRip : TEST_RIP =
struct

	local
		structure ILM = IntListMap2
	in

    val minisatPath = ref "../../work/tools/bin/minisat"
    val tmpDir = ref "."

    fun time f arg=
	let
	    val timer = Timer.startCPUTimer ()
	    val start = TimeUtil.checkTime timer 
	    val result = f arg
	    val stop = TimeUtil.checkTime timer 
	    val _ = print (" ("  
			   ^ (TimeUtil.reportTime (start,stop)) 
			   ^ " msec.)\n")	
	in
	    result
	end

    fun riCheck sorts tupdecl rules conj prec =

	let
		val opt = {useE2 = false,
				   useQB = false,
				   useDC = true,
				   useSDC = false,
				   useSG = false,
				   useMG = false,
				   useESG =false,
				   useDP = false,
				   useTP = false,
				   useRS = false,
				   useLS = false
				   }
	    val _ = Ind.runDebug := true (* $B%G%P%C%0=PNO$9$k$+(B *)
	in
	    Riprover.riprover (!minisatPath) (!tmpDir) opt 
			      (tupdecl,rules,conj,prec)
	end

    val p = ILM.empty;

    fun riTest 1 = 
		time riCheck 
	    (IOMstrs.rdSorts ["Nat"])
	    (IOMstrs.rdDecls ["plus : Nat*Nat=>Nat",
						  "s    : Nat=>Nat",
						  "0    : Nat"])
			(IOMstrs.rdRules
			 ["plus(0,?y:Nat) -> ?y:Nat",
			  "plus(s(?x:Nat),?y:Nat) -> s(plus(?x:Nat,?y:Nat))"])
			(IOMstrs.rdEqs
			 ["plus(plus(?x:Nat,?y:Nat),?z:Nat) = plus(?x:Nat,plus(?y:Nat,?z:Nat))"])
		[(3,Atom.atom "plus", Order.MUL,[]),
			 (2,Atom.atom "s", Order.LEX,[1]),
			 (1,Atom.atom "0", Order.LEX,[])]
(**          [(Atom.atom "plus",ILM.insert (ILM.insert (ILM.insert (p,[],1),[1],2),[2],1)),
          (Atom.atom "s",ILM.insert (ILM.insert (p,[],1),[1],1)),
          (Atom.atom "0",ILM.insert (p,[],1))] **)
             

     
	  | riTest 2 = 
			time riCheck 
			(IOMstrs.rdSorts ["Nat"])
			(IOMstrs.rdDecls ["plus : Nat*Nat=>Nat",
							  "s : Nat=>Nat",
								  "0 : Nat"])
			(IOMstrs.rdRules
			 ["plus(0,?y:Nat) -> ?y:Nat",
			  "plus(s(?x:Nat),?y:Nat) -> s(plus(?x:Nat,?y:Nat))"])
		   (IOMstrs.rdEqs
			 ["plus(plus(?x:Nat,?y:Nat),?z:Nat) = plus(?x:Nat,plus(?y:Nat,?z:Nat))"])
			 [(3,Atom.atom "plus", Order.LEX,[]),
			  (2,Atom.atom "s", Order.LEX,[1]),
			  (1,Atom.atom "0", Order.LEX,[])]
(**            [(Atom.atom "plus",ILM.insert (ILM.insert (ILM.insert (p,[],1),[1],2),[2],1)),
             (Atom.atom "s",ILM.insert (ILM.insert (p,[],1),[1],1)),
             (Atom.atom "0",ILM.insert (p,[],1))] **)

	  | riTest 3 = 
			time riCheck 
			(IOMstrs.rdSorts ["Nat", "List"])
			(IOMstrs.rdDecls ["rev : List=>List",
							  "app : List*List=>List",
							  "cons : Nat*List=>List",
							  "s : Nat=>Nat",
							  "0 : Nat",
							  "nil : List"])
				(IOMstrs.rdRules
				 ["rev(nil) -> nil",
				  "rev(cons(?x:Nat,?xs:List)) -> app(rev(?xs:List),cons(?x:Nat,nil))",
				  "app(nil,?ys:List) -> ?ys:List",
				  "app(cons(?x:Nat,?xs:List),?ys:List) -> cons(?x:Nat,app(?xs:List,?ys:List))"])
				(IOMstrs.rdEqs
				 ["rev(rev(?xs:List)) = ?xs:List"])
				[(6,Atom.atom "rev", Order.LEX,[1]),
				 (5,Atom.atom "app", Order.LEX,[1,2]),
				 (4,Atom.atom "cons", Order.LEX,[1,2]),
				 (3,Atom.atom "s", Order.LEX,[1]),
				 (2,Atom.atom "nil", Order.LEX,[]),
				 (1,Atom.atom "0", Order.LEX,[])]  
(*
				[(Atom.atom "rev", ILM.insert 
                        (ILM.insert (p,[],1),[1,1],1)),
				 (Atom.atom "app", ILM.insert 
                        (ILM.insert 
                        (ILM.insert (p,[],1),[1],2),[2],1)),
				 (Atom.atom "cons",ILM.insert
                        (ILM.insert 
                        (ILM.insert (p,[],2 ),[1],1),[2],1)),
				 (Atom.atom "s",   ILM.insert 
                        (ILM.insert (p,[],1),[1],1)),
				 (Atom.atom "nil", ILM.insert (p,[],0)),
				 (Atom.atom "0",   ILM.insert (p,[],1))] 
*)
      | riTest _ = false

	end

end



