(******************************************************************************
 * file: rwtools/stsrs/stspec.lex
 * description: Lexer Specification for STSRS
 * author: AOTO Takahito
 * 
 * Copyright (c) 2008, Toyama&Aoto Laboratory, Tohoku University
 ******************************************************************************)

structure Tokens = Tokens;
type pos = int;
type svalue = Tokens.svalue;
type ('a,'b) token = ('a,'b) Tokens.token;
type lexresult = (svalue, pos) token;

val pos = ref 1;
val eof = fn () => (pos:=1; Tokens.EOF(!pos,!pos));

%%
%header (functor StspecLexFun (structure Tokens: Stspec_TOKENS));
%s COMMENT;
%%
\n+	     => (pos:=(!pos)+1; lex()  (* Ԥ *));
[\ \t]+	     => (lex() (* ϥå *));
\;    	     => (Tokens.SEMICOLON(!pos,!pos));
\:    	     => (Tokens.COLON(!pos,!pos));
\=    	     => (Tokens.EQ(!pos,!pos));
\=>    	     => (Tokens.THICKARROW(!pos,!pos));
\->    	     => (Tokens.THINARROW(!pos,!pos));
\*    	     => (Tokens.TIMES(!pos,!pos));
\(    	     => (Tokens.LPAREN(!pos,!pos));
\,    	     => (Tokens.COM(!pos,!pos));
\)    	     => (Tokens.RPAREN(!pos,!pos));
\?    	     => (Tokens.QUESTION(!pos,!pos));
\_    	     => (Tokens.BAR(!pos,!pos));
SORTS        => (Tokens.SORTS(!pos,!pos));
FUNCTIONS    => (Tokens.FUNCTIONS(!pos,!pos));
RULES        => (Tokens.RULES(!pos,!pos));
[a-zA-Z0-9]+ => (Tokens.ID(yytext,!pos,!pos));
.      	     => (print ("Warning: ignoring bad character: " 
    	    	    	    	  ^ yytext ^ "at line(" ^ (Int.toString (!pos))
    	    	    	    	  ^ ")\n"); lex());
