(******************************************************************************
 * file: rwtools/stsrs/stspec.yacc
 * description: Parser Specification for STSRS
 * author: AOTO Takahito
 * 
 * Copyright (c) 2008, Toyama&Aoto Laboratory, Tohoku University
 ******************************************************************************)

type yaccexpr = Atom.atom list * (Atom.atom * Sort.sort) list * (Term.term * Term.term ) list

(* 桼 *)
%%
%name		Stspec
%term           ID of string | 
                SEMICOLON | COLON | THICKARROW | THINARROW |
                TIMES | LPAREN | RPAREN | COM | EQ | QUESTION | BAR | 
    		SORTS | FUNCTIONS | RULES | EOF 
%nonterm	START of yaccexpr | 
                SPART of Atom.atom list | 
                FPART of (Atom.atom * Sort.sort) list | 
                RPART of (Term.term * Term.term) list | 
                SNAMELIST of Atom.atom list | 
                SPECLIST of (Atom.atom * Sort.sort) list  | 
                SPEC of Atom.atom * Sort.sort | 
                PROCTYPE of Sort.sort |
                PROCTYPE' of Sort.sort |
                SORTLIST of Sort.sort list |
                RULELIST of (Term.term * Term.term) list | 
                RULE of Term.term * Term.term | 
                TERM of Term.term | 
                TERMLIST of Term.term list| 
	        VAR of (Atom.atom * int)
%pos	   	int
%eop		EOF
%noshift	EOF


%%
START:      SPART FPART RPART              ((SPART,FPART,RPART))
SPART:      SORTS SNAMELIST                (SNAMELIST)
FPART:      FUNCTIONS SPECLIST             (SPECLIST)
RPART:      RULES RULELIST                 (RULELIST)

SNAMELIST:   ID SEMICOLON SNAMELIST        (Atom.atom ID::SNAMELIST1)
            | ID		           ([Atom.atom ID])
            |                              ([])

SPECLIST:   SPEC SEMICOLON SPECLIST        (SPEC::SPECLIST1)
            | SPEC		           ([SPEC])
            |                              ([])

SPEC:       ID COLON PROCTYPE              ((Atom.atom ID,PROCTYPE))

PROCTYPE:   SORTLIST THICKARROW PROCTYPE'  (Sort.Proc (SORTLIST, PROCTYPE'))
            | ID                           (Sort.Base (Atom.atom ID))

PROCTYPE':   LPAREN SORTLIST THICKARROW PROCTYPE' RPAREN  (Sort.Proc (SORTLIST, PROCTYPE'))
            | ID                           (Sort.Base (Atom.atom ID))

SORTLIST:   PROCTYPE' TIMES SORTLIST        (PROCTYPE'::SORTLIST)
            | PROCTYPE'                     ([PROCTYPE'])

RULELIST:   RULE SEMICOLON RULELIST        (RULE::RULELIST1)
            | RULE 			   ([RULE])
            |                              ([])

RULE:       TERM THINARROW TERM            ((TERM1,TERM2))

TERM:       LPAREN TERMLIST RPAREN      (Term.Fun (Atom.atom "@",TERMLIST,Sort.null))
            | ID                           (Term.Fun (Atom.atom ID,[],Sort.null))
            | VAR PROCTYPE           (Term.Var (VAR,PROCTYPE))

TERMLIST:   TERM TERMLIST 	           (TERM::TERMLIST1)
            | TERM 			   ([TERM])

VAR:        QUESTION ID COLON                 (Atom.atom ID1,0)
            | QUESTION ID BAR ID COLON                 (Atom.atom ID1,valOf (Int.fromString ID2))

